<?php

namespace WPEP\API;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class Authentication {

  public static function express( $request_data ) {
    $_maybe = self::_maybe_standard_response( $request_data );

    if( $_maybe !== false )
      return $_maybe;

    $auth_information = $request_data['auth_information'];

    if( $auth_information['email_address'] == '' || !is_email( $auth_information['email_address'] ) )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-auth-notification-login-details-required' ),
        'notification_class'  => 'wpep-notification'
      ], $auth_information, 'express' );

    $site_url = get_site_url();

    $existent_user = get_user_by('email', $auth_information['email_address'] );

    $email_title = sprintf(
      ( $existent_user === false ? wpep_get_setting_translated( 'email-express-authentication-register-title' ) : wpep_get_setting_translated( 'email-express-authentication-login-title' ) ),
      ( is_multisite() ? get_network()->site_name : wp_specialchars_decode(get_option('blogname'), ENT_QUOTES) )
    );

    $email_content = ( $existent_user === false ? wpep_get_setting_translated( 'email-express-authentication-register-content' ) : wpep_get_setting_translated( 'email-express-authentication-login-content' ) );
    $email_content = str_replace( '[site_url]', $site_url, $email_content );
    $email_content = str_replace( '[email_address]', $auth_information['email_address'], $email_content );

    $redirect = ( isset( $request_data[ 'current_url' ] ) ? $request_data[ 'current_url' ] : $site_url );

    if( $auth_information[ 'success_response_type' ] == 'redirect' && isset( $auth_information[ 'success_response_redirect' ] ) )
      $redirect = $auth_information[ 'success_response_redirect' ];

    if( $existent_user === false ) {
      $password = wpep_generate_random_token( 16 );

      $link = add_query_arg( 'wpep-express-authentication', 'register', $site_url );
      $link = add_query_arg( 'email', urlencode( $auth_information['email_address'] ), $link );
      $link = add_query_arg( 'token', md5( AUTH_SALT . $auth_information['email_address'] ), $link );
      $link = add_query_arg( 'password', $password, $link );
      $link = add_query_arg( 'redirect', urlencode( $redirect ), $link );

      $email_content = str_replace( '[register_link]', $link, $email_content );
      $email_content = str_replace( '[password]', $password, $email_content );
    } else {
      $key = wpep_generate_random_token( 24 );

      $link = add_query_arg( 'wpep-express-authentication', 'login', $site_url );
      $link = add_query_arg( 'email', urlencode( $auth_information['email_address'] ), $link );
      $link = add_query_arg( 'token', md5( AUTH_SALT . $auth_information['email_address'] ), $link );
      $link = add_query_arg( 'key', $key, $link );
      $link = add_query_arg( 'redirect', urlencode( $redirect ), $link );

      $email_content = str_replace( '[login_link]', $link, $email_content );

      update_user_meta( $existent_user->ID, 'wpep_express_authentication_key', $key );
      update_user_meta( $existent_user->ID, 'wpep_express_authentication_timestamp', time() );

      WPEP_Entity_ActionTracking::instance()->track_event( $existent_user->ID, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_EXPRESS_REQUEST, '' );
    }

    \wpep_controller()->email->send( $auth_information['email_address'], $email_title, $email_content );

    return self::_filter_response_return( [
      'status'              => 'ok',
      'notification'        => wpep_get_setting_translated( 'user-auth-notification-check-email' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'check_email'         => 1
    ], $auth_information, 'express' );
  }

  public static function login( $request_data ) {
    $_maybe = self::_maybe_standard_response( $request_data );

    if( $_maybe !== false )
      return $_maybe;

    $auth_information = $request_data['auth_information'];

    if( $auth_information['login_identifier'] == '' || $auth_information['password'] == '' )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-auth-notification-login-details-required' ),
        'notification_class'  => 'wpep-notification'
      ], $auth_information, 'login' );

    remove_action( "wp_login", [ \wpep_controller()->userEngagement, '_on_user_login' ] );

    $user = wp_signon( [
      'user_login'    => $auth_information['login_identifier'],
      'user_password' => $auth_information['password'],
      'remember'      => true
    ], is_ssl() );

    if ( is_wp_error( $user ) )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => ( isset( $user->errors['incorrect_password'] )
          ? ( wpep_get_setting_translated( "user-auth-notification-password-incorrect" ) . ' <span class="reset-password-text" data-field-type-switch="reset_password">' . wpep_get_setting_translated( 'user-auth-open-tab-reset-password' ) . '</span>' )
          : $user->get_error_message()
        ),
        'notification_class'  => 'wpep-notification wpep-notification-warning'
      ], $auth_information, 'login' );

    wp_set_current_user( $user->ID );

    return self::_filter_response_return( [
      'status'              => 'ok',
      'notification'        => wpep_get_setting_translated( 'user-auth-notification-logged-in' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'refresh'             => 1,
      'refresh_delay'       => 1000
    ], $auth_information, 'login' );
  }

  public static function register( $request_data ) {
    $_maybe = self::_maybe_standard_response( $request_data );

    if( $_maybe !== false )
      return $_maybe;

    $allow_registration = apply_filters( 'wpep_auth_allow_registration', ( wpep_get_setting_flag( 'user-auth-enable-registration' ) || wpep_get_setting_flag( 'enable-post-member-magnet-option' ) ) );

    if( !$allow_registration )
      return false;

    $auth_information = $request_data['auth_information'];

    if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) && ( !isset( $auth_information['service_agreement'] ) || intval( $auth_information['service_agreement'] ) != 1 ) )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => wpep_get_setting( 'user-service-agreement-error' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ], $auth_information, 'register' );

    if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) && ( !isset( $auth_information['privacy_agreement'] ) || intval( $auth_information['privacy_agreement'] ) != 1 ) )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => wpep_get_setting( 'user-privacy-agreement-error' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ], $auth_information, 'register' );


    $user_id = wp_create_user( $auth_information['username'], $auth_information['password'], $auth_information['email_address'] );

    if( is_wp_error( $user_id ) )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => $user_id->get_error_message(),
        'notification_class'  => 'wpep-notification wpep-notification-warning'
      ], $auth_information, 'register' );


    $first_name = ( isset( $auth_information['first_name'] ) ? sanitize_text_field( $auth_information['first_name'] ) : '' );
    $last_name  = ( isset( $auth_information['last_name'] ) ? sanitize_text_field( $auth_information['last_name'] ) : '' );

    if( $first_name !== '' && $last_name !== '' ) {
      wp_update_user( [
        'ID'           => $user_id,
        'first_name'   => $first_name,
        'last_name'    => $last_name,
        'display_name' => ( $first_name == '' && $last_name == '' ? $auth_information['username'] : trim( $first_name . ' ' . $last_name ) )
      ] );
    }

    if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) )
      update_user_meta( $user_id, WPEP_USER_META_AGREEMENT_PRIVACY_DATE, date( "Y-m-d" ) );
    if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) )
      update_user_meta( $user_id, WPEP_USER_META_AGREEMENT_TOS_DATE, date( "Y-m-d" ) );

    $user = get_user_by( 'id', $user_id );

    wp_set_current_user( $user_id );
    wp_set_auth_cookie( $user_id );
    do_action( 'wp_login', $user->user_login, $user );

    if( is_wp_error( $user_id ) )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => $user_id->get_error_message(),
        'notification_class'  => 'wpep-notification wpep-notification-warning'
      ], $auth_information, 'register' );

    return self::_filter_response_return( [
      'status'              => 'ok',
      'notification'        => wpep_get_setting_translated( 'user-auth-notification-account-registered' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'refresh'             => 1,
      'refresh_delay'       => 2000
    ], $auth_information, 'register' );
  }

  public static function reset_password( $request_data ) {
    $_maybe = self::_maybe_standard_response( $request_data );

    if( $_maybe !== false )
      return $_maybe;

    $auth_information = $request_data['auth_information'];

    if( !isset( $auth_information['login_identifier'] ) )
      return false;

    $response = wpep_retrieve_password( $auth_information['login_identifier'] );

    if( is_wp_error( $response ) )
      return self::_filter_response_return( [
        'status'              => 'error',
        'notification'        => $response->get_error_message(),
        'notification_class'  => 'wpep-notification wpep-notification-warning'
      ], $auth_information, 'reset_password' );

    return self::_filter_response_return( [
      'status'              => 'ok',
      'notification'        => wpep_get_setting_translated( 'user-auth-notification-password-reset' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'check_email'         => 1,
    ], $auth_information, 'reset_password' );
  }

  private static function _filter_response_return( $response, $auth_information, $auth_type ) {
    if( !isset( $response['status'] ) || $response['status'] != 'ok' )
      return json_encode( $response );

    $post_id = ( isset( $auth_information[ 'wpep_post_id'] ) ? intval( $auth_information[ 'wpep_post_id'] ) : 0 );
    $lead_segment = ( isset( $auth_information[ 'wpep_lead_segment'] ) ? $auth_information[ 'wpep_lead_segment'] : ( $post_id !== 0 ? 'lead_magnet' : 'registration_form' ) );

    if( $auth_type == 'register' ) {
      WPEP_Entity_ActionTracking::instance()->track_event( $post_id, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_REGISTRATION, $lead_segment );

      if( $post_id !== 0 && wpep_post_has_member_magnet_option( $post_id ) ) {
        $custom_redirect = trim( get_post_meta( $post_id, 'wpep_member_magnet_option_registration_redirect_url', true ) );

        if( $custom_redirect !== '' ) {
          wpep_user_course_started_event( $post_id, get_current_user_id() );

          $response = [
            'status'       => 'ok',
            'redirect'     => $custom_redirect,
            'redirect_top' => 1
          ];
        }
      }
    } elseif( $auth_type === 'reset_password' ) {
      WPEP_Entity_ActionTracking::instance()->track_event( $post_id, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, $lead_segment );
    } elseif( $auth_type === 'login' ) {
      WPEP_Entity_ActionTracking::instance()->track_event( $post_id, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_LOGIN, $lead_segment );
    }

    if( isset( $auth_information[ 'success_response_type' ] )
      && $auth_information[ 'success_response_type' ] !== ''
      && in_array( $auth_type, [ 'login', 'register' ] ) ) {
      if( $auth_information[ 'success_response_type' ] == 'redirect' && isset( $auth_information[ 'success_response_redirect' ] ) ) {
        $response = [
          'status'       => 'ok',
          'redirect'     => $auth_information[ 'success_response_redirect' ]
        ];
      } elseif( $auth_information[ 'success_response_type' ] == 'redirect-to-public-profile' && is_user_logged_in() ) {
        $response = [
          'status'       => 'ok',
          'redirect'     => wpep_user_profile_public_link()
        ];
      }

    }

    return json_encode( $response );
  }

  private static function _maybe_standard_response( $request_data ) {
    if( is_user_logged_in() )
      return [
        'status'              => 'ok',
        'notification'        => wpep_get_setting_translated( 'user-auth-notification-already-logged-in' ),
        'notification_class'  => 'wpep-notification wpep-notification-success',
        'refresh'             => 1,
        'refresh_delay'       => 2000
      ];

    if( !isset( $request_data['auth_information'] ) )
      return [
        'status'              => 'error',
        'notification'        => __( "Invalid Request, the page will reload", "wpep" ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ];

    if( !isset( $request_data['auth_information']['wpep_auth_nonce'] ) || wp_verify_nonce( $request_data['auth_information']['wpep_auth_nonce'], 'wpep_auth_nonce' ) == false )
      return [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-auth-notification-page-expired' ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ];

    return false;
  }

}
