<?php

namespace WPEP\API\Discussion;

use WPEP\Entity\Discussion as WPEP_Discussion;
use WPEP\Container\FilterHacks as WPEP_FilterHacks;

class Comments {

  use WPEP_FilterHacks;

  public static function entries( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_id' ] ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ];

    $namespace     = $request_data[ 'namespace' ];
    $discussion_id = intval( $request_data[ 'discussion_id' ] );

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return [
        'status'  => 'error',
        'error'   => __( "Invalid Discussion namespace.", "wpep" )
      ];

    if( !$integration->current_user_can_view( $discussion_id ) )
      return [
        'status'  => 'error',
        'error'   => __( "Current user does not have access to post.", "wpep" )
      ];

    WPEP_Discussion::instance()->set_discussion_user_data(WPEP_DISCUSSION_USER_DATA_LAST_VIEW, time(), $discussion_id, get_current_user_id() );

    self::_static_filter_hack_deque( 'embed_oembed_html' );

    $response = [
      'status'        => 'ok',
      'timestamp'     => time(),
      'comments_data' => wpep_controller()->discussion->data->get_comments( $discussion_id, self::_discussion_comments_args( $discussion_id, $request_data, $integration ) )
    ];

    self::_static_filter_hack_reque( 'embed_oembed_html' );

    return $response;
  }

  public static function replies( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_id' ] ) || !isset( $request_data[ 'comment_parent_id' ] ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ];

    $namespace            = $request_data[ 'namespace' ];
    $discussion_id        = intval( $request_data[ 'discussion_id' ] );
    $comment_parent_id    = intval( $request_data[ 'comment_parent_id' ] );
    $last_activity_before = isset( $request_data[ 'last_activity_before' ] ) && $request_data[ 'last_activity_before' ] != false ? $request_data[ 'last_activity_before' ] : false;

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ];

    if( !$integration->current_user_can_view( $discussion_id ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to post.", "wpep" )
      ];

    $can_reply_comment = $integration->current_user_can_post( $discussion_id );

    if( $comment_parent_id !== 0 ) {
      $discussion_comment_parent = WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => $comment_parent_id ] );

      if( $discussion_comment_parent->discussion_id != $discussion_id )
        return [
          'status'  =>  'error',
          'error'   =>  __( "Parent Comment Discussion ID doesn't match the current discussion.", "wpep" )
        ];

      if( $can_reply_comment )
        $can_reply_comment = ( intval( $discussion_comment_parent->parent_id ) === 0 );
    }

    self::_static_filter_hack_deque( 'embed_oembed_html' );

    $response = [
      'status'               => 'ok',
      'timestamp'            => time(),
      'comment_replies_data' => wpep_controller()->discussion->data->get_comment_replies( $discussion_id, [
        'comment_parent_id'     => $comment_parent_id,
        'discussion_integration'=> $integration,
        'last_activity_before'  => $last_activity_before,
        'can_reply_comment'     => $can_reply_comment
      ] )
    ];

    self::_static_filter_hack_reque( 'embed_oembed_html' );

    return $response;
  }

  /**
   * @param $discussion_id
   * @param array $request_data
   * @param \WPEP_Discussion_Integration $discussion_integration
   * @return array
   */
  private static function _discussion_comments_args( $discussion_id, $request_data, $discussion_integration ) {
    $comment_args = [
      'discussion_integration'  => $discussion_integration,
      'pagination'              => true,
      'pagination_current_page' => 1,
      'search'                  => '',
      'filter_by_user'          => false,
      'structure'               => 'threaded',
      'type'                    => '',
      'parent__in'              => '',
      'date_query'              => null,
      'include_comment_by_id'   => false
    ];

    if( isset( $request_data[ 'wpep_post_comments_sync' ] ) && $request_data[ 'wpep_post_comments_sync' ] )
      $comment_args[ 'structure' ] = false;

    if( isset( $request_data[ 'wpep_post_comments_page' ] ) )
      $comment_args[ 'pagination_current_page' ] = intval( $request_data[ 'wpep_post_comments_page' ] );

    if( isset( $request_data[ 'wpep_post_comments_search'] ) )
      $comment_args[ 'search' ] = sanitize_text_field( $request_data[ 'wpep_post_comments_search'] );

    if( isset( $request_data[ 'wpep_post_comments_filter_by_user_id' ] ) ) {
      if( !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ) {
        $comment_args[ 'filter_by_user' ] = get_current_user_id();
      } else {
        $user_id = absint( $request_data[ 'wpep_post_comments_filter_by_user_id' ] );

        $comment_args[ 'filter_by_user' ] = ( $user_id === 0 ? false : $user_id );
      }
    }

    if( isset( $request_data[ 'wpep_post_comments_type'] ) )
      $comment_args[ 'type' ] = ( $discussion_integration->has_comment_type( $request_data[ 'wpep_post_comments_type'], $discussion_id ) ? $request_data[ 'wpep_post_comments_type'] : '' );

    if( isset( $request_data[ 'wpep_post_comments_parent_ids'] ) ) {
      $parent_ids = is_string( $request_data[ 'wpep_post_comments_parent_ids'] ) ? explode( ",", $request_data[ 'wpep_post_comments_parent_ids'] ) : $request_data[ 'wpep_post_comments_parent_ids'];

      foreach( $parent_ids as $parent_id_key => $parent_id )
        $parent_ids[ $parent_id_key ] = intval( $parent_id );

      if( !empty( $parent_ids ) )
        $comment_args[ 'parent__in' ] = $parent_ids;
    }

    if( isset( $request_data[ 'wpep_post_comment_highlight_by_id' ] ) && intval( $request_data[ 'wpep_post_comment_highlight_by_id' ] ) !== 0 )
      $comment_args[ 'include_comment_by_id' ] = intval( $request_data[ 'wpep_post_comment_highlight_by_id' ] );

    if( isset( $request_data[ 'wpep_post_comments_time_from'] ) ) {
      $request_data[ 'wpep_post_comments_time_from' ] = intval( $request_data[ 'wpep_post_comments_time_from' ] );

      if( $request_data[ 'wpep_post_comments_time_from' ] !== 0 ) {
        if( !isset( $comment_args[ 'date_query' ] ) || !is_array( $comment_args[ 'date_query' ] ) )
          $comment_args[ 'date_query' ] = [];

        $comment_args[ 'date_query' ][ '>' ] = date( 'Y-m-d H:i:s', $request_data[ 'wpep_post_comments_time_from' ] );
      }
    }

    if( isset( $request_data[ 'wpep_post_comments_time_to'] ) ) {
      $request_data[ 'wpep_post_comments_time_to' ] = intval( $request_data[ 'wpep_post_comments_time_to' ] );

      if( $request_data[ 'wpep_post_comments_time_to' ] !== 0 ) {
        if( !isset( $comment_args[ 'date_query' ] ) || !is_array( $comment_args[ 'date_query' ] ) )
          $comment_args[ 'date_query' ] = [];

        $comment_args[ 'date_query' ][ '<' ] = date( 'Y-m-d H:i:s', $request_data[ 'wpep_post_comments_time_to' ] );
      }
    }

    return $comment_args;
  }

}
