<?php

namespace WPEP\API\System;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;
use WPEP\API\Utils\Report as API_Utils_Report;

class Log {

  public static function entries_namespace_list( $request_data ) {
    return [
      'status'  =>  'ok',
      'entries' => WPEP_Entity_SystemLog::get_namespace_list()
    ];
  }

  public static function entries( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $pagination_limit = 10;
    $current_page     = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );
    $namespace        = false;

    if( isset( $request_data[ 'namespace' ] ) && in_array( $request_data[ 'namespace' ], WPEP_Entity_SystemLog::get_namespace_list() ) )
      $namespace = $request_data[ 'namespace' ];

    $get_entries_args = [
      'count_total' => true,
      'limit'       => $pagination_limit,
      'offset'      => ( $current_page - 1 ) * $pagination_limit
    ];

    if( $namespace !== false )
      $get_entries_args[ 'namespace' ] = $namespace;

    if( $namespace !== false && isset( $request_data[ 'event' ] ) ) {
      $get_entries_args[ 'event' ] = $request_data[ 'event' ];
    }

    if( isset( $request_data[ 'completed_at' ] ) && intval( $request_data[ 'completed_at' ] ) === 0 )
      $get_entries_args[ 'completed_at' ] = 0;

    if( isset( $request_data[ 'report_time_from' ] ) && isset( $request_data[ 'report_time_to' ] ) )
      $get_entries_args[ 'started_at' ] = [ $request_data[ 'report_time_from' ] => $request_data[ 'report_time_to' ]  ];

    $system_log_information = WPEP_Entity_SystemLog::get_entries( $get_entries_args );

    foreach( $system_log_information[ 'results' ] as $entry_key => $entry ) {
      $system_log_information[ 'results' ][ $entry_key ]->status    = ( intval( $entry->completed_at ) > 0 ? 'success' : ( intval( $entry->started_at ) + 5 < time() ? 'danger' : 'warning' ) );
      $system_log_information[ 'results' ][ $entry_key ]->arguments_formatted = [
        'type'    => 'textarea',
        'content' => '--- Started ' . wpep_format_date_with_time( $entry->started_at, 'log' ) . " \n\n" . $entry->started_arguments .
                     "\n--- Completed " . ( intval( $entry->completed_at ) > 0 ? wpep_format_date_with_time( $entry->completed_at, "log" ) : '???' ) . " \n\n" . $entry->completed_arguments
      ];
    }

    return [
      'status'          =>  'ok',
      'entries_header'  => [
        'namespace'           => __( "Namespace", "wpep" ),
        'event'               => __( "Event", "wpep" ),
        'arguments_formatted' => __( "Arguments", "wpep" )
      ],
      'entries'         => $system_log_information[ 'results' ],
      'pagination'      => [
        'current'       => $current_page,
        'total'         => ceil( $system_log_information[ 'total' ] / $pagination_limit ),
        'total_entries' => $system_log_information[ 'total' ]
      ]
    ];
  }

  public static function activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $data = WPEP_Entity_SystemLog::get_event_count_by_args( [
        'group_by'   => [ 'namespace', 'event' ],
        'time_from'  => $time_slot_from,
        'time_to'    => $time_slot_to
      ] );

      foreach( $data as $information )
        $current_time_slot_data[ $information->namespace . ' - ' . $information->event ] = $information->entries;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ] ;
  }

  public static function configuration_report( $request_data ) {
    $request_data[ 'start_time' ] = WPEP_Entity_SystemLog::get_reporting_start_at();

    $response = API_Utils_Report::configuration_for_statistics( $request_data );
    $response[ 'status' ] = 'ok';
    $response[ 'report_data_sources' ] = [
      'analyze' => [
        'title'       => __( "Activity", "wpep" ),
        'type'        => 'data-view-analyze',
        'data_action' => 'wpep/system/log/activity-report'
      ],
      'list' => [
        'title'       => __( "Entries", "wpep" ),
        'type'        => 'data-view-list',
        'data_action' => 'wpep/system/log/entries'
      ],
      'list-error' => [
        'title'            => __( "Errors", "wpep" ),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/system/log/entries',
        'data_action_args' => [
          'completed_at'  =>  0
        ]
      ]
    ];

    $namespace_list = WPEP_Entity_SystemLog::get_namespace_list();

    foreach( $namespace_list as $namespace ) {
      $event_list = WPEP_Entity_SystemLog::get_event_list_by_namespace( $namespace );

      if( empty( $event_list ) )
        continue;

      $response[ 'report_data_sources' ][ 'namespace-' . $namespace ] = [
        'title'       => $namespace,
        'type'        => 'group',
        'group'       => []
      ];

      foreach( $event_list as $event ) {
        $response[ 'report_data_sources' ][ 'namespace-' . $namespace ][ 'group' ][ $namespace . '-' . $event ] = [
          'title'       => $event,
          'type'        => 'data-view-list',
          'data_action' => 'wpep/system/log/entries',
          'data_action_args' => [
            'namespace' => $namespace,
            'event'     => $event
          ]
        ];
      }
    }

    return $response;
  }

}