<?php

namespace WPEP\Admin;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\SystemCache as Entity_SystemCache;

class Controller {

  /**
   * @var Coordinator\AddOnsSection
   */
  public $addOnsSection;
  /**
   * @var Coordinator\Assets
   */
  public $assets;
  /**
   * @var Coordinator\DataSystem
   */
  public $dataSystem;
  /**
   * @var Coordinator\Embed
   */
  public $embed;
  /**
   * @var Coordinator\PlatformHealth
   */
  public $platformHealth;
  /**
   * @var Coordinator\Discussion
   */
  public $discussion;
  /**
   * @var Coordinator\Post
   */
  public $post;
  /**
   * @var Coordinator\License
   */
  public $license;
  /**
   * @var Coordinator\Menu
   */
  public $menu;
  /**
   * @var Coordinator\FieldLibrary
   */
  public $fieldLibrary;
  /**
   * @var Coordinator\Migration
   */
  public $migration;
  /**
   * @var Coordinator\ShortCodeBuilder
   */
  public $shortCodeBuilder;
  /**
   * @var Coordinator\Statistics
   */
  public $statistics;
  /**
   * @var Coordinator\Settings
   */
  public $settings;
  /**
   * @var Coordinator\Toolbar
   */
  public $toolbar;
  /**
   * @var Coordinator\UserEdit
   */
  public $userEdit;
  /**
   * @var Coordinator\LearningCenter
   */
  public $learningCenter;

  /**
   * @return Controller
   */
  public function __construct() {
    $this->migration     = new Coordinator\Migration();

    return $this;
  }

  public function dispatch() {
    if( !is_admin() )
      return false;

    $this->_define();
    $this->_setup();

    add_action( 'activated_plugin', [ $this, '_any_plugin_activation_deactivation' ], 10 );
    add_action( 'deactivated_plugin', [ $this, '_any_plugin_activation_deactivation' ], 10 );

    add_action( "init", [ $this, '_setup_action_init' ], 10 );
    add_action( "admin_init", [ $this, '_setup_admin_action_init' ], 10 );
    add_action( "admin_menu", [ $this, '_setup_action_admin_menu' ], 10 );

    add_filter( 'plugin_action_links_' . plugin_basename( WPEP_BASE_FILE_PATH ), [ $this, 'action_links' ] );
    add_filter( 'user_row_actions', [ $this, 'user_row_actions' ], 10, 2 );

    add_action( 'create_course_category', [ $this, '_term_created' ] );

    if( wpep_get_setting_flag( 'configuration-setup-rules-flushed' ) == false )
      add_action( 'init', [ $this, '_setup_rules_flush_action_init' ], 7 );

    if( isset( $_GET['wpep-editor-display'] ) )
      add_action( 'admin_init', [ $this, 'wpep_admin_editor'], 100 );

    return true;
  }

  private function _define() {
    $this->addOnsSection = new Coordinator\AddOnsSection();
    $this->assets        = new Coordinator\Assets();
    $this->dataSystem    = new Coordinator\DataSystem();
    $this->discussion    = new Coordinator\Discussion();
    $this->embed         = new Coordinator\Embed();
    $this->platformHealth= new Coordinator\PlatformHealth();
    $this->post          = new Coordinator\Post();
    $this->license       = new Coordinator\License();
    $this->menu          = new Coordinator\Menu();
    $this->fieldLibrary  = new Coordinator\FieldLibrary();
    $this->settings      = new Coordinator\Settings();
    $this->shortCodeBuilder  = new Coordinator\ShortCodeBuilder();
    $this->statistics        = new Coordinator\Statistics();
    $this->toolbar           = new Coordinator\Toolbar();
    $this->userEdit          = new Coordinator\UserEdit();
    $this->learningCenter    = new Coordinator\LearningCenter();
  }

  private function _setup() {
    $this->addOnsSection->setup();
    $this->assets->setup();
    $this->dataSystem->setup();
    $this->embed->setup();
    $this->post->setup();
    $this->license->setup();
    $this->menu->setup();
    $this->fieldLibrary->setup();
    $this->settings->setup();
    $this->shortCodeBuilder->setup();
    $this->toolbar->setup();
    $this->userEdit->setup();
  }

  public function _setup_action_init() {
    $this->migration->_init();
    $this->post->_init();
    $this->learningCenter->_init();
    $this->platformHealth->_init();
  }

  public function _setup_admin_action_init() {
    if( function_exists( 'wp_add_privacy_policy_content' ) )
      wp_add_privacy_policy_content( WPEP_PLUGIN_NAME, $this->get_privacy_message() );
  }

  public function _setup_action_admin_menu() {
    $this->menu->_register_menu();

    do_action( 'wpep_menu_registered' );

    $this->discussion->_register_menu();
    $this->settings->_register_menu();
    $this->statistics->_register_menu();
    $this->platformHealth->_register_menu();
    $this->addOnsSection->_register_menu();
    $this->learningCenter->_register_menu();

    foreach (wpep_content_type()->standardContentTypes as $contentType ) {
      if ( !in_array('custom-fields', $contentType->get_post_supports() ) )
        continue;

      remove_meta_box( 'postcustom', $contentType->get_post_type(), 'normal' );
    }
  }

  public function _setup_rules_flush_action_init() {
    if( wpep_request()->is_request_type( 'ajax' ) )
      return;

    try {
      flush_rewrite_rules();

      WPEP_Controller::instance()->capabilities->register_caps_and_roles();

      \WPEPCronController::reset_cron_events();

      wpep_platform_health()->service->reset_notification_count_transient();
    } catch ( \Exception $exception ) {

    }

    wpep_set_setting( 'configuration-setup-rules-flushed', 1, true );

    add_action( "init", function() {
      wpep_redirect( wpep_get_current_url() );
      exit;
    }, 500 );

  }

  public function _term_created( $term_id ) {
    update_term_meta( $term_id, 'wpep_order', intval( get_term_meta( $term_id, 'wpep_order', true  ) ) );
  }

  public function action_links( $links ) {
    $links[] = '<a href="'. esc_url( wpep_admin_url_settings() ) .'">' . __( "Settings", 'wpep' )  . '</a>';
    return $links;
  }

  public function user_row_actions( $actions, $user_object ) {
    $actions[ 'wpep_user_analytics' ] = '<a data-ec-component-trigger="open" data-ec-component-trigger-args="' . esc_attr(
        json_encode( [
          'component' => 'interval-navigation-panel',
          'action'    => 'wpep/system/statistics/user-id-configuration-report',
          'user_id'   => $user_object->ID
        ] )
      ). '" href="#">' . sprintf( __( "%s Analytics", "wpep" ), WPEP_PLUGIN_NAME ) . '</a>';

    if( wpep_public_profiles_enabled() )
      $actions[ 'wpep_public_profile' ] = '<a href="' . wpep_user_profile_public_link( $user_object ). '">' . sprintf( __( "%s Public Profile", "wpep" ), WPEP_PLUGIN_NAME ) . '</a>';

    return $actions;
  }

  public function get_privacy_message() {
    $content = '
			<div contenteditable="false">' .
      '<p class="wp-policy-help">' .
      sprintf( __( 'This sample language includes the basics around what personal data your %s powered eLearning Platform may be collecting, storing and sharing, as well as who may have access to that data. 

Depending on what settings are enabled and which additional plugins are used, the specific information shared by your store will vary. 

We recommend consulting with a lawyer when deciding what information to disclose on your privacy policy.
.', 'wpep' ), WPEP_PLUGIN_NAME ) .
      '</p>' .
      '</div>' .
      '<p>' . __( 'We collect information about you while doing Courses & Assessments.', 'wpep' ) . '</p>' .
      '<h2>' . __( 'What we collect and store', 'wpep' ) . '</h2>' .
      '<p>' . __( 'While you visit our elearning platform, we’ll track:', 'wpep' ) . '</p>' .
      '<ul>' .
      '<li>' . __( 'Courses you have started, including the time you have started them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Courses you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Lessons you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Learning Paths you have chosen within courses.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Going through Assessments will store your answers, assessment status & the timestamp of your last interaction with the assessment.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Certifications you have earned, including the time you earned it..', 'wpep' ) . '</li>' .
      '<li>' . __( 'Certifications you have downloaded, including the time you downloaded it.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Videos you have accessed from an Index Page, including the time you accessed it.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Offers you have claimed from an Index Page, including the time you claimed it.', 'wpep' ) . '</li>' .
      '<li>' . __( 'E-Books you have downloaded from an Index Page, including the time you downloaded it.', 'wpep' ) . '</li>' .
      '<li>' . __( 'List Items you have accessed from an Index Page, including the time you accessed it.', 'wpep' ) . '</li>' .
      '</ul>' .
      '<h2>' . __( 'Who on our team has access', 'wpep' ) . '</h2>' .
      '<p>' . __( 'Members of our team such as administrators, platform managers and individual courses managers and instructors have access to the information you provide us. They can access:', 'wpep' ) . '</p>' .
      '<ul>' .
      '<li>' . __( 'Courses you have started, including the time you have started them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Courses you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Lessons you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Learning Paths you have chosen within courses.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Going through Assessments will store your answers, assessment status & the timestamp of your last interaction with the assessment..', 'wpep' ) . '</li>' .
      '</ul>' .
      '<p>' . __( 'Our team members have access to this information to improve the training & to support you on your learning journey.', 'wpep' ) . '</p>';

    return apply_filters( 'wpep_privacy_policy_content', $content );
  }

  public function _any_plugin_activation_deactivation() {
    wpep_platform_health()->service->reset_notification_count_transient();
    Entity_SystemCache::truncate_table();

    wpep_set_setting( 'configuration-setup-rules-flushed', 0, true );
  }

  public function wpep_admin_editor() {
    $content = ( isset( $_POST['content'] ) ? wpep_utilities_sanitize_input( $_POST['content'] ) : '' );

    if( $content == ''
      && (
        current_user_can( WPEP_CAP_EDIT_COURSE )
        || current_user_can( WPEP_CAP_EDIT_ASSESSMENT )
        || current_user_can( WPEP_CAP_MANAGE_SETTINGS )
      ) ) {

      if( isset( $_POST[ 'entity_id'] ) && $_POST[ 'entity_alias' ] ) {
        if( $_POST[ 'entity_alias' ] === 'assessment' ) {
          $assessment_type = ( isset( $_POST['assessment_type'] ) ? sanitize_title( $_POST['assessment_type'] ) : '' );

          $content = ( $assessment_type == 'success'
            ? get_post_meta( intval( $_POST['entity_id'] ), 'wpep_success_message', true )
            : ( $assessment_type == 'failed'
              ? get_post_meta( intval( $_POST['entity_id'] ), 'wpep_failed_message', true )
              : get_post_meta( intval( $_POST['entity_id'] ), 'wpep_assessment_description', true )
            )
          );

          if( $content == '' && $assessment_type == 'success' )
            $content = wpep_get_setting_translated( 'assessment-success-message' );

          if( $content == '' && $assessment_type == 'failed' )
            $content = wpep_get_setting_translated( 'assessment-status-failed' );

        } else if( $_POST[ 'entity_alias' ] === 'section_lesson' ) {
          $content = wpep_wpdb()->get_col('SELECT `content` FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . ' WHERE `id` = ' . intval( $_POST['entity_id'] ) );
        } else if( $_POST[ 'entity_alias' ] === 'section' ) {
          $content = wpep_wpdb()->get_col('SELECT `content` FROM ' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . ' WHERE `id` = ' . intval( $_POST['entity_id'] ) );
        }
      }

      if( is_array( $content ) )
        $content = array_shift( $content );

      if( !is_string( $content ) )
        $content = '';
    }

    $editor_args = [];

    if( isset( $_POST['editor_class'] ) )
      $editor_args['editor_class'] = sanitize_text_field( $_POST['editor_class'] );

    wpep_editor( $content, $_POST['id'], $_POST['name'], $editor_args );

    exit;
  }

}