<?php

namespace WPEP\Admin\Coordinator\DataSystem;

use WPEP\Container\FormFields as WPEP_FormFieds;
use WPEP\Container\DataWalker as WPEP_DataWalker;

class CSVCourses {

  use WPEP_FormFieds, WPEP_DataWalker;

  public function setup() {
    add_action( 'wp_ajax_wpep_data_system_import_csv_courses_process', [ $this, 'import_csv_process' ] );
  }

  public function import_csv_process() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    if( !wpep_content_type()->is_enabled( WPEP_POST_TYPE_COURSE ) )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Course Content Type is not enabled.", "wpep" ) . '</div>');

    if( isset( $_POST[ 'import_file' ] ) ) {
      $csv_file = $_POST[ 'import_file' ];

      if( !wpep_string_ends_with( $csv_file, '.csv' ) ) {
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "You must upload a CSV file, must end with .csv", "wpep" ) . '</div>';

        $this->_select_file();

        exit();
      }

      $upload_dir   = wp_upload_dir();

      $csv_file = str_replace( $upload_dir['baseurl'], $upload_dir['basedir'], $csv_file );

      if ( ($handle = fopen($csv_file, "r") ) !== FALSE ) {
        update_option( '_wpep_data_system_csv_import_progress', [
          'file'          => $csv_file,
          'current_line'  => 0
        ] );

        fclose($handle);

        echo '<div class="wpep-notification wpep-notification-success">' . __( "CSV File is being prepared for the import, please wait... ", "wpep" ) . '</div>';

        exit;
      }

      echo '<div class="wpep-notification wpep-notification-danger">' . __( "The CSV File couldn't be read, please retry. ", "wpep" ) . '</div>';

      $this->_select_file();

      exit();
    } else {
      $this->_maybe_continue_import_process();
    }

    $this->_select_file();

    exit;
  }

  private function _maybe_continue_import_process() {
    $import_progress_data = get_option( '_wpep_data_system_csv_import_progress', false );

    if( $import_progress_data == false )
      return;

    @set_time_limit(0);
    @ini_set('memory_limit', '512M');

    $csvFile = file($import_progress_data['file'] );

    if( $csvFile === false ) {
      delete_option( '_wpep_data_system_csv_import_progress' );

      echo '<div class="wpep-notification wpep-notification-danger">' . __( "The CSV File couldn't be read, please retry. ", "wpep" ) . '</div>';

      $this->_select_file();
    }

    $csv_current_line = 1;

    $course_id     = 0;
    $section_id    = 0;
    $section_order = 1;
    $lesson_order  = 1;

    foreach ($csvFile as $line) {
      $line = str_getcsv( $line, ( strpos( $line, ';') === false ? ',' : ';' ) );

      if( $csv_current_line == 1 ) { $csv_current_line++; continue; }

      if( $line[0] !== '' ) {

        $course_id = wp_insert_post( [
          'post_content'    => $line[1],
          'post_title'      => $line[0],
          'post_status'     => 'draft',
          'post_type'       => WPEP_POST_TYPE_COURSE,
        ] );

        update_post_meta( $course_id, WPEP_COURSE_TYPE_POST_META, WPEP_COURSE_TYPE_EXECUTION_PLAN );

        $section_id    = 0;
        $section_order = 1;
        $lesson_order  = 1;
      }

      if( $course_id == 0 )
        continue;

      if( $line[2] !== '' ) {
        wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION, [
          'post_id' => $course_id,
          'title'   => $line[2],
          'content' => $line[3],
          'order'   => $section_order,
          'options' => json_encode( [] )
        ] );

        $section_id = wpep_wpdb()->insert_id;
        $section_order++;
      }

      if( $section_id == 0 )
        continue;

      if( $line[4] !== '' ) {
        $downloads_list = [];

        if( isset( $line[6] ) && !empty( $line[6] ) ) {
          $tokens = explode( ",", $line[6] );

          foreach( $tokens as $link ) {
            if( strpos( $link, ':::' ) === false ) {
              $downloads_list[ trim( $link ) ] = __( 'Download', "wpep" );

              continue;
            }

            $current_pieces = explode( ":::", $link );

            if( !isset( $current_pieces[0] ) || !isset( $current_pieces[1] ) )
              continue;

            if( strpos( $current_pieces[0], 'http' ) === 0 || strpos( $current_pieces[0], 'www' ) === 0 ) {
              $downloads_list[ trim( $current_pieces[0] ) ] = trim( $current_pieces[1] );
              continue;
            }

            $downloads_list[ trim( $current_pieces[1] ) ] = trim( $current_pieces[0] );
          }
        }

        wpep_wpdb()->insert( wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON, [
          'post_id'   => $course_id,
          'section_id'=> $section_id,
          'title'     => $line[4],
          'content'   => $line[5],
          'order'     => $lesson_order,
          'options'   => json_encode( [
            'downloads_list'  => $downloads_list
          ] )
        ] );

        $lesson_order++;
      }
    }

    delete_option( '_wpep_data_system_csv_import_progress' );

    echo '<div class="wpep-notification wpep-notification-success">' . __( "The CSV File has been imported successfully. ", "wpep" ) . '</div>';

    $this->_select_file();
    exit;
  }

  private function _select_file() {
    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Import CSV File", "wpep" ) . '</h3>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "Start Import", "wpep" ) . '"/>';
    echo '</div>';

    echo '<div class="wpep-import-file-container">';
    $this->display_file_field("import_file", "import_file" );
    echo    '<p class="wpep-description">';
    echo        __( "The CSV may result in unexpected results if you do not follow the Standard.", "wpep" );
    echo        ' <strong><a href="' . WPEP_BASE_URL_PATH . 'assets/wpep-csv-import-sample.csv">' . __( "Download Sample", "wpep" ) . '</a></strong>';
    echo    '</p>';
    echo    '<p class="wpep-description">' . __( "Course Title;Course Description;Section Title;Section Description;Lesson Title;Lesson Content;Lesson Downloads", "wpep" ) . '</p>';

    echo '</div>';


  }

}
