<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;
use WPEP\Container\FormFields as WPEP_FormFields;
use WPEP\Entity\SystemCache as Entity_SystemCache;

class Settings {

  use WPEP_FormFields;

  /**
   * @var Settings\AddOns
   */
  public $addOns;
  /**
   * @var Settings\Data
   */
  public $data;

  private $_default_field_prefix = 'wpep_settings';

  /**
   * @var bool|string
   */
  private $_method_get_value = false;
  /**
   * @var bool|string
   */
  private $_method_get_value_translated = false;

  public function __construct() {
    $this->set_field_prefix( $this->_default_field_prefix );
  }

  public function setup() {
    $this->addOns = new Settings\AddOns();
    $this->data   = new Settings\Data();

    add_action( 'init', [ $this, '_init' ], 0 );
  }

  public function _register_menu() {
    wpep_admin_register_menu(
      sprintf( __("%s Settings", 'wpep'), WPEP_PLUGIN_NAME ),
      __( "Settings", 'wpep'),
      WPEP_CAP_MANAGE_SETTINGS,
      "wpep-settings",
      [ $this, "display" ]
    );
  }

  public function _init() {
    if( !is_admin() )
      return;

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;

    if( !isset( $_POST['wpep_settings_save'] ) )
      return;

    $this->save_settings( $_POST );

    $url = add_query_arg( 'wpep-settings-saved', 'true', wpep_get_current_url() );
    $url = remove_query_arg( 'wpep-walk-trough', $url );

    wpep_redirect( $url );
    exit;
  }

  public function display() {
    echo '<div class="wrap wpep-settings-container" 
               data-wpep-page="settings" 
               data-wpep-tab-count="' . ( ( !empty( WPEP_Controller::instance()->addOns->get_add_on_settings_list() ) ? 2 : 1 ) + $this->data->get_map_item_count() ) . '"
          >';
    echo '<h2 class="wpep-title-page">' . sprintf( __( "%s Settings", 'wpep' ), WPEP_PLUGIN_NAME ) . '</h2>';

    if( isset( $_GET['wpep-settings-saved']) ) {
      echo '<div class="updated notice is-dismissible">';
      echo  '<p>' . __('Settings have been updated.', 'wpep') . '</p>';
      echo  '<button class="notice-dismiss" type="button"><span class="screen-reader-text">' . __( "Dismiss this notice.", 'wpep') . '</span></button>';
      echo '</div>';
    }

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : 'dashboard' );

    if( !$this->data->has_map_key( $current_tab ) && $current_tab !== 'addon' )
      $current_tab = 'dashboard';

    echo wpep_administration_navigation(
      [
          'dashboard' => __( "Dashboard", "wpep" )
        ] + $this->data->get_map_top_level_navigation_data() + (
          !empty( WPEP_Controller::instance()->addOns->get_add_on_settings_list() ) ? [ 'addon' => __( 'AddOns Settings', "wpep" ) ] : []
        ),
        $current_tab,
        wpep_admin_url_settings(),
      'tab',
      'primary',
      'settings'
    );

    if( $current_tab === 'dashboard' )
      $this->_display_dashboard();
    elseif( $current_tab === 'addon' )
      $this->_display_addon_settings();
    else
      $this->_display_management_by_data_key( $current_tab );

    echo '</div>';
  }

  private function _display_dashboard() {
    $addon_settings_list = WPEP_Controller::instance()->addOns->get_add_on_settings_list();

    echo '<form data-wpep-form="search" 
                data-wpep-form-action="wpep/system/setting/search" 
                data-wpep-form-content-target="#wpep-settings-search-target"
                data-wpep-form-alternative-target=".wpep-settings-search-target-alternative"
                data-wpep-form-search-min-character-count="3"
                data-wpep-form-search-min-character-notification="' . __( "Please write in minimum %s characters to search...", "wpep" ) . '">';
    echo    '<input type="text" name="search" value="" placeholder="' . sprintf( __( "Search for %s Settings", "wpep" ), WPEP_PLUGIN_NAME ) . '"/>';
    echo '</form>';

    echo '<div id="wpep-settings-search-target"></div>';

    echo '<h2 class="wpep-settings-dashboard-title wpep-settings-search-target-alternative">' . sprintf( __( "Direct Access Links to %s Core Features, Settings & Options:", "wpep" ), WPEP_PLUGIN_NAME ) . '</h2>';

    echo '<div class="wpep-settings-dashboard wpep-settings-search-target-alternative" data-wpep-navigation="settings">';

    foreach( WPEP_Controller::instance()->adminController->settings->data->get_map_dashboard_data() as $data_map_key => $data_map ) {
      echo '<div class="wpep-settings-dashboard-item">';
      echo    '<h2 data-wpep-tab="' . $data_map[ 'alias' ] . '">' . $data_map[ 'name' ] . '</h2>';
      echo    '<ul>';

      foreach( $data_map[ 'sections' ] as $data_map_section_key => $data_map_section_data ) {
        echo '<li>';
        echo  '<a data-wpep-tab="' . $data_map[ 'alias' ] . '-' . $data_map_section_data[ 'alias' ] . '" href="' . wpep_admin_url_settings( [ 'tab' => $data_map[ 'alias' ], 'tab_inner' => $data_map_section_data[ 'alias' ] ] ). '">';
        echo    '<span>' . $data_map_section_data[ 'name' ] . '</span>';
        echo  '</a>';
        echo '</li>';
      }

      echo    '</ul>';
      echo '</div>';
    }

    if( !empty( $addon_settings_list ) ) {
      echo '<div class="wpep-settings-dashboard-item">';
      echo    '<h2 data-wpep-tab="addon">' . __( "AddOn Settings" ) . '</h2>';
      echo    '<ul>';

      foreach( $addon_settings_list as $inner_tab_key => $addon ) {
        $image = '';

        $class = '';

        echo '<li>';
        echo  '<a class="' . $class . '" href="' . wpep_admin_url_settings( [ 'tab' => 'addon', 'addon' => $addon->get_alias() ] ). '">';
        echo    ( $image !== '' ? '<img src="' . $image . '"/>' : '' );
        echo    '<span>' . $addon->get_name() . '</span>';
        echo  '</a>';
        echo '</li>';
      }

      echo    '</ul>';
      echo '</div>';
    }

    echo '</div>';

    echo '<div id="wpep-settings-preload-editor-container" style="display:none !important;opacity: 0 !important;">';
    wp_editor( '', 'wpep-settings-preload-editor' );
    echo '</div>';
    echo '<script>jQuery( "#wpep-settings-preload-editor-container" ).remove(); </script>';
  }

  private function _display_management_by_data_key( $data_key ) {
    $inner_data_key = ( isset( $_GET['tab_inner'] ) ? $_GET['tab_inner'] : false );

    if( $inner_data_key === false
        || !$this->data->has_map_inner_key( $data_key, $inner_data_key ) )
      $inner_data_key = $this->data->get_map_first_inner_key( $data_key );

    echo wpep_administration_navigation(
      $this->data->get_map_secondary_level_navigation_data( $data_key ),
      $inner_data_key,
      wpep_admin_url_settings( [ 'tab' => $data_key ] ),
      'tab_inner',
      'secondary',
      'settings',
      $data_key . '-'
    );

    $management_data    = $this->data->get_map_for_management( $data_key, $inner_data_key );

    $start_table_row_attr = $this->get_table_row_data_attributes();

    $this->setup_field_map_settings( $this->data->get_map_field_settings( $data_key ) );

    wpep_template_load( 'admin/settings/management-by-data-key.php', [
      'settings_instance' => $this,
      'field_map'         => $management_data[ 'field_map' ],
      'has_live_preview'  => ( isset( $management_data[ 'has_live_preview' ] ) ? $management_data[ 'has_live_preview' ] : false )
    ] );

    $this->reset_field_map_settings( $start_table_row_attr );
  }

  private function _display_addon_settings() {
    echo '<form novalidate="novalidate" method="post">';

    $this->addOns->display();

    echo  '<p class="submit"><input type="submit" value="' . __( "Save Changes", "wpep" ) . '" class="button button-primary" id="submit" name="wpep_settings_save"></p>';

    echo '</form>';
  }

  public function setup_field_map_settings( $map_field_settings ) {
    if( isset( $map_field_settings[ 'field_prefix' ] ))
      $this->set_field_prefix( $map_field_settings[ 'field_prefix' ] );
    if( isset( $map_field_settings[ 'get_setting' ] ) )
      $this->_method_get_value = $map_field_settings[ 'get_setting' ];
    if( isset( $map_field_settings[ 'get_setting_translated' ] ) )
      $this->_method_get_value_translated = $map_field_settings[ 'get_setting_translated' ];
  }

  public function reset_field_map_settings( $start_table_row_attr = [] ) {
    $this->set_field_prefix( $this->_default_field_prefix );
    $this->set_table_row_data_attributes( $start_table_row_attr );
    $this->_method_get_value = false;
    $this->_method_get_value_translated = false;
  }

  public function get_value( $field_key, $field_default = '' ) {
    $response_inner_index = strpos( $field_key, '[' );

    if( $response_inner_index !== false ) {
      $base_field_key = $field_key;

      $field_key = substr( $base_field_key, 0, $response_inner_index );
      $response_inner_index = trim( substr( $base_field_key, $response_inner_index ), '[]' );
    }

    $response = ( $this->_method_get_value !== false ? call_user_func( $this->_method_get_value, $field_key, $field_default ) : WPEP_Settings::instance()->get_option( $field_key, $field_default ) );

    if( !$response_inner_index )
      return $response;

    if( !is_array( $response ) )
      return $field_default;

    return ( isset( $response[ $response_inner_index ] ) ? $response[ $response_inner_index ] : $field_default );
  }

  public function get_value_translated( $field_key ) {
    if( $this->_method_get_value_translated !== false )
      return call_user_func( $this->_method_get_value_translated, $field_key );

    return WPEP_Settings::instance()->get_option_translated( $field_key );
  }

  public function get_data_map_search_content( $search_term, $refresh_cache = false, $show_updated_at = true ) {
    if( strpos( $search_term, '||' ) !== false ) {
      $search_tokens = explode( '||', $search_term );

      if( isset( $search_tokens[ 0 ], $search_tokens[ 1 ] ) ) {
        $management_map = $this->data->get_map_for_management( $search_tokens[ 0 ], $search_tokens[ 1 ] );

        if( false !== $management_map ) {
          $search_result = [
            'map'         => $management_map,
            'updated_at'  => time()
          ];
        }

      }
    }

    if( !isset( $search_result ) )
      $search_result = $this->data->get_map_by_args( [ 'search' => $search_term, 'refresh_cache' => $refresh_cache ] );

    return wpep_template_get( 'admin/settings/management-by-search-results.php', [
      'settings_instance' => $this,
      'data_map'          => $search_result[ 'map' ],
      'updated_at'        => ( $show_updated_at ? $search_result[ 'updated_at' ] : false )
    ] );
  }

  public function get_data_map_search_by_index_content( $search_term ) {
    $search_tokens = explode( '||', $search_term );
    $data_map_management = [];

    if( count( $search_tokens ) >= 2 ) {
      $data_map_management = $this->data->get_map_for_management( $search_tokens[ 0 ], $search_tokens[ 1 ] );

      if( false === $data_map_management ) {
        $data_map_management = [];
      } elseif( isset( $search_tokens[ 2 ] ) ) {
        if( isset( $data_map_management[ 'field_map' ][ $search_tokens[ 2 ] ] ) ) {
          $data_map_management[ 'field_map' ] = [
            $search_tokens[ 2 ] => $data_map_management[ 'field_map' ][ $search_tokens[ 2 ] ]
          ];

          $data_map_management[ 'field_map' ][ $search_tokens[ 2 ] ][ 'has_title' ] = false;
        } else {
          $data_map_management = [];
        }
      }
    }

    if( !empty( $data_map_management ) )
      $this->setup_field_map_settings( $this->data->get_map_field_settings( $search_tokens[ 0 ] ) );

    return wpep_template_get( 'admin/settings/management-by-search-index.php', [
      'settings_instance'   => $this,
      'data_map_management' => $data_map_management
    ] );
  }

  public function save_settings( $request_data ) {
    if( empty( $request_data ) )
      return;

    if( isset( $request_data['wpep_settings'] ) )
      WPEP_Settings::instance()->update_settings( $request_data['wpep_settings'] );

    if( isset( $request_data['wpep_settings_post_order'] ) )
      WPEP_Settings::instance()->update_post_order( array_keys( $request_data['wpep_settings_post_order'] ) );

    if( isset( $request_data['wpep_settings_taxonomy_order'] ) )
      WPEP_Settings::instance()->update_taxonomy_order( array_keys( $request_data['wpep_settings_taxonomy_order'] ) );

    if( isset( $request_data['wpep_add_on_settings'] ) )
      WPEP_Settings::instance()->update_addon_settings( $request_data['wpep_add_on_settings'] );

    do_action( "wpep_administration_settings_saved", $request_data );

    wpep_platform_health()->service->reset_notification_count_transient();
    Entity_SystemCache::truncate_table();
  }

}