<?php

namespace WPEP;

use WPEP\Settings as WPEP_Settings;

/**
 * Class ContentType
 * @package WPEP
 * @author Robert Rusu
 */
class ContentType {

  use ContentType\Container\Library;
  use ContentType\Container\Admin;

  /**
   * @var ContentType
   */
  protected static $_instance;

  public static function instance() {
    if( self::$_instance === null )
      self::$_instance = new self();

    return self::$_instance;
  }

  /**
   * @var ContentType\Skeleton\Skeleton[]
   */
  public $contentTypes        = [];
  /**
   * @var ContentType\Skeleton\Library[]
   */
  public $switchableContentTypes  = [];
  /**
   * @var ContentType\Skeleton\Standard[]
   */
  public $standardContentTypes = [];
  /**
   * @var ContentType\Skeleton\Library[]
   */
  public $libraryContentTypes = [];

  private $_data = [];

  public function setup() {
    $this->contentTypes           = [];
    $this->switchableContentTypes = [];
    $this->standardContentTypes   = [];
    $this->libraryContentTypes    = [];
    foreach( [
               new ContentType\Course(),
               new ContentType\Assessment(),
               new ContentType\Certification(),
               new ContentType\Video(),
               new ContentType\EBook(),
               new ContentType\Offer(),
               new ContentType\ListItem()
             ] as $contentType ) {
      $this->_register_content_type( $contentType );
    }

    do_action( 'wpep_content_types_registered' );

    add_action( 'init', [ $this, '_register_post_types' ], WPEP_POST_TYPE_REGISTRATION_PRIORITY );
  }

  /**
   * @param ContentType\Skeleton\Skeleton $contentType
   * @return bool
   */
  private function _register_content_type( $contentType ) {
    if( !$contentType->is_enabled() ) {
      if( is_subclass_of( $contentType, '\WPEP\ContentType\Skeleton\Library' ) )
        $this->switchableContentTypes[ $contentType->get_post_type() ] = $contentType;

      return true;
    }

    $this->contentTypes[ $contentType->get_post_type() ] = $contentType;

    if( is_subclass_of( $contentType, '\WPEP\ContentType\Skeleton\Standard' ) ) {
      $this->standardContentTypes[$contentType->get_post_type()] = $contentType;

      foreach( $contentType->get_settings_defaults() as $key => $value )
        WPEP_Settings::instance()->defaultOptions[ $key ] = $value;
    }

    if( is_subclass_of( $contentType, '\WPEP\ContentType\Skeleton\Library' ) ) {
      $this->libraryContentTypes[ $contentType->get_post_type() ] = $contentType;
      $this->switchableContentTypes[ $contentType->get_post_type() ] = $contentType;
    }

    $this->contentTypes[ $contentType->get_post_type() ]->setup();

    return true;
  }

  /**
   * @param ContentType\Skeleton\Skeleton $contentType
   * @return bool
   */
  public function register_content_type( $contentType ) {
    if( did_action( 'init' ) )
      return false;

    return $this->_register_content_type( $contentType );
  }

  public function _register_post_types() {
    foreach( $this->contentTypes as $contentType )
      $contentType->register_post_type();


    foreach( $this->libraryContentTypes as $libraryContentType ) {
      $libraryContentType->library_register_settings();
    }

    foreach( $this->contentTypes as $contentType )
      $contentType->init();

    if( !is_admin() )
      return;

    $this->_setup_administration();
  }

  public function is_enabled( $post_type ) {
    return isset( $this->contentTypes[ $post_type ] ) ? $this->contentTypes[ $post_type ]->is_enabled() : false;
  }

  /**
   * @return ContentType\Skeleton\Integration\LearningJourney[]
   */
  public function get_learning_journey_instances() {
    if( isset( $this->_data[ 'learning_journey_instances' ] ) )
      return $this->_data[ 'learning_journey_instances' ];

    $this->_data[ 'learning_journey_instances' ] = [];

    foreach( $this->contentTypes as $contentType ) {
      $current_learning_journey_instance = $contentType->get_learning_journey_instance();

      if( $current_learning_journey_instance === false )
        continue;

      $this->_data[ 'learning_journey_instances' ][ $contentType->get_post_type() ] = $current_learning_journey_instance;
    }

    return $this->_data[ 'learning_journey_instances' ];
  }

  public function data_post_connection_array() {
    if( isset( $this->_data[ 'post_connection_array' ] ) )
      return $this->_data[ 'post_connection_array' ];

    $this->_data[ 'post_connection_array' ] = [];

    foreach( $this->contentTypes as $contentType )
      $this->_data[ 'post_connection_array' ] += $contentType->data_post_connection();

    return $this->_data[ 'post_connection_array' ];
  }

  public function data_export_array() {
    $response = [];

    foreach( $this->standardContentTypes as $contentType ) {
      $response[ $contentType->get_post_type() ] = $contentType->data_export_array();

      if( !is_array( $response[ $contentType->get_post_type() ] ) )
        unset( $response[ $contentType->get_post_type() ] );
    }

    return $response;
  }

  public function data_import_array( $content_array, $id_mapping = [] ) {
    foreach( $this->standardContentTypes as $contentType ) {
      if( !isset( $content_array[ $contentType->get_post_type() ] ) )
        continue;

      $id_mapping[ $contentType->get_post_type() ] = $contentType->data_import_array( $content_array[ $contentType->get_post_type() ], $id_mapping );
    }

    return $id_mapping;
  }

  public function data_import_array_post_connection( $content_array, $post_id_map ) {
    foreach( $this->standardContentTypes as $contentType ) {
      if( !isset( $content_array[ $contentType->get_post_type() ] ) )
        continue;

      $contentType->data_import_array_post_connection( $content_array[ $contentType->get_post_type() ], $post_id_map );
    }

    return true;
  }

  public function data_asset_map() {
    $response = [];

    foreach( $this->standardContentTypes as $standardContentType )
      $response[ $standardContentType->get_post_type() ] = $standardContentType->data_asset_map();

    return apply_filters( 'wpep_content_type_data_asset_map', $response );
  }

  public function user_has_created_content( $user_id ) {
    return !empty( get_posts( [
      'author'          => $user_id,
      'paged'           => 1,
      'post_type'       => array_keys( wpep_content_type()->contentTypes ),
      'fields'          => 'ids',
      'posts_per_page'  => 1
    ] ) );
  }

  public function get_user_created_content_ids( $user_id ) {
    return get_posts( [
      'author'          => $user_id,
      'paged'           => 0,
      'post_type'       => array_keys( wpep_content_type()->contentTypes ),
      'fields'          => 'ids',
      'posts_per_page'  => -1
    ] );
  }

  public function are_registered_content_types_reachable() {
    $standardContentTypes = wpep_content_type()->standardContentTypes;

    foreach( $standardContentTypes as $standardContentType ) {
      if( !$standardContentType->is_publicly_queryable() )
        continue;

      $tested_post    = get_posts( [
        'post_type'   => $standardContentType->get_post_type(),
        'numberposts' => 1
      ] );

      if( empty( $tested_post ) )
        continue;

      $tested_post = array_shift( $tested_post );

      if( $tested_post->ID != wpep_get_post_id_by_url( get_permalink( $tested_post ) ) )
        return false;
    }

    return true;
  }

}