<?php

namespace WPEP\ContentType\Assessment\Delivery;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Assessment as EntityAssessment;

class Request {

  public $assessment_id = 0;
  public $assessment_time_limit = 0;
  public $assessment_type       = WPEP_ASSESSMENT_TYPE_STANDARD;
  public $assessment_navigation_step = 0;
  public $assessment_is_reviewing_submission = false;

  public $course_id     = 0;
  public $section_id    = 0;

  public $user_id       = 0;
  public $user_status     = 0;
  /**
   * @var bool|int
   */
  public $user_start_time = false;

  private $_request_settings = [];

  public function __construct( $assessment_id, $user_id, $request_settings = [] ) {
    $this->assessment_id = $assessment_id;
    $this->user_id       = intval( $user_id );

    $this->_request_settings = is_array( $request_settings ) ? $request_settings : [];
  }

  private function _setup_data() {
    $this->course_id     = wpep_get_assessment_course_id( $this->assessment_id );
    $this->section_id    = wpep_get_assessment_course_section_id( $this->assessment_id );
    $this->user_status   = wpep_user_assessment_status( $this->assessment_id );
    $this->user_start_time = ( $this->user_id === 0 ? false : wpep_content_type_assessment_instance()->user->data->get_value(
      WPEP_USER_ASSESSMENT_START_TIME, $this->assessment_id, 0, 0, $this->user_id
    ) );

    $this->assessment_type = wpep_get_assessment_type( $this->assessment_id );

    if( $this->assessment_type !== WPEP_ASSESSMENT_TYPE_ASSIGNMENT ) {
      $this->assessment_time_limit = intval( get_post_meta( $this->assessment_id, 'wpep_available_minutes', true ) );

      if( $this->assessment_time_limit != 0 )
        $this->assessment_time_limit *= 60;
    } else {
      $this->assessment_time_limit = 0;
    }
  }

  public function set_user_status( $user_status ) {
    $this->user_status = $user_status;

    if( $this->user_id === 0 )
      return;

    wpep_content_type_assessment_instance()->user->data->set(
      WPEP_USER_ASSESSMENT_STATUS, $user_status, $this->assessment_id, 0, 0, $this->user_id
    );
  }

  public function set_user_start_time( $start_time ) {
    $this->user_start_time = $start_time;

    if( $this->user_id === 0 )
      return;

    wpep_content_type_assessment_instance()->user->data->set(
      WPEP_USER_ASSESSMENT_START_TIME, $start_time, $this->assessment_id, 0, 0, $this->user_id
    );
  }

  public function serve( $request_type = 'view-assessment', $request_data = [] ) {
    $this->_setup_data();

    if( $this->course_id !== 0 ) {
      $previous_assessments = wpep_user_assessment_requirements_for_section( $this->section_id );

      if( is_array( $previous_assessments ) && !empty( $previous_assessments ) ) {
        $previous_assessment = array_shift( $previous_assessments );

        return [
          'type'          => 'switch_assessment',
          'assessment_id' => $previous_assessment
        ];
      }
    }

    if( $request_type == 'retake-assessment' ) {
      wpep_content_type_assessment_instance()->user->data->delete(
        false,
        $this->assessment_id,
        false,
        false,
        $this->user_id
      );

      return [
        'type'          => 'refresh_assessment',
        'assessment_id' => $this->assessment_id
      ];
    }

    if( $this->user_start_time === false && $request_type == 'start-assessment' ) {
      $this->set_user_status( WPEP_USER_ASSESSMENT_STATUS_PENDING );
      $this->set_user_start_time( time() );
    }

    $this->assessment_navigation_step = isset( $request_data[ 'current_step' ] ) ? intval( $request_data[ 'current_step' ] ) : 0;
    $this->assessment_is_reviewing_submission = isset( $request_data[ 'display_questions' ] ) ? $request_data[ 'display_questions' ] : false;

    if( isset( $request_data[ 'answers' ] ) && !empty( $request_data[ 'answers' ] ) ) {
      $allow_save_answers = wpep_allow_assessment_submission( $this->assessment_id, $this->user_status );

      if( $allow_save_answers && $this->assessment_time_limit != 0 && $this->user_start_time !== false )
        $allow_save_answers = ( intval( $this->user_start_time ) + $this->assessment_time_limit + 20 - time() >= 0 );

      if( $allow_save_answers )
        $this->save_answers( $request_data[ 'answers' ] );
    }


    if( $request_type == 'verify-assessment' ) {
      if( $this->user_start_time === false )
        $this->set_user_start_time( ( isset( $request_data[ 'user_start_time' ] ) && intval( $request_data[ 'user_start_time' ] ) < time() ? $request_data[ 'user_start_time' ] : time() ) );

      wpep_content_type_assessment_instance()->user->data->set(
        WPEP_USER_ASSESSMENT_SUBMISSION_TIME, time(), $this->assessment_id, 0, 0, $this->user_id
      );

      wpep_content_type_assessment_instance()->user->check_submission_answers( $this->assessment_id, get_current_user_id(), $this->assessment_type );

      $this->user_status = wpep_user_assessment_status( $this->assessment_id );

      return [
        'type'          => 'refresh_assessment',
        'assessment_id' => $this->assessment_id
      ];
    } elseif( $request_type == 'navigation-previous' ) {
      return [
        'type'          => 'refresh_assessment',
        'assessment_id' => $this->assessment_id,
        'step'          => $this->assessment_navigation_step - 1
      ];
    } elseif( $request_type == 'navigation-next' ) {
      return [
        'type'          => 'refresh_assessment',
        'assessment_id' => $this->assessment_id,
        'step'          => $this->assessment_navigation_step + 1
      ];
    }

    return [
      'type'          => 'view_assessment',
      'assessment_id' => $this->assessment_id
    ];
  }

  public function save_answers( $current_request_answers ) {
    if( $this->user_id === 0 )
      return;

    $internal_question_list = wpep_utilities_map_object( EntityAssessment::instance()->get_questions_by_assessment_id( $this->assessment_id, false, false ), 'id' );
    $internal_answer_list   = EntityAssessment::instance()->get_answers_by_assessment_id( $this->assessment_id );
    $question_type_data     = wpep_get_assessment_field_types();

    foreach( $current_request_answers as $current_request_question_id => $current_request_answer ) {
      if( !isset( $question_type_data[$internal_question_list[$current_request_question_id]->type] ) )
        continue;

      $current_request_answer = wpep_utilities_sanitize_input( $current_request_answer );

      $current_question_type_data = $question_type_data[$internal_question_list[$current_request_question_id]->type];

      if( $current_question_type_data['multiple_answers'] ) {
        wpep_content_type_assessment_instance()->user->data->delete( 'answer', $this->assessment_id, $current_request_question_id, false, $this->user_id );

        foreach( $current_request_answer as $current_question_answer ) {
          $answer_id  = $internal_answer_list[$current_question_answer]->id;

          wpep_content_type_assessment_instance()->user->data->add(
            'answer', $current_question_answer, $this->assessment_id,
            $current_request_question_id, $answer_id, $this->user_id
          );
        }
      } else if( $current_question_type_data['has_answers'] ) {

        if( isset( $internal_answer_list[$current_request_answer] ) ) {
          $answer_id = $internal_answer_list[$current_request_answer]->id;
          $current_request_answer = $internal_answer_list[$answer_id]->title;
        } else {
          $answer_id = 0;

          foreach( $internal_answer_list as $answer_item ) {
            if( $current_request_question_id != $answer_item->question_id )
              continue;

            if( strtolower( $answer_item->title ) != strtolower( $current_request_answer ) )
              continue;

            $answer_id = $answer_item->id;
          }
        }

        wpep_content_type_assessment_instance()->user->data->delete( 'answer', $this->assessment_id, $current_request_question_id, false, $this->user_id );
        wpep_content_type_assessment_instance()->user->data->set(
          'answer', $current_request_answer, $this->assessment_id,
          $current_request_question_id, $answer_id, $this->user_id
        );
      } else {
        wpep_content_type_assessment_instance()->user->data->set(
          'answer', $current_request_answer, $this->assessment_id,
          $current_request_question_id, 0, $this->user_id
        );
      }
    }
  }

  public function display_content() {
    $content = $this->get_content();

    $content = wpep_format_content_autoembed( $content );

    echo $content;
  }

  public function get_content() {
    if( $this->user_start_time === false
        && $this->assessment_type !== WPEP_ASSESSMENT_TYPE_ASSIGNMENT
        && intval( get_post_meta( $this->assessment_id, 'wpep_enable_welcome_page', true ) ) == 1 ) {
      return WPEP_Controller::instance()->template->get_template( 'assessment/welcome.php', [
        'display_return_button'=> true,
        'assessment_id'        => $this->assessment_id,
        'assessment_title'     => get_the_title( $this->assessment_id ),
        'allowed_time'         => $this->assessment_time_limit,
        'return_button_url'    => $this->get_return_button_link( 'welcome' ),
        'return_button_text'   => $this->get_return_button_text( 'welcome' ),
        'question_count'       => count( EntityAssessment::instance()->get_questions_by_assessment_id( $this->assessment_id, false, false ) ),
        'score_requirement'    => ( $this->assessment_type === WPEP_ASSESSMENT_TYPE_STANDARD ? ( in_array( get_post_meta( $this->assessment_id, 'wpep_grading_option', true ), [ 2,3 ] )
          ? intval( get_post_meta( $this->assessment_id, 'wpep_grading_requirement', true ) )
          : 0
        ) : 0 )
      ]);
    } // return_button_url

    $notification_template = ( $this->user_status == WPEP_USER_ASSESSMENT_STATUS_COMPLETED ? 'assessment/success.php' : ( $this->user_status == WPEP_USER_ASSESSMENT_STATUS_FAILED ? 'assessment/failed.php' : false ) );

    if( $notification_template !== false ) {
      if( $this->assessment_type === WPEP_ASSESSMENT_TYPE_ASSIGNMENT )
        return $this->get_form_content();

      $is_viewing_questions  = ( get_post_meta( $this->assessment_id, 'wpep_enable_view_questions', true ) ? ( $this->assessment_navigation_step != 0 || $this->assessment_is_reviewing_submission ) : false  );

      if( $is_viewing_questions )
        return $this->get_form_content();
    }

    if( $notification_template !== false )
      return WPEP_Controller::instance()->template->get_template( $notification_template, $this->_get_message_template_data( $this->user_status == WPEP_USER_ASSESSMENT_STATUS_COMPLETED ? 'success' : 'failed' ) );

    if( $this->user_start_time === false )
      $this->set_user_start_time( time() );

    return $this->get_form_content();
  }

  private function _get_message_template_data( $context = 'success' ) {
    $assessment_questions = EntityAssessment::instance()->get_questions_by_assessment_id( $this->assessment_id, false, false );

    $failed_message = get_post_meta( $this->assessment_id, 'wpep_failed_message', true );

    if( $failed_message == '' )
      $failed_message = wpep_get_setting_translated( 'assessment-status-failed' );

    $template_information = [
      'course_id'            => $this->course_id,
      'return_button_url'    => $this->get_return_button_link( $context ),
      'return_button_text'   => $this->get_return_button_text( $context ),
      'assessment_id'               => $this->assessment_id,
      'assessment_type'             => $this->assessment_type,
      'assessment_title'            => get_the_title( $this->assessment_id ),
      'assessment_view_permalink'   => add_query_arg( 'display-questions', 1, get_permalink( $this->assessment_id ) ),
      'assessment_retake_permalink' => wp_nonce_url( get_permalink( $this->assessment_id ), 'retake_assessment', 'retake_assessment' ),
      'user_status'               => $this->user_status,
      'failed_message'            => $failed_message,
      'success_message'           => get_post_meta( $this->assessment_id, 'wpep_success_message', true ),
      'enable_view_questions'     => ( get_post_meta( $this->assessment_id, 'wpep_enable_view_questions', true ) ),
      'allow_submission'          => wpep_allow_assessment_submission( $this->assessment_id, $this->user_status ),
      'question_count'            => count( $assessment_questions ),
      'grading_option'            => 0,
      'success_visual_indicators' => false,
      'failed_visual_indicators'  => false,
      'success_show_title'        => false,
      'failed_show_title'         => false,
      'display_score'             => false,
      'score'                     => 0,
      'correct_answers'           => 0
    ];

    if( $this->assessment_type !== WPEP_ASSESSMENT_TYPE_ASSIGNMENT ) {
      $template_information[ 'success_show_title' ] = ( intval( get_post_meta( $this->assessment_id, 'wpep_success_message_hide_title', true ) ) ? false : true );
      $template_information[ 'failed_show_title' ]  = ( intval( get_post_meta( $this->assessment_id, 'wpep_failed_message_hide_title', true ) ) ? false : true );
      $template_information[ 'display_score' ]      = ( intval( get_post_meta( $this->assessment_id, 'disable_score_display', true ) ) ? false : true );
      $template_information[ 'score' ]              = wpep_content_type_assessment_instance()->user->data->get_value(
        WPEP_USER_ASSESSMENT_SCORE,
        $this->assessment_id,
        false,
        false,
        $this->user_id
      );
    }

    if( $this->assessment_type === WPEP_ASSESSMENT_TYPE_STANDARD ) {
      $template_information[ 'correct_answers' ] = count( wpep_content_type_assessment_instance()->user->data->get(
        'is_correct',
        $this->assessment_id,
        false,
        false,
        $this->user_id,
        false,
        1
      ) );

      $grading_option = intval( get_post_meta( $this->assessment_id, 'wpep_grading_option', true ) );

      if( $grading_option != 0 ) {
        $template_information[ 'grading_option' ]            = $grading_option;
        $template_information[ 'success_visual_indicators' ] = !get_post_meta( $this->assessment_id, 'wpep_success_message_disable_indicators', true );
        $template_information[ 'failed_visual_indicators' ]  = !get_post_meta( $this->assessment_id, 'wpep_failed_message_disable_indicators', true );
      }
    }

    return $template_information;
  }

  private function get_return_button_text( $context = 'welcome' ) {
    if( isset( $this->_request_settings[ 'return_button_text' ][ $context ] ) )
      return htmlentities( $this->_request_settings[ 'return_button_text' ][ $context ] );

    if( $context === 'welcome' )
      return wpep_get_setting_translated( 'assessment-return-to-course' );

    return wpep_get_setting_translated( 'assessment-go-to-course' );
  }

  private function get_return_button_link( $context = 'welcome' ) {
    if( isset( $this->_request_settings[ 'return_button_url' ][ $context ] ) )
      return esc_url( $this->_request_settings[ 'return_button_url' ][ $context ] );

    $response = '';
    
    if( $this->course_id !== 0 ) {
      $response = apply_filters( 'wpep_primary_content_item_link', wpep_course_permalink( $this->course_id ), $this->course_id );

      if( $response == '' || $response == '#' )
        $response = wpep_course_permalink( $this->course_id );
    }

    return $response;
  }

  public function get_form_content() {
    $allow_submission = wpep_allow_assessment_submission( $this->assessment_id, $this->user_status );

    if( $allow_submission && $this->assessment_time_limit != 0 && ( $this->user_start_time + $this->assessment_time_limit < time() ) )
      $this->set_user_start_time( time() );

    $assessment_questions = EntityAssessment::instance()->get_questions_by_assessment_id( $this->assessment_id, true, true );

    $questions_per_page = ( $this->user_id === 0 ? 0 : intval( get_post_meta( $this->assessment_id, 'wpep_paginate_questions', true ) ) );
    $user_answers = ( $this->user_id === 0 ? [] : wpep_content_type_assessment_instance()->user->data->get(
      'answer', $this->assessment_id, false, false, $this->user_id, false
    ) );

    $template = $questions_per_page != 0 ? 'assessment/form-paginated.php' : 'assessment/form.php';
    $template_data = [
      'assessment_id'          => $this->assessment_id,
      'assessment_type'        => $this->assessment_type,
      'assessment_title'       => get_the_title( $this->assessment_id ),
      'assessment_description' => get_post_meta( $this->assessment_id, 'wpep_assessment_description', true ),
      'form_action'            => add_query_arg( 'display-questions', 1, get_the_permalink( $this->assessment_id ) ),
      'assessment_questions'   => $assessment_questions,
      'allowed_time'           => $this->assessment_time_limit,
      'submission_started'     => $this->user_start_time,
      'course_id'              => $this->course_id,
      'return_button_url'      => $this->get_return_button_link( 'form' ),
      'return_button_text'     => $this->get_return_button_text( 'form' ),
      'section_id'             => $this->section_id,
      'user_status'            => $this->user_status,
      'allow_submission'       => $allow_submission,
      'field_types_data'       => wpep_get_assessment_field_types(),
      'grading_option'         => ( $this->assessment_type === WPEP_ASSESSMENT_TYPE_ASSIGNMENT ? WPEP_ASSESSMENT_GRADING_MANUAL : intval( get_post_meta( $this->assessment_id, 'wpep_grading_option', true ) ) ),
      'allow_answers_save'     => ( $this->user_id === 0 ? false : ( intval( get_post_meta( $this->assessment_id, 'disable_answers_save', true ) ) ? false : true ) ),
      'user_question_status'   => ( $this->assessment_type === WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING ? [] : (
        $this->user_id === 0 ? [] : wpep_utility_map_to_array_assoc(
          wpep_content_type_assessment_instance()->user->data->get(
            'is_correct', $this->assessment_id, false, false, $this->user_id, false
          ),
          'question_id',
          'value'
        )
      ) ),
      'user_question_answer' => wpep_utility_map_to_array_assoc(
        $user_answers,
        'question_id',
        'value',
        true
      ),
      'user_question_answer_id' => wpep_utility_map_to_array_assoc(
        $user_answers,
        'question_id',
        'question_answer_id',
        true
      )
    ];


    if( $questions_per_page != 0 ) {
      $offset = ( $this->assessment_navigation_step == 0 ? 0 : $this->assessment_navigation_step * $questions_per_page );

      $template_data['assessment_questions'] = array_slice( $assessment_questions, $offset, $questions_per_page );

      $i = 1; foreach( $template_data['assessment_questions'] as $index => $question ) {
        $template_data['assessment_questions'][$index]->question_number = $offset + $i;
        $i++;
      }

      $template_data['form_action']         = add_query_arg( 'step', $this->assessment_navigation_step, $template_data['form_action'] );
      $template_data['pagination_step']     = $this->assessment_navigation_step;
      $template_data['pagination_previous'] = ( $offset > 0 );
      $template_data['pagination_next']     = ( $questions_per_page + $offset < count( $assessment_questions ) );
    }

    return WPEP_Controller::instance()->template->get_template( $template, $template_data );
  }

}
