<?php

namespace WPEP\ContentType;

use WPEP\API as WPEP_API;
use WPEP\ContentType\Course\Container\Library as Course_Library;
use WPEP\ContentType\Course\Container\Data as Course_Data;

use WPEP\ACL\User as WPEP_ACL_User;

class Course extends Skeleton\Library {

  public $data_post_ignored_meta = [ '_edit_lock', '_edit_last', '_oembed_*', 'wpep_prerequisites' ];

  use Course_Library,
      Course_Data;

  /**
   * @var Course\Admin
   */
  public $admin;

  /**
   * @var Course\Block
   */
  public $block;

  /**
   * @var Course\Delivery
   */
  public $delivery;

  /**
   * @var Course\Events
   */
  public $events;

  /**
   * @var Course\PreAuth
   */
  public $preAuth;

  /**
   * @var Course\Privacy
   */
  public $privacy;

  /**
   * @var Course\User
   */
  public $user;

  /**
   * @var Course\Integration\Discussion
   */
  public $discussionIntegration;

  /**
   * @var Course\Container\PostDependency[]
   */
  protected $_post_dependencies = [];
  protected $_current_directory = '';

  /**
   * @var \WPEP\ContentType\Course\Integration\Builder
   */
  protected $_builder_instance;

  public function setup() {
    $this->_current_directory = dirname( __FILE__ );

    require_once( $this->_current_directory . '/Course/_inc/definitions.php' );
    require_once( $this->_current_directory . '/Course/_inc/functions.php' );

    $this->admin       = new Course\Admin();
    $this->block       = new Course\Block();
    $this->delivery    = new Course\Delivery();
    $this->events      = new Course\Events();
    $this->preAuth     = new Course\PreAuth();
    $this->privacy     = new Course\Privacy();
    $this->user        = new Course\User();

    if( wpep_get_setting_flag( 'enable-course-discussions-option' ) )
      $this->discussionIntegration = new Course\Integration\Discussion();
    else
      $this->discussionIntegration = null;

    add_filter( 'wpep_data_system_third_party_import_instances', [ $this, '_register_third_party_import' ] );
    add_filter( 'wpep_content_user_profile_tab_list', [ $this, '_attach_user_profile_tabs' ], 5, 2 );

    $this->_register_api_methods();
  }

  public function init() {
    if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_ASSESSMENT ) )
      $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ] = wpep_content_type()->standardContentTypes[ WPEP_POST_TYPE_ASSESSMENT ];

    if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_CERTIFICATE ) )
      $this->_post_dependencies[ WPEP_POST_TYPE_CERTIFICATE ] = wpep_content_type()->standardContentTypes[ WPEP_POST_TYPE_CERTIFICATE ];


    $this->preAuth->init();
    $this->admin->init();
  }

  /**
   * @return Course\Integration\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new Course\Integration\LearningJourney( $this );
  }

  /**
   * @return \WPEP\ContentType\Course\Integration\Builder
   */
  public function get_builder_instance() {
    if( $this->_builder_instance === null ) {
      $this->_builder_instance = new Course\Integration\Builder( $this );

      $this->_builder_instance->init();
    }

    return $this->_builder_instance;
  }

  public function get_post_supports() {
    return [ 'title', "custom-fields" ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_COURSE;
  }

  public function get_singular_name() {
    return __( 'Course', "wpep" );
  }

  public function get_plural_name() {
    return __( 'Courses', "wpep" );
  }

  public function get_label_all_items() {
    return __( 'All Courses', "wpep" );
  }

  public function get_index_page_settings_field_map() {
    return [
      'general' => [
        'name'      => __( "General Options", "wpep" ),
        'has_title' => false,
        'alias'     => 'general',
        'fields'    => [
          'index-page-courses-per-row'  => [
            'name'    =>  __( 'Courses Per Row', "wpep" ),
            'alias'   => 'index-page-courses-per-row',
            'type'    => 'select',
            'options' => wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE )
          ],
          'index-page-enable-hover-image'  => [
            'name'        =>  __( "Enable Image Hover", "wpep" ),
            'alias'       => 'index-page-enable-hover-image',
            'type'        => 'checkbox',
            'description' => __( "Enable a different image on the presentation card on the Index Page on hover.", "wpep" )
          ],
          'index-page-progress-bars'  => [
            'name'        =>  __( 'Progress Bars', "wpep" ),
            'alias'       => 'index-page-progress-bars',
            'type'        => 'checkbox',
            'description' => __( "Display Progress Bars under each course to hint the student which courses are completed.", "wpep" )
          ],
        ]
      ],
      'button' => [
        'name'      => __( "Button", "wpep" ),
        'has_title' => true,
        'alias'     => 'button',
        'fields'    => [
          'opt-course-start-text'  => [
            'name'        =>  __('Start Text', "wpep"),
            'alias'       => 'opt-course-start-text',
            'type'        => 'text_translated',
            'description' => __( 'Personalize the Course Start Button Text', "wpep" )
          ],
          'opt-course-continue-text'  => [
            'name'        =>  __('Continue Text', "wpep"),
            'alias'       => 'opt-course-continue-text',
            'type'        => 'text_translated',
            'description' => __( 'Personalize the Course Continue Button Text', "wpep" )
          ],
          'opt-course-coming-soon'  => [
            'name'        =>  __('Coming Soon', "wpep"),
            'alias'       => 'opt-course-coming-soon',
            'type'        => 'text_translated',
            'description' => __( 'Personalize the Coming Soon Button Text, when an Course is going to be available soon.', "wpep" )
          ],
          'index-page-start-text-color'  => [
            'name'        =>  __('Text Colour', "wpep"),
            'alias'       => 'index-page-start-text-color',
            'type'        => 'color',
            'description' => __( 'Personalize the Button Text Colour', "wpep" )
          ],
          'index-page-start-text-color-background'  => [
            'name'        =>  __('Text Background Colour', "wpep"),
            'alias'       => 'index-page-start-text-color-background',
            'type'        => 'color',
            'description' => __( 'Personalize the Button Text Background Colour', "wpep" )
          ],
        ]
      ],
      'ribbon' => [
        'name'      => __( "Ribbon", "wpep" ),
        'has_title' => true,
        'alias'     => 'ribbon',
        'fields'    => [
          'index-page-ribbon-text-color'  => [
            'name'        =>  __('Text Colour', "wpep"),
            'alias'       => 'index-page-ribbon-text-color',
            'type'        => 'color',
            'description' => __( 'Personalize the Ribbon Text Colour', "wpep" )
          ],
          'index-page-ribbon-text-color-background'  => [
            'name'        =>   __('Background Colour', "wpep"),
            'alias'       => 'index-page-ribbon-text-color-background',
            'type'        => 'color',
            'description' => __( 'Personalize the Ribbon  Background Colour', "wpep" )
          ],
        ]
      ],
    ];
  }

  public function get_settings_defaults() {
    return [
      'opt-course-progressbar'            => 'The Completion Level of Your Training.',
      'opt-course-percentage-bar'         => 1,
      'opt-course-percentage-bar-number'  => 1,
      'show-lesson-continue-button'       => 0,
      'show-lesson-hover-color'           => 0,
      'lesson-learning-path-placeholder'  => 'Select Learning Path',
      'lesson-quick-continue-button-text' => 'Next',
      'lesson-quick-continue-final-button-text' => 'Complete Course',
      'lesson-continue-button-position'   => 'default',

      'prerequisite-suggested-notification'  => 'In order to start this Course you need to go thru %s first.',
      'prerequisite-required-notification'   => 'In order to start this Course you need to complete the following %s first.',
      'prerequisite-next-courses-title'      => "What's next ?",
      'prerequisite-course-button-lock'      => 0,
      'prerequisite-course-button-lock-text'    => "Not Available",

      'course-coming-soon-student-notification' => '[' . WPEP_SHORT_CODE_PREFIX . 'notification type="danger"]This course is not available yet.[/' . WPEP_SHORT_CODE_PREFIX . 'notification]',
      'course-downloads-template'            => "standard",
      'course-downloads-position'            => 'bottom',
      'course-downloads-list-title'          => "Available Deliverables",
      'course-downloads-list-download'       => "Download",
      'course-use-the-content-filter'        => 1,
      'course-pre-authentication-tracking'   => 1,
      'course-resize-wistia-embeds'          => 0,
      'course-standard-allow-index-page'        => 0,
      'course-standard-allow-index-page-widgets'=> 1,
      'course-standard-redirect-on-completion'  => 0,
      'course-standard-section'         => 'Section',
      'course-standard-lesson'          => 'Lesson',
      'course-standard-start-lesson'    => 'Start Lesson',
      'course-standard-section-list'    => 'Section List : ',
      'course-standard-course-progress' => 'Course Progress',
      'course-standard-course-navigation' => 'Course Navigation',
      'course-standard-lesson-completed'  => 'This lesson has been completed.',
      'course-standard-complete-lesson'   => 'Complete Lesson',
      'course-standard-previous-lesson'   => 'Previous Lesson',
      'course-standard-next-lesson'       => 'Next Lesson',
      'course-video-allow-index-page'        => 1,
      'course-video-redirect-on-completion'  => 0,
      'course-video-course-status'         => 'Course Status',
      'course-video-course-status-pending'  => 'Pending',
      'course-video-course-status-progress' => 'Progressing',
      'course-video-course-status-completed'=> 'Completed',
      'course-video-course-table-sections'  => 'Sections',
      'course-video-course-table-status'  => 'Status',
      'course-video-lesson-back'          => 'Back to Overview',
      'course-video-lesson-description'   => 'Available Downloads',
      'course-video-lesson-description-icon'   => 'dashicons dashicons-download',
      'course-video-lesson-completed'     => 'Video has been watched.',
      'course-video-lesson-complete'      => "Mark as completed.",
      'course-video-previous-lesson'      => 'Previous Video',
      'course-video-next-lesson'          => 'Next Video',
      'course-video-course-progress'      => 'Course Progress',
      'course-video-start-section'        => 'Start Section',
      'course-focus-percentage-text'      => 'Complete',
      'course-focus-overview-start'       => 'Start',
      'course-focus-overview-view'        => 'View',
      'course-focus-overview-assessment'  => 'View',
      'course-focus-previous-lesson'      => 'Previous Lesson',
      'course-focus-next-lesson'          => 'Next Lesson',
      'course-focus-complete-and-next-lesson' => 'Complete and Continue',
      'course-focus-complete-lesson'      => 'Complete',
      'course-focus-discussion-title'     => 'Discussion',
      'course-basic-enable-video-description' => 0,
      'course-basic-complete-course'          => 'Complete Course',
      'course-basic-course-completed'         => 'Course Completed',
      'course-basic-jump-link-title'          => 'Video Index',
      'course-challenge-introduction-text'       => "Your objectives for this module:",
      'course-challenge-section-slug'            => 'day-id',
      'course-challenge-social-share-status'   => 1,
      'course-challenge-social-share-cta'      => "<p>USE THE SOCIAL LINKS BELOW TO SHARE</p><p>THE CHALLENGE WITH YOUR FRIENDS!</p>",
      'course-challenge-social-share-networks' => [
        'facebook', 'twitter', 'pinterest'
      ],

      'course-scorm-button-open'                => 'Start Training',
      'course-scorm-notification-completed'     => 'This course has been completed !',

      'enable-course-deliverables'              => 1,
      'enable-course-comments'                  => 0,
      'enable-course-assessments'               => 0,
      'enable-course-bookmarks'                 => 0,
      'enable-course-timing-option'             => 0,
      'enable-course-time-spent-option'         => 0,
      'enable-course-discussions-option'        => 1,
      'disable-course-discussion-access-verification' => 0,
      'enable-course-on-boarding-options'       => 0,
      'enable-lesson-incomplete-option'         => 0,
      'enable-course-tracking-code-option'      => 0,

      'enable-progress-repeat-support'  => 0,
      'enable-certifications-support'   => 0,

      'progress-repeat-button'          => "Repeat Course",
      'progress-repeat-headline'        => "Course Repeat History",

      'course-download-button'         => 'Download as PDF',

      'index-page-courses-per-row'     => 3,
      'index-page-enable-hover-image'  => 0,
      'index-page-progress-bars'       => 0,
      'opt-course-start-text'          => 'START',
      'opt-course-continue-text'       => 'Continue',
      'opt-course-coming-soon'         => 'Coming Soon',
      'index-page-start-text-color'              => '',
      'index-page-start-text-color-background'   => '',
      'index-page-ribbon-text-color'             => '',
      'index-page-ribbon-text-color-background'  => '',
    ];
  }

  public function standard_get_post_meta_fields() {
    return $this->admin->get_meta_fields();
  }

  public function standard_get_post_meta_fields_context() {
    return 'side';
  }

  /**
   * @return mixed
   */
  public function get_cap_name() {
    return WPEP_POST_CAP_COURSE;
  }

  /**
   * @return Course\Container\PostDependency[]
   */
  public function get_dependent_content_types() {
    return $this->_post_dependencies;
  }

  /**
   * @param $instances
   * @return mixed
   */
  public function _register_third_party_import( $instances ) {
    if( !is_array( $instances ) )
      $instances = [];

    foreach( [ new Course\ThirdPartyImport\LearnDash(),
               new Course\ThirdPartyImport\LearnPress(),
               new Course\ThirdPartyImport\LifterLMS(),
               new Course\ThirdPartyImport\ProgressAlly(),
               new Course\ThirdPartyImport\Sensei(),
               new Course\ThirdPartyImport\ThriveApprentice(),
               new Course\ThirdPartyImport\WPCourseWare(),
               new Course\ThirdPartyImport\WPLMS()
             ] as $importer_instance )
      $instances[ $importer_instance->get_alias() ] = $importer_instance;

    return $instances;
  }

  public function _attach_user_profile_tabs( $tab_list, $user_id ) {
    if( !wpep_get_setting_flag( 'user-profile-tab-default-tabs-enabled' ) )
      return $tab_list;

    if( $user_id === 0 )
      return $tab_list + [
          'progressing_course_list' => [
            'title'   => wpep_get_setting_translated( 'user-profile-tab-progressing-courses' ),
            'content' => '',
          ],
          'completed_course_list' => [
            'title'   => wpep_get_setting_translated( 'user-profile-tab-completed-courses' ),
            'content' => '',
          ],
        ] + ( wpep_course_has_bookmarks() ? [
          'bookmarked_course_list' => [
            'title'   => wpep_get_setting_translated( 'user-profile-tab-bookmarked-courses' ),
            'content' => '',
          ]
        ] : [] ); //

    if( !WPEP_ACL_User::analytics( $user_id, 'view' ) )
      return $tab_list;

    $progressing_course_list = wpep_content_type_course_instance()->user->get_progressing_course_list( $user_id );
    $completed_course_list   = wpep_content_type_course_instance()->user->get_completed_course_list( $user_id );

    if( !empty( $progressing_course_list ) )
      $tab_list['progressing_course_list'] = [
        'title'         => wpep_get_setting_translated( 'user-profile-tab-progressing-courses' ),
        'entries_count' => count( $progressing_course_list ),
        'content'       => [
          'type'        => 'xhr',
          'action'      => 'wpep/content-type/courses/template-user-progressing-content',
          'action_args' => [
            'user_id' => $user_id
          ]
        ]
      ];

    if( !empty( $completed_course_list ) )
      $tab_list['completed_course_list'] = [
        'title'         => wpep_get_setting_translated( 'user-profile-tab-completed-courses' ),
        'entries_count' => count( $completed_course_list ),
        'content'       => [
          'type'        => 'xhr',
          'action'      => 'wpep/content-type/courses/template-user-completed-content',
          'action_args' => [
            'user_id' => $user_id
          ]
        ]
      ];

    if( wpep_course_has_bookmarks() ) {
      $bookmarked_course_list   = wpep_content_type_course_instance()->user->get_bookmarked_course_list( $user_id, 1 );

      if( !empty( $bookmarked_course_list ) )
        $tab_list['bookmarked_course_list'] = [
          'title'         => wpep_get_setting_translated( 'user-profile-tab-bookmarked-courses' ),
          'entries_count' => count( $bookmarked_course_list ),
          'content'       => [
            'type'        => 'xhr',
            'action'      => 'wpep/content-type/courses/template-user-bookmarked-content',
            'action_args' => [
              'user_id' => $user_id
            ]
          ]
        ];
    }

    return $tab_list;
  }

  private function _register_api_methods() {
    add_filter( 'wpep_api_request_response_system/statistics', [ $this, '_api_response_system_statistics' ], 50, 3 );
    add_filter( 'wpep_api_request_response_system/statistics-current-user', [ $this, '_api_response_system_statistics_current_user' ], 50, 3 );

    $method_map = [
      'lesson-content'             => [
        'callback' => '\WPEP\ContentType\Course\API::lesson_content',
        'type'    => WPEP_API_METHOD_TYPE_PUBLIC
      ],
      'user-flag-started-at'       => '\WPEP\ContentType\Course\API::user_flag_started_at',

      'user-progress-repeat'       => '\WPEP\ContentType\Course\API::user_progress_repeat',

      'user-lesson-flag-completed' => [
        'callback' => '\WPEP\ContentType\Course\API::user_lesson_flag_completed',
        'type'    => wpep_get_setting_flag( 'course-pre-authentication-tracking' ) ? WPEP_API_METHOD_TYPE_PUBLIC : WPEP_API_METHOD_TYPE_PRIVATE
      ],
      'user-section-learning-path' => '\WPEP\ContentType\Course\API::user_section_learning_path',
      'admin-test-on-boarding'     => '\WPEP\ContentType\Course\API::admin_test_on_boarding',
      'pdf-download-data' => [
        'callback' => '\WPEP\ContentType\Course\API::pdf_download_data',
        'type'     => WPEP_API_METHOD_TYPE_PUBLIC
      ],
      'template-user-progressing-content' => [
        'callback' => '\WPEP\ContentType\Course\API::template_user_progressing_content',
        'type'     => WPEP_API_METHOD_TYPE_PUBLIC
      ],
      'template-user-completed-content' => [
        'callback' => '\WPEP\ContentType\Course\API::template_user_completed_content',
        'type'     => WPEP_API_METHOD_TYPE_PUBLIC
      ],
      'scorm-configuration' => [
        'callback' => '\WPEP\ContentType\Course\API::get_scorm_configuration',
        'type'     => WPEP_API_METHOD_TYPE_PUBLIC
      ],
      'scorm-set-user-information' => '\WPEP\ContentType\Course\API::set_scorm_user_information',
      'user-recommendation-list' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_user_id',
        'callback'  => '\WPEP\ContentType\Course\API::user_recommendation_list'
      ],
      'completed-lesson-list' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::completed_lessons_list'
      ],
      'progress-repeat-list' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::progress_repeat_list'
      ],
      'completed-lesson-report' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::completed_lessons_report'
      ],
      'section-completed-lesson-report' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::section_completed_lessons_report'
      ],
      'course-completed-lesson-report' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::course_completed_lessons_report'
      ],
      'category-completed-lesson-report' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::category_completed_lessons_report'
      ],
      'user-completed-lesson-report' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::user_completed_lessons_report'
      ],
      'bird-eye-view-list' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::bird_eye_view_list'
      ],
      'bird-eye-view-report' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_content_type',
        'callback'  => '\WPEP\ContentType\Course\API::bird_eye_view_report'
      ],
      'user-id-post-id-data-view-plain' => [
        'user_can'  => '\WPEP\ACL\API::analytics_view_user_id_post_id',
        'callback'  => '\WPEP\ContentType\Course\API::user_id_post_id_data_view_plain'
      ]
    ];

    if( wpep_course_has_bookmarks() )
      $method_map[ 'user-bookmark-status' ] = '\WPEP\ContentType\Course\API::user_flag_bookmark';

    if( wpep_get_setting_flag( 'enable-course-time-spent-option' ) )
      $method_map[ 'user-time-spent-tracking' ] = '\WPEP\ContentType\Course\API::user_time_spent_tracking';

    if( wpep_course_has_bookmarks() )
      $method_map[ 'template-user-bookmarked-content' ] = [
        'callback' => '\WPEP\ContentType\Course\API::template_user_bookmarked_content',
        'type'     => WPEP_API_METHOD_TYPE_PUBLIC
      ];

    WPEP_API::instance()->register_content_type( $this->get_post_type(), $method_map );
  }

  public function _api_response_system_statistics( $response, $action, $request_data ) {
    if( !in_array( $action, [ 'configuration-report', 'user-id-configuration-report', 'post-id-configuration-report', 'user-id-post-id-configuration-report' ] ) )
      return $response;

    if( !isset( $response[ 'report_data_sources' ] ) )
      return $response;

    if( $action === 'user-id-post-id-configuration-report' ) {
      if( \get_post_type( $request_data[ 'post_id' ] ) !== WPEP_POST_TYPE_COURSE )
        return $response;

      $attach_report = [
        'entity-course-user-view' => [
          'title'            => __( "Report", "wpep" ),
          'type'             => 'data-view-plain',
          'data_action'      => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/user-id-post-id-data-view-plain',
          'data_action_args' => [
            'user_id' => $request_data[ 'user_id' ],
            'post_id' => $request_data[ 'post_id' ]
          ]
        ]
      ];

      if( wpep_course_type_has_lessons( wpep_course_type_from_id( $request_data[ 'post_id' ] ) ) ) {
        $attach_report['course-completed-lesson-list'] = [
          'title'            => __( "Course Completed Lessons", "wpep" ),
          'type'             => 'data-view-list',
          'data_action'      => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/completed-lesson-list',
          'data_action_args' => [
            'user_id' => $request_data[ 'user_id' ],
            'post_id' => $request_data[ 'post_id' ]
          ]
        ];
      }

      $response[ 'report_data_sources' ] = $attach_report + $response[ 'report_data_sources' ];

      return $response;
    }

    $attach_report = [
      'course-completed-lesson-list' => [
        'title'       => __( "Course Completed Lessons", "wpep" ),
        'type'        => 'data-view-list',
        'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/completed-lesson-list',
      ],
      'course-progress-repeat-list' => [
        'title'       => __( "Course Progress Repeat", "wpep" ),
        'type'        => 'data-view-list',
        'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/progress-repeat-list',
      ]
    ];

    if( isset( $request_data[ 'user_ids' ] ) && !empty( $request_data[ 'user_ids' ] ) )
      $attach_report[ 'course-completed-lesson-list' ][ 'data_action_args' ] = [
        'user_ids'  => $request_data[ 'user_ids' ]
      ];

    if( $action === 'post-id-configuration-report' ) {
      if( !isset( $request_data[ 'post_id' ] ) )
        return $response;

      if( get_post_type( $request_data[ 'post_id' ] ) !== WPEP_POST_TYPE_COURSE )
        return $response;

      if( !wpep_course_type_has_lessons( wpep_course_type_from_id( $request_data[ 'post_id' ] ) ) )
        return $response;

      $attach_report['course-completed-lesson-list'][ 'data_action_args' ] = [
        'post_id' => intval( $request_data[ 'post_id' ] )
      ];

      $attach_report['course-bird-eye-view-report'] = [
        'title'            => __( "Course Bird Eye Report", "wpep" ),
        'type'             => 'data-view-analyze',
        'data_action'      => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/bird-eye-view-report',
        'data_action_args' => [
          'post_id' => $request_data[ 'post_id' ]
        ]
      ];

      $attach_report['course-bird-eye-view-list'] = [
        'title'            => __( "Course Bird Eye View", "wpep" ),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/bird-eye-view-list',
        'data_action_args' => [
          'post_id' => $request_data[ 'post_id' ]
        ]
      ];
    }

    if( $action === 'user-id-configuration-report' ) {
      if( !isset( $request_data[ 'user_id' ] ) )
        return $response;

      $attach_report['course-user-recommendation-list'] = [
        'title'            => __( "Recommended Courses - Raw", "wpep" ),
        'type'             => 'data-view-list',
        'data_action'      => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/user-recommendation-list',
        'data_action_args' => [
          'user_id' => $request_data[ 'user_id' ]
        ]
      ];

      $attach_report['course-completed-lesson-list'][ 'data_action_args' ] = [
        'user_id' => intval( $request_data[ 'user_id' ] )
      ];
    }

    if( isset( $response[ 'report_data_sources' ][ 'activity-list' ] ) )
      $response[ 'report_data_sources' ][ 'activity-list' ][ 'group' ] += $attach_report;
    else
      $response[ 'report_data_sources' ] += $attach_report;

    if( 'configuration-report' === $action ) {
      $response[ 'report_data_sources' ][ 'course_lesson_report_directory' ] = [
        'title'       => __( "Lesson Completed Metrics", "wpep" ),
        'type'        => 'group',
        'group'       => [
          'completed-lesson-report' => [
            'title'       => __( "Group By Lesson", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/completed-lesson-report',
          ],
          'section-completed-lesson-report' => [
            'title'       => __( "Group By Section", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/section-completed-lesson-report',
          ],
          'course-completed-lesson-report' => [
            'title'       => __( "Group By Course", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/course-completed-lesson-report',
          ],
          'category-completed-lesson-report' => [
            'title'       => __( "Group By Category", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/category-completed-lesson-report',
          ],
          'top-category-completed-lesson-report' => [
            'title'       => __( "Group By Top Category", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/category-completed-lesson-report',
            'data_action_args' => [
              'term_parent_id' => 0
            ]
          ],
          'user-completed-lesson-report' => [
            'title'       => __( "Group By User", "wpep" ),
            'type'        => 'data-view-analyze',
            'data_action' => 'wpep/content-type/' . WPEP_POST_TYPE_COURSE . '/user-completed-lesson-report',
          ],
        ]
      ];

      if( isset( $request_data[ 'user_ids' ] ) && !empty( $request_data[ 'user_ids' ] ) ) {
        foreach( $response[ 'report_data_sources' ][ 'course_lesson_report_directory' ][ 'group' ] as $group_item_index => $group_item ) {
          if( !isset( $response[ 'report_data_sources' ][ 'course_lesson_report_directory' ][ 'group' ][$group_item_index][ 'data_action_args' ] ) )
            $response[ 'report_data_sources' ][ 'course_lesson_report_directory' ][ 'group' ][$group_item_index][ 'data_action_args' ] = [];

          $response[ 'report_data_sources' ][ 'course_lesson_report_directory' ][ 'group' ][$group_item_index][ 'data_action_args' ][ 'user_ids' ] = $request_data[ 'user_ids' ];
        }
      }
    }

    return $response;
  }

  public function _api_response_system_statistics_current_user( $response, $action, $request_data ) {
    if( 'configuration-report' !== $action )
      return $response;

    if( !is_array( $response ) || !isset( $response[ 'user_ids' ] ) )
      return $response;

    $request_data[ 'user_ids' ] = $response[ 'user_ids' ];

    return $this->_api_response_system_statistics( $response, $action, $request_data );
  }

  /**
   * @param \WP_Post|int|bool $post
   * @return string
   */
  public function get_post_permalink( $post ) {
    return wpep_course_permalink( $post );
  }

}