<?php

namespace WPEP\ContentType\Course;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as WPEP_Entity_Course;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\Entity\CourseType\PDF as WPEP_Entity_CourseType_PDF;

use WPEP\API\Utils\User as API_Utils_User;
use WPEP\API\Utils\Report as API_Utils_Report;
use WPEP\API\Utils\Post as API_Utils_Post;

use WPEP\ACL\User as WPEP_ACL_User;

class API {

  public static function lesson_content( $request_data ) {
    if( !isset( $request_data['lesson_id'] ) )
      return false;

    $lesson_id = intval( $request_data[ 'lesson_id' ] );
    $course_id = intval( WPEP_Entity_Course::instance()->get_course_id_by_lesson_id( $lesson_id ) );

    if( !WPEP_Controller::instance()->contentLibrary->has_access_to_post( $course_id ) )
      return false;

    $lesson = WPEP_Entity_Course::instance()->get_lesson( $lesson_id );

    return [
      'status'         => 'ok',
      'lesson_content' => wpep_format_content_autoembed( wpep_format_content( $lesson->content ) )
    ];
  }

  public static function user_flag_started_at( $request_data ) {
    if( !isset( $request_data['course_id'] )  )
      return false;

    $course_id = intval( $request_data[ 'course_id' ] );

    if( get_post_type( $course_id ) != WPEP_POST_TYPE_COURSE )
      return false;

    return [
      'status'      => 'ok',
      'timestamp'   => wpep_user_course_started_event( $course_id, get_current_user_id() )
    ];
  }

  public static function user_progress_repeat( $request_data ) {
    if( !isset( $request_data['course_id'] )  )
      return false;

    $course_id = intval( $request_data[ 'course_id' ] );

    if( get_post_type( $course_id ) != WPEP_POST_TYPE_COURSE )
      return false;

    wpep_content_type_course_instance()->user->reset_progress( $course_id, get_current_user_id() );

    return true;
  }

  public static function user_lesson_flag_completed( $request_data ) {
    if( !isset( $request_data['id'] ) || !isset( $request_data['course_id'] )  || get_post_type( intval( $request_data[ 'course_id' ] ) ) != WPEP_POST_TYPE_COURSE )
      return false;

    $default_lesson_flag = ( isset( $request_data['is_completed'] ) && $request_data['is_completed'] ? 1 : 0 );

    $course_id = intval( $request_data[ 'course_id' ] );

    if( is_array( $request_data['id'] ) ) {
      foreach( $request_data['id'] as $lesson_id => $lesson_status ) {
        if( null === $lesson_status )
          $lesson_status = $default_lesson_flag;

        wpep_content_type_course_instance()->user->lesson->set_status(
          $lesson_id, $lesson_status, get_current_user_id(), false, false
        );
      }
    } else {
      wpep_content_type_course_instance()->user->lesson->set_status(
        $request_data['id'], $default_lesson_flag, get_current_user_id(), false, false
      );
    }

    if( isset( $request_data[ 'course_progress' ] ) ) {
      $course_is_completed = ( $request_data['course_progress'] == 100 ? 1 : 0 );

      wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_PROGRESS, round( floatval( $request_data['course_progress'] ), 2 ), $course_id, 0, 0, get_current_user_id() );
      wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_COMPLETED, $course_is_completed, $course_id, 0, 0, get_current_user_id() );

      if( $course_is_completed ) {
        wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_COMPLETED_AT, time(), $course_id, 0, 0, get_current_user_id());
      } else {
        wpep_content_type_course_instance()->user->data->delete(WPEP_USER_COURSE_COMPLETED_AT, null, $course_id, 0, 0, get_current_user_id());
      }
    } else {
      $course_is_completed = wpep_user_course_is_completed( $request_data['course_id'], get_current_user_id() );
    }

    $response = [
      'status'           => 'ok',
      'course_completed' => $course_is_completed
    ];

    if( !$course_is_completed )
      return $response;

    $response[ 'tracking_code' ] = wpep_prepare_tracking_code( get_post_meta( $course_id, 'wpep_tracking_code_completed', true ), 'course-completed' );

    return $response;
  }

  public static function user_section_learning_path( $request_data ) {
    if( !isset( $request_data['course_id'] ) || !isset( $request_data['section_id'] ) || !isset( $request_data['learning_path'] ) )
      return false;

    wpep_content_type_course_instance()->user->flag->set_section_learning_path(
      intval( $request_data['course_id'] ), intval( $request_data['section_id'] ), intval( $request_data['learning_path'] ), get_current_user_id()
    );

    return true;
  }

  public static function user_flag_bookmark( $request_data ) {
    if( !isset( $request_data['course_id'] ) || !isset( $request_data['is_bookmarked'] ) )
      return false;

    wpep_content_type_course_instance()->user->flag->set_bookmark_status(
      intval( $request_data['course_id'] ),
      ( intval( $request_data['is_bookmarked'] ) ? 1 : 0 )
    );

    return true;
  }

  public static function admin_test_on_boarding( $request_data ) {
    if( !isset( $request_data['course_id'] )  )
      return false;

    $post = get_post( intval( $request_data['course_id'] ) );

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) || ( !isset( $post->post_author ) || $post->post_author != get_current_user_id() ) )
      return false;

    $on_boarding_status = get_post_meta( $post->ID, 'wpep_course_on_boarding_status', true );

    if( $on_boarding_status === 'email-notification' ) {
      WPEP_Controller::instance()->userEngagement->send_email(
        get_current_user_id(),
        get_post_meta( $post->ID, 'wpep_course_on_boarding_title', true ),
        get_post_meta( $post->ID, 'wpep_course_on_boarding_content', true ),
        true
      );
    }

    return true;
  }

  public static function pdf_download_data( $request_data ) {
    if( !isset( $request_data[ 'post_id' ] ) )
      return false;

    $post_id = $request_data[ 'post_id' ];

    if( get_post_type( $post_id ) !== WPEP_POST_TYPE_COURSE )
      return false;

    if( !\wpep_controller()->contentLibrary->has_access_to_post( $post_id ) )
      return false;

    if( is_user_logged_in() )
      WPEP_Entity_ActionTracking::instance()->track_event( $post_id, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_DOWNLOAD_AS_PDF, '' );

    $deliveryCoordinator = new WPEP_Entity_CourseType_PDF( $post_id );

    $deliveryCoordinator->init();

    return [
      'title'   => get_the_title( $post_id ),
      'image'   => wpep_course_thumbnail_url( $post_id ),
      'content' => $deliveryCoordinator->content()
    ];
  }

  public static function user_time_spent_tracking( $request_data ) {
    if( !isset( $request_data['setup_timestamp'] ) || !isset( $request_data['last_sync_start'] ) || !isset( $request_data['last_sync_success'] )
      || !isset( $request_data['last_sync_skip' ] ) || !isset( $request_data['tracking_map'] ) || !isset( $request_data['screen_viewport_map'] ) )
      return false;

    $user_id              = get_current_user_id();
    $setup_timestamp      = intval( $request_data[ 'setup_timestamp' ] );
    $last_sync_skip       = intval( $request_data[ 'last_sync_skip' ] );
    $last_sync_start      = intval( $request_data[ 'last_sync_start' ] );
    $last_sync_success    = intval( $request_data[ 'last_sync_success' ] );
    $tracking_map         = $request_data[ 'tracking_map' ];
    $screen_viewport_map  = $request_data[ 'screen_viewport_map' ];

    $increase_by = $last_sync_start - ( $last_sync_skip !== 0 ? $last_sync_skip : ( $last_sync_success === 0 ? $setup_timestamp : $last_sync_success ) );

    if( $increase_by > 20 )
      return false;

    foreach( $tracking_map as $tracking_map_item ) {
      if( !isset( $tracking_map_item['_nonce'] ) || !isset( $tracking_map_item['course_id'] ) )
        continue;

      if( false === wp_verify_nonce( $tracking_map_item['_nonce'], 'wpep_time_tracking_' . $tracking_map_item[ 'course_id' ] . '_' . $user_id ) )
        continue;

      $course_id = intval( $tracking_map_item['course_id'] );

      if( 0 === $course_id )
        continue;

      $in_viewport = isset( $screen_viewport_map[ $tracking_map_item['course_id'] ] ) ? ( $screen_viewport_map[ $tracking_map_item['course_id'] ] ? true : false ) : false;

      $time_spent = max( 0, intval( wpep_content_type_course_instance()->user->data->get_value(WPEP_USER_COURSE_TIME_SPENT, $course_id, false, false, $user_id, true ) ) );

      wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_TIME_SPENT, $time_spent + $increase_by, $course_id, false, false, $user_id );

      if( $in_viewport ) {
        $time_spent_focused = max( 0, intval( wpep_content_type_course_instance()->user->data->get_value(WPEP_USER_COURSE_TIME_SPENT_FOCUSED, $course_id, false, false, $user_id, true ) ) );

        wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_TIME_SPENT_FOCUSED, $time_spent_focused + $increase_by, $course_id, false, false, $user_id );
      }

      wpep_content_type_course_instance()->user->data->set(WPEP_USER_COURSE_TIME_SPENT_LAST_SYNC, time(), $course_id, false, false, $user_id );
    }

    return true;
  }

  public static function template_user_progressing_content( $request_data ) {
    if( !isset( $request_data[ 'user_id' ] ) )
      return false;

    $user_id = intval( $request_data[ 'user_id' ] );

    if( !WPEP_ACL_User::analytics( $user_id ) )
      return false;

    return wpep_content_primary_course_list_by_ids(
      wpep_content_type_course_instance()->user->get_progressing_course_list( $user_id ),
      [
        'course_progress_bars'  => (
        wpep_get_setting_flag( 'user-profile-course-public-progress-bars' )
          ? true
          : WPEP_ACL_User::analytics( $user_id, 'view' )
        ),
        'courses_per_row'       => wpep_get_setting( 'user-profile-courses-per-row' ),
        'user_id'               => $user_id
      ]
    );
  }

  public static function template_user_completed_content( $request_data ) {
    if( !isset( $request_data[ 'user_id' ] ) )
      return false;

    $user_id = intval( $request_data[ 'user_id' ] );

    if( !WPEP_ACL_User::analytics( $user_id ) )
      return false;

    return wpep_content_primary_course_list_by_ids(
      wpep_content_type_course_instance()->user->get_completed_course_list( $user_id ),
      [
        'course_progress_bars' => 0,
        'courses_per_row' => wpep_get_setting( 'user-profile-courses-per-row' )
      ]
    );
  }

  public static function template_user_bookmarked_content( $request_data ) {
    if( !isset( $request_data[ 'user_id' ] ) || !wpep_course_has_bookmarks() )
      return false;

    $user_id = intval( $request_data[ 'user_id' ] );

    if( !WPEP_ACL_User::analytics( $user_id ) )
      return false;

    return wpep_content_primary_course_list_by_ids(
      wpep_content_type_course_instance()->user->get_bookmarked_course_list( $user_id, 1 ),
      [
        'courses_per_row' => wpep_get_setting( 'user-profile-courses-per-row' )
      ]
    );
  }

  public static function get_scorm_configuration( $request_data ) {
    if( !isset( $request_data['post_id'] ) )
      return false;

    $course_id = intval( $request_data['post_id'] );

    if( get_post_type( $course_id ) != WPEP_POST_TYPE_COURSE || wpep_course_type_from_id( $course_id ) != WPEP_COURSE_TYPE_SCORM )
      return false;

    $directory = wpep_course_scorm_directory_path( $course_id );

    if( empty( $directory ) )
      return false;

    $dispatch_file = get_post_meta( $course_id, 'wpep_scorm_dispatch', true );
    $dispatch_file = $directory . '/' . ( $dispatch_file === '' ? 'dispatch.html' : $dispatch_file );

    $user_id = get_current_user_id();

    $scorm_learner = [
      'cmi.core._children'       => 'student_id,student_name,lesson_status,lesson_location,lesson_mode,score,credit,entry,exit,session_time,total_time',
      'cmi.core.score._children' => 'raw',
      'cmi.core.student_id'      => 'wpep_student_' . $user_id,
      'cmi.core.student_name'    => wpep_content_user_profile_format_name( $user_id ),
      'cmi.core.lesson_status'   => 'not attempted', // 'not attempted', 'incomplete', 'completed', 'passed'
      'cmi.core.score.raw'       => '',
      'cmi.core.lesson_location' => '',
      'cmi.suspend_data'         => '',
      'cmi.core.session_time'    => '0000:00:00.00',
      'cmi.core.credit'          => 'credit', /* "credit" or "no-credit" */
      'cmi.core.entry'           => 'ab-initio', /* "resume" or "ab-initio" */
      'cmi.core.lesson_mode'     => 'normal', /* "browse",  "normal" or "review" */
      'cmi.core.exit'            => '' /* "time-out", "suspend" or "logout" */
    ];

    if( $user_id != 0 ) {
      foreach( wpep_course_scorm_student_data_mapping() as $scorm_key => $wpep_key ) {
        $value = wpep_content_type_course_instance()->user->data->get_value(
          $wpep_key,
          $course_id,
          false,
          false,
          $user_id
        );

        if( $value === NULL )
          continue;

        $scorm_learner[ $scorm_key ] = $value;
      }
    }

    return [
      'status'        => 'ok',
      'dispatch_file' => $dispatch_file,
      'scorm_learner' => $scorm_learner
    ];
  }

  public static function set_scorm_user_information( $request_data ) {
    if( !isset( $request_data['post_id'] ) || !isset( $request_data['information'] ) )
      return false;

    $course_id  = $request_data[ 'post_id' ];

    if( get_post_type( $course_id ) != WPEP_POST_TYPE_COURSE || wpep_course_type_from_id( $course_id ) != WPEP_COURSE_TYPE_SCORM )
      return false;

    $information = $request_data[ 'information' ];

    foreach( wpep_course_scorm_student_data_mapping() as $scorm_key => $wpep_key ) {
      if( !isset( $information[ $scorm_key ] ) )
        continue;

      wpep_content_type_course_instance()->user->data->set(
        $wpep_key,
        $information[ $scorm_key ],
        $course_id,
        false,
        false,
        get_current_user_id()
      );
    }

    if( isset( $information['cmi.core.lesson_status'] ) ) {
      if( in_array( $information['cmi.core.lesson_status'], [ 'completed', 'passed' ] ) ) {
        wpep_content_type_course_instance()->user->flag->set_status_completed( $course_id, get_current_user_id() );
      } else {
        wpep_content_type_course_instance()->user->flag->set_status_incomplete( $course_id, get_current_user_id() );
      }
    }

    return true;
  }

  public static function user_recommendation_list( $request_data ) {
    if( !isset( $request_data[ 'user_id' ] ) )
      return false;

    $request_data[ 'user_id' ] = intval( $request_data[ 'user_id' ] );

    $recommendation_list = wpep_content_type_course_instance()->user->get_recommended_ids( $request_data[ 'user_id' ], 9999 );

    $total_recommendations = count( $recommendation_list );
    $pagination_limit = 20;

    $response = [
      'status'          => 'ok',
      'entries_header'  => [
        'post_id_formatted'    => __( "Post ID", "wpep" ),
        'progress_formatted'   => __( "Progress", "wpep" ),
        'started_at_formatted' => __( "Completed At", "wpep" ),
      ],
      'entries'         => [],
      'pagination'      => [
        'current'       => ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 ),
        'total'         => ceil( $total_recommendations / $pagination_limit ),
        'total_entries' => $total_recommendations
      ]
    ];

    $recommendation_list = array_slice( $recommendation_list, ( $response[ 'current'] - 1 ) * $pagination_limit, $pagination_limit );

    foreach( $recommendation_list as $recommendation_id ) {
      $started_at = wpep_user_course_started_at( $recommendation_id, $request_data[ 'user_id' ] );
      $progress = wpep_user_course_progress_percentage( $recommendation_id, $request_data[ 'user_id' ] );

      $response[ 'entries' ][] = [
        'post_id'              => $recommendation_id,
        'post_id_formatted'    => '#' . $recommendation_id,
        'started_at'           => $started_at,
        'started_at_formatted' => wpep_format_date_with_time( $started_at ),
        'progress'             => $progress,
        'progress_formatted'   => wpep_get_percentage( $progress )
      ];
    }

    $response = API_Utils_Post::entries( $response, $request_data );

    return $response;
  }

  /**
   * @param $request_data
   * @return array|bool
   */
  public static function completed_lessons_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $pagination_page  = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );
    $pagination_limit = 20;

    $response = [
      'status'          => 'ok',
      'entries_header'  => [
        'section_title'          => __( "Section", "wpep" ),
        'lesson_title'           => __( "Lesson", "wpep" ),
        'completed_at_formatted' => __( "Completed At", "wpep" ),
        'course_type_label'      => __( "Course Type", "wpep" ),
      ],
      'entries'         => [],
      'pagination'      => [
        'current'       => $pagination_page,
        'total'         => 0,
        'total_entries' => 0
      ]
    ];

    $entry_query_sql = 'SELECT SQL_CALC_FOUND_ROWS
                               join_section_lesson.`post_id`          as post_id,  
                               section_lesson_completed.`user_id`     as user_id,
                               join_section_lesson.`id`               as lesson_id,
                               join_section_lesson.`title`            as lesson_title,
                               join_section.`id`                      as section_id,
                               join_section.`title`                   as section_title,
                               section_lesson_completed.`created_at`  as completed_at
                          FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` section_lesson_completed
                    INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` join_section_lesson 
                            ON join_section_lesson.`id` = section_lesson_completed.`lesson_id`
                    INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` join_section
                            ON join_section.`id` = join_section_lesson.`section_id`
                    INNER JOIN `' . wpep_wpdb()->posts . '` posts 
                            ON posts.`ID` = join_section_lesson.`post_id`
                           AND posts.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )
                         WHERE 1 = 1';

    if( isset( $request_data[ 'user_ids' ] ) ) {
      if( is_numeric( $request_data[ 'user_ids' ] ) )
        $entry_query_sql .= ' AND section_lesson_completed.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
      else if( is_array( $request_data[ 'user_ids' ] ) )
        $entry_query_sql .= ' AND section_lesson_completed.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
    }

    if( isset( $request_data[ 'post_ids' ] ) ) {
      if( is_numeric( $request_data[ 'post_ids' ] ) )
        $entry_query_sql .= ' AND join_section_lesson.`post_id` = ' . intval( $request_data[ 'post_ids' ] );
      else if( is_array( $request_data[ 'post_ids' ] ) )
        $entry_query_sql .= ' AND join_section_lesson.`post_id` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
    }

    if( isset( $request_data[ 'report_time_from' ] ) )
      $entry_query_sql .= ' AND section_lesson_completed.`created_at` >= "' . date( "Y-m-d H:i:s", $request_data[ 'report_time_from' ] ) . '"';

    if( isset( $request_data[ 'report_time_to' ] ) )
      $entry_query_sql .= ' AND section_lesson_completed.`created_at` < "' . date( "Y-m-d H:i:s", $request_data[ 'report_time_to' ] ) . '"';


    $entry_query_sql .= ' ORDER BY section_lesson_completed.`created_at` DESC
                                  LIMIT ' . $pagination_limit . ( $pagination_page === 1 ? '' : ' OFFSET ' . ( ( $pagination_page - 1 ) * $pagination_limit) );

    $entries = wpep_wpdb()->get_results( $entry_query_sql );

    if( empty( $entries ) )
      return $response;

    $response[ 'pagination' ][ 'total_entries' ] = absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) );
    $response[ 'pagination' ][ 'total' ] = ceil( $response[ 'pagination' ][ 'total_entries' ] / $pagination_limit );

    foreach( $entries as $entry ) {
      $course_type = wpep_course_type_from_id( $entry->post_id );

      $response[ 'entries' ][] = [
        'user_id'    => $entry->user_id,
        'post_id'    => $entry->post_id,
        'course_type'=> $course_type,
        'course_type_label' => wpep_course_layout_type_label( $course_type ),
        'section_id' => $entry->section_id,
        'lesson_id'  => $entry->lesson_id,
        'section_title'          => $entry->section_title,
        'lesson_title'           => $entry->lesson_title,
        'completed_at'           => strtotime( $entry->completed_at ),
        'completed_at_formatted' => wpep_format_date_with_time( $entry->completed_at )
      ];
    }

    $response = API_Utils_Post::entries( $response, $request_data );

    if( isset( $request_data[ 'user_ids' ] ) && is_numeric( $request_data[ 'user_ids' ] ) )
      return $response;

    $user_ids = wpep_utility_map_to_array_assoc(
      $response[ 'entries' ],
      'user_id',
      'user_id'
    );

    if( empty( $user_ids ) )
      return $response;

    $entries_user_objects = ( isset( $response[ 'entries' ] ) ? wpep_utilities_map_object(
      get_users( [
        'include' => $user_ids
      ] ),
      'ID'
    ) : [] );

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  public static function progress_repeat_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $pagination_page  = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );
    $pagination_limit = 20;

    $response = [
      'status'          => 'ok',
      'entries_header'  => [
        'repeat_at_formatted' => __( "Repeat At", "wpep" ),
      ],
      'entries'         => [],
      'pagination'      => [
        'current'       => $pagination_page,
        'total'         => 0,
        'total_entries' => 0
      ]
    ];

    $entry_query_sql = 'SELECT SQL_CALC_FOUND_ROWS
                               course_user_data.`course_id` as post_id,  
                               course_user_data.`user_id`   as user_id,
                               course_user_data.`value`     as repeat_at
                          FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_USER_DATA . '` course_user_data
                    INNER JOIN `' . wpep_wpdb()->posts . '` posts 
                            ON posts.`ID` = course_user_data.`course_id`
                           AND posts.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )
                         WHERE course_user_data.key = "' . WPEP_USER_COURSE_REPEAT_TRACKER . '"';

    if( isset( $request_data[ 'user_ids' ] ) ) {
      if( is_numeric( $request_data[ 'user_ids' ] ) )
        $entry_query_sql .= ' AND course_user_data.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
      else if( is_array( $request_data[ 'user_ids' ] ) )
        $entry_query_sql .= ' AND course_user_data.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
    }

    if( isset( $request_data[ 'post_ids' ] ) ) {
      if( is_numeric( $request_data[ 'post_ids' ] ) )
        $entry_query_sql .= ' AND course_user_data.`course_id` = ' . intval( $request_data[ 'post_ids' ] );
      else if( is_array( $request_data[ 'post_ids' ] ) )
        $entry_query_sql .= ' AND course_user_data.`course_id` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
    }

    if( isset( $request_data[ 'report_time_from' ] ) )
      $entry_query_sql .= ' AND CAST( course_user_data.`value` AS UNSIGNED ) >= "' . $request_data[ 'report_time_from' ] . '"';

    if( isset( $request_data[ 'report_time_to' ] ) )
      $entry_query_sql .= ' AND CAST( course_user_data.`value` AS UNSIGNED ) < "' . $request_data[ 'report_time_to' ] . '"';


    $entry_query_sql .= ' ORDER BY CAST( course_user_data.`value` AS UNSIGNED ) DESC
                                  LIMIT ' . $pagination_limit . ( $pagination_page === 1 ? '' : ' OFFSET ' . ( ( $pagination_page - 1 ) * $pagination_limit) );

    $entries = wpep_wpdb()->get_results( $entry_query_sql );

    if( empty( $entries ) )
      return $response;

    $response[ 'pagination' ][ 'total_entries' ] = absint( wpep_wpdb()->get_var( 'SELECT FOUND_ROWS();' ) );
    $response[ 'pagination' ][ 'total' ] = ceil( $response[ 'pagination' ][ 'total_entries' ] / $pagination_limit );

    foreach( $entries as $entry ) {
      $response[ 'entries' ][] = [
        'user_id'             => $entry->user_id,
        'post_id'             => $entry->post_id,
        'repeat_at'           => $entry->repeat_at,
        'repeat_at_formatted' => wpep_format_date_with_time( $entry->repeat_at )
      ];
    }

    $response = API_Utils_Post::entries( $response, $request_data );

    if( isset( $request_data[ 'user_ids' ] ) && is_numeric( $request_data[ 'user_ids' ] ) )
      return $response;

    $user_ids = wpep_utility_map_to_array_assoc(
      $response[ 'entries' ],
      'user_id',
      'user_id'
    );

    if( empty( $user_ids ) )
      return $response;

    $entries_user_objects = ( isset( $response[ 'entries' ] ) ? wpep_utilities_map_object(
      get_users( [
        'include' => $user_ids
      ] ),
      'ID'
    ) : [] );

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  public static function completed_lessons_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $sql = 'SELECT COUNT( lc.id ) as lesson_completed_count,
                     l.title as lesson_title,
                     s.title as section_title,
                     p.post_title as post_title
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
                  ON l.id = lc.lesson_id
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` s
                  ON s.id = l.section_id
          INNER JOIN `' . wpep_wpdb()->posts . '` p
                  ON p.ID = s.post_id
                 AND p.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )
               WHERE lc.`created_at` >= "' . date( "Y-m-d H:i:s", $time_slot_from ) . '" 
                 AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $time_slot_to ) . '"';

      if( isset( $request_data[ 'user_ids' ] ) ) {
        if( is_numeric( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
        else if( is_array( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
      }

      if( isset( $request_data[ 'post_ids' ] ) ) {
        if( is_numeric( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` = ' . intval( $request_data[ 'post_ids' ] );
        else if( is_array( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
      }

      $sql .= ' GROUP BY lc.`lesson_id`';

      $lessons_completed = wpep_wpdb()->get_results( $sql );

      foreach( $lessons_completed as $lesson_completed )
        $current_time_slot_data[
          sprintf( __( "Lesson Completed - %s", "wpep" ), $lesson_completed->post_title . ' - ' . $lesson_completed->section_title . ' - ' . $lesson_completed->lesson_title )
        ] = $lesson_completed->lesson_completed_count;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ];
  }

  public static function section_completed_lessons_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $sql = 'SELECT COUNT( lc.id ) as lesson_completed_count,
                     s.title as section_title,
                     p.post_title as post_title
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
                  ON l.id = lc.lesson_id
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` s
                  ON s.id = l.section_id
          INNER JOIN `' . wpep_wpdb()->posts . '` p
                  ON p.ID = s.post_id
                 AND p.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )
               WHERE lc.`created_at` >= "' . date( "Y-m-d H:i:s", $time_slot_from ) . '" 
                 AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $time_slot_to ) . '"';

      if( isset( $request_data[ 'user_ids' ] ) ) {
        if( is_numeric( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
        else if( is_array( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
      }

      if( isset( $request_data[ 'post_ids' ] ) ) {
        if( is_numeric( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` = ' . intval( $request_data[ 'post_ids' ] );
        else if( is_array( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
      }

      $sql .= ' GROUP BY l.`section_id`';

      $lessons_completed = wpep_wpdb()->get_results( $sql );

      foreach( $lessons_completed as $lesson_completed )
        $current_time_slot_data[
        sprintf( __( "Section Lesson Completed - %s", "wpep" ), $lesson_completed->post_title . ' - ' . $lesson_completed->section_title )
        ] = $lesson_completed->lesson_completed_count;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ];
  }

  public static function course_completed_lessons_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $sql = 'SELECT COUNT( lc.id ) as lesson_completed_count,
                     p.post_title as post_title
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
                  ON l.id = lc.lesson_id
          INNER JOIN `' . wpep_wpdb()->posts . '` p
                  ON p.ID = l.post_id
                 AND p.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )
               WHERE lc.`created_at` >= "' . date( "Y-m-d H:i:s", $time_slot_from ) . '" 
                 AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $time_slot_to ) . '"';

      if( isset( $request_data[ 'user_ids' ] ) ) {
        if( is_numeric( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
        else if( is_array( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
      }

      if( isset( $request_data[ 'post_ids' ] ) ) {
        if( is_numeric( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` = ' . intval( $request_data[ 'post_ids' ] );
        else if( is_array( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
      }

      $sql .= ' GROUP BY l.`post_id`';

      $lessons_completed = wpep_wpdb()->get_results( $sql );

      foreach( $lessons_completed as $lesson_completed )
        $current_time_slot_data[ sprintf( __( "Course Lesson Completed - %s", "wpep" ), $lesson_completed->post_title ) ] = $lesson_completed->lesson_completed_count;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ];
  }

  public static function category_completed_lessons_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $sql = 'SELECT COUNT( lc.id ) as lesson_completed_count,
                     terms.name as term_name
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
                  ON l.id = lc.lesson_id
          INNER JOIN `' . wpep_wpdb()->posts . '` p
                  ON p.ID = l.post_id
                 AND p.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )
          INNER JOIN `' . wpep_wpdb()->term_relationships . '` term_rel
                  ON term_rel.object_id = p.ID
          INNER JOIN `' . wpep_wpdb()->term_taxonomy . '` term_tax
                  ON term_tax.term_id = term_rel.term_taxonomy_id
          INNER JOIN `' . wpep_wpdb()->terms . '` terms
                  ON terms.term_id = term_tax.term_taxonomy_id
               WHERE lc.`created_at` >= "' . date( "Y-m-d H:i:s", $time_slot_from ) . '" 
                 AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $time_slot_to ) . '"';

      if( isset( $request_data[ 'term_parent_id' ] ) )
        $sql .= ' AND terms.`parent` = ' . intval( $request_data[ 'term_parent_id' ] );

      if( isset( $request_data[ 'user_ids' ] ) ) {
        if( is_numeric( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
        else if( is_array( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
      }

      if( isset( $request_data[ 'post_ids' ] ) ) {
        if( is_numeric( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` = ' . intval( $request_data[ 'post_ids' ] );
        else if( is_array( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
      }

      $sql .= ' GROUP BY terms.`term_id`';

      $lessons_completed = wpep_wpdb()->get_results( $sql );

      foreach( $lessons_completed as $lesson_completed )
        $current_time_slot_data[
          sprintf( __( "Category Lesson Completed - %s", "wpep" ), $lesson_completed->term_name )
        ] = $lesson_completed->lesson_completed_count;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ];
  }

  public static function user_completed_lessons_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $sql = 'SELECT COUNT( lc.id ) as lesson_completed_count,
                     lc.user_id as user_id
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
                  ON l.id = lc.lesson_id
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` s
                  ON s.id = l.section_id
          INNER JOIN `' . wpep_wpdb()->posts . '` p
                  ON p.ID = s.post_id
                 AND p.`post_status` IN ( "' . implode( '","', wpep_post_monitored_post_status_list() ) . '" )
               WHERE lc.`created_at` >= "' . date( "Y-m-d H:i:s", $time_slot_from ) . '" 
                 AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $time_slot_to ) . '"';

      if( isset( $request_data[ 'user_ids' ] ) ) {
        if( is_numeric( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` = ' . intval( $request_data[ 'user_ids' ] );
        else if( is_array( $request_data[ 'user_ids' ] ) )
          $sql .= ' AND lc.`user_id` IN (' . implode( ',', $request_data[ 'user_ids' ] ) . ')';
      }

      if( isset( $request_data[ 'post_ids' ] ) ) {
        if( is_numeric( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` = ' . intval( $request_data[ 'post_ids' ] );
        else if( is_array( $request_data[ 'post_ids' ] ) )
          $sql .= ' AND l.`post_id` IN (' . implode( ',', $request_data[ 'post_ids' ] ) . ')';
      }

      $sql .= ' GROUP BY lc.`user_id`';

      $lessons_completed = wpep_wpdb()->get_results( $sql );

      foreach( $lessons_completed as $lesson_completed )
        $current_time_slot_data[
        sprintf( __( "User Lesson Completed - %s", "wpep" ), wpep_content_user_profile_format_name( $lesson_completed->user_id ) )
        ] = $lesson_completed->lesson_completed_count;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ];
  }

  public static function user_id_post_id_data_view_plain( $request_data ) {
    $request_data[ 'user_id' ] = intval( $request_data[ 'user_id' ] );
    $request_data[ 'post_id' ] = intval( $request_data[ 'post_id' ] );

    $user_information = get_user_by( 'ID', $request_data[ 'user_id' ] );

    return [
      'status'  => 'ok',
      'html'    => wpep_template_get( 'admin/data-view-plain-user-course.php', [
        'user_information'      => $user_information,
        'user_progress_repeat'  => wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_REPEAT_TRACKER, $request_data[ 'post_id' ], false, false, $request_data[ 'user_id' ], false ),
        'user_course_map'       => wpep_content_type_course_instance()->user->section->get_all_by_course_id( $request_data[ 'post_id' ], $user_information->ID ),
        'course_id'             => $request_data[ 'post_id' ],
        'user_id'               => $request_data[ 'user_id' ]
      ] )
    ];
  }

  public static function bird_eye_view_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_id' ] ) )
      return false;

    $request_data[ 'post_id' ] = intval( $request_data[ 'post_id' ] );

    if( !wpep_course_type_has_lessons( wpep_course_type_from_id( $request_data[ 'post_id' ] ) ) )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $sql = 'SELECT COUNT( lc.id ) as lesson_completed_count,
                     l.title as lesson_title,
                     s.title as section_title
                FROM `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON_COMPLETED . '` lc
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION_LESSON . '` l
                  ON l.id = lc.lesson_id
          INNER JOIN `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_COURSE_SECTION . '` s
                  ON s.id = l.section_id
               WHERE lc.`created_at` >= "' . date( "Y-m-d H:i:s", $time_slot_from ) . '" 
                 AND lc.`created_at` < "' . date( "Y-m-d H:i:s", $time_slot_to ) . '"
                 AND l.`post_id` = ' . intval( $request_data[ 'post_id' ] ) . '
            GROUP BY lc.`lesson_id`';

      $lessons_completed = wpep_wpdb()->get_results( $sql );

      foreach( $lessons_completed as $lesson_completed )
        $current_time_slot_data[ $lesson_completed->section_title . ' - ' . $lesson_completed->lesson_title ] = $lesson_completed->lesson_completed_count;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ];
  }

  public static function bird_eye_view_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_id' ] ) )
      return false;

    $request_data[ 'post_id' ] = intval( $request_data[ 'post_id' ] );

    if( !wpep_course_type_has_lessons( wpep_course_type_from_id( $request_data[ 'post_id' ] ) ) )
      return false;

    $args = [
      'current_page'     => ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 ),
      'pagination_limit' => 20,
      'post_ids'         => $request_data[ 'post_id' ]
    ];

    if( isset( $request_data[ 'report_time_from' ] ) )
      $args[ 'time_from' ] = $request_data[ 'report_time_from' ];

    if( isset( $request_data[ 'report_time_to' ] ) )
      $args[ 'time_to' ] = $request_data[ 'report_time_to' ];

    $response = wpep_content_type_course_instance()->data_user_post_activity_list( $args );

    $response[ 'status' ] = 'ok';

    if( empty( $response[ 'entries' ] ) )
      return $response;

    $final_assessment_id = ( wpep_assessments_enabled() ? wpep_get_course_assessment_id( $request_data[ 'post_id' ], 0 ) : 0 );

    foreach( $response[ 'entries' ] as $response_entry_key => $response_entry ) {
      $course_sections_map = wpep_content_type_course_instance()->user->section->get_all_by_course_id_mapped( $request_data[ 'post_id' ], $response_entry[ 'user_id' ] );

      foreach( $course_sections_map as $course_section_id => $course_section ) {

        if( !isset( $response[ 'entries_header' ][ 'course_section_' . $course_section_id ] ) )
          $response[ 'entries_header' ][ 'course_section_' . $course_section_id ] = 'S. ' . $course_section[ 'title' ];

        if( ( isset( $course_section['options'] ) && isset( $course_section['options']['is_disabled'] ) && $course_section['options']['is_disabled'] ) )
          $response[ 'entries' ][ $response_entry_key ][ 'course_section_' . $course_section_id ] = __( "Disabled", "wpep" );
        else if( $course_section[ 'is_completed' ] )
          $response[ 'entries' ][ $response_entry_key ][ 'course_section_' . $course_section_id ] = __( "Completed", "wpep" );
        else
          $response[ 'entries' ][ $response_entry_key ][ 'course_section_' . $course_section_id ] = __( "Pending", "wpep" );

        foreach( $course_section[ 'lessons' ] as $lesson_id => $lesson ) {

          if( !isset( $response[ 'entries_header' ][ 'course_lesson_' . $lesson_id ] ) )
            $response[ 'entries_header' ][ 'course_lesson_' . $lesson_id ] = 'L. ' . $lesson[ 'title' ];

          if( ( isset( $lesson['options'] ) && isset( $lesson['options']['is_disabled'] ) && $lesson['options']['is_disabled'] ) )
            $response[ 'entries' ][ $response_entry_key ][ 'course_lesson_' . $lesson_id ] = __( "Disabled", "wpep" );
          else if( $lesson[ 'is_completed' ] )
            $response[ 'entries' ][ $response_entry_key ][ 'course_lesson_' . $lesson_id ] = __( "Completed", "wpep" );
          else
            $response[ 'entries' ][ $response_entry_key ][ 'course_lesson_' . $lesson_id ] = __( "Pending", "wpep" );
        }

        if( !isset( $course_section[ 'assessment_id' ] ) || $course_section[ 'assessment_id' ] === 0 )
          continue;

        if( !isset( $response[ 'entries_header' ][ 'course_assessment_' . $course_section[ 'assessment_id' ] ] ) )
          $response[ 'entries_header' ][ 'course_assessment_' . $course_section[ 'assessment_id' ] ] = 'A. ' . get_the_title( $course_section[ 'assessment_id' ] );

        $response[ 'entries' ][ $response_entry_key ][ 'course_assessment_' . $course_section[ 'assessment_id' ] ] = wpep_user_assessment_status( $course_section[ 'assessment_id' ], $response_entry[ 'user_id' ] );
      }

      if( $final_assessment_id !== 0 )
        $response[ 'entries' ][ $response_entry_key ][ 'course_assessment_' . $final_assessment_id ] = wpep_user_assessment_status( $final_assessment_id, $response_entry[ 'user_id' ] );
    }

    if( $final_assessment_id !== 0 )
      $response[ 'entries_header' ][ 'course_assessment_' . $final_assessment_id ] = 'A. ' . get_the_title( $final_assessment_id );

    $user_ids = wpep_utility_map_to_array_assoc(
      $response[ 'entries' ],
      'user_id',
      'user_id'
    );

    if( empty( $user_ids ) )
      return $response;

    $entries_user_objects = ( isset( $response[ 'entries' ] ) ? wpep_utilities_map_object(
      get_users( [
        'include' => $user_ids
      ] ),
      'ID'
    ) : [] );

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

}