<?php

namespace WPEP\ContentType\Course;

use WPEP\Container\FormFields as WPEP_FormFields_Configuration;

class Admin {

  /**
   * @var Admin\Settings
   */
  public $settings;

  use WPEP_FormFields_Configuration;

  public function __construct() {
    $this->settings = new Admin\Settings();
  }

  public function init() {
    if( !is_admin() )
      return;

    $this->set_field_prefix( 'wpep_meta_value' );

    add_action('post_row_actions', [ $this, 'row_actions' ], 10, 2);

    add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
    add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( $post_type != WPEP_POST_TYPE_COURSE )
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'course_type' => __( "Course Type", "wpep" ),
      'shortcode'   => __( "Shortcode", "wpep"),
    ];

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post;

    if( $post->post_type != WPEP_POST_TYPE_COURSE )
      return;

    if ( $column == 'shortcode' ) {
      echo wpep_admin_shortcode_builder(
        WPEP_SHORT_CODE_PREFIX . 'course',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_course_options'
        ],
        __( "ShortCode", "wpep" ),
        true
      );

      return;
    }

    if( $column == 'course_type' ) {
      $course_layouts = wpep_course_layout_types();
      $course_type = wpep_course_type_from_id( $post->ID );;

      echo isset( $course_layouts[ $course_type ] ) ? $course_layouts[ $course_type ] : $course_type;
      return;
    }
  }

  public function row_actions($actions, $post) {
    if( $post->post_type != WPEP_POST_TYPE_COURSE )
      return $actions;

    if( get_post_meta( $post->ID , WPEP_COURSE_TYPE_POST_META, true ) != WPEP_COURSE_TYPE_ONE_OFF ) {
      $actions['wpep_preview'] = wpep_admin_shortcode_builder(
        WPEP_SHORT_CODE_PREFIX . 'course_preview',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_course_preview_options'
        ],
        __( "Course Preview", "wpep" ),
        false
      );
    }

    return $actions;
  }

  public function get_meta_fields() {
    $wpep_course_options_fields = [];

    if( wpep_has_sid_support() ) {
      $wpep_course_options_fields[ 'wpep_course_identifier' ] = [
        'name'        => __( 'Course Identifier', "wpep" ),
        'id'          => 'wpep_course_identifier',
        'type'        => 'text',
        'description' => __( "Intended for higher education or government reporting or in connecting other systems.", "wpep" )
      ];
      $wpep_course_options_fields[ 'wpep_course_identifier_version' ] = [
        'name'        => __( 'Course Identifier Version', "wpep" ),
        'id'          => 'wpep_course_identifier_version',
        'type'        => 'text',
        'description' => __( "Intended for higher education or government reporting or in connecting other systems.", "wpep" )
      ];
    }

    $wpep_course_options_fields[ 'wpep_social_share_status' ] = [
      'name'            => __( 'Social Share', "wpep" ),
      'id'              => 'wpep_disable_social_share',
      'type'            => 'select',
      'options' => [
        '' => sprintf( __( "Course Type Default ( %s )" ), ( wpep_get_setting_flag( 'course-challenge-social-share-status' ) ? __( 'Enabled', "wpep" ) : __( "Disabled", "wpep" ) ) ),
      ] + wpep_flag_setting_field_options(),
      'content_before'  => '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_CHALLENGE . '">',
      'content_after'   => '</div>'
    ];

    $wpep_course_options_fields[ 'wpep_social_share_link' ] = [
      'name'            => __( 'Social Share Link', "wpep" ),
      'id'              => 'wpep_social_share_link',
      'type'            => 'text',
      'description'     => __( "Leave this empty to share the course link instead.", "wppe" ),
      'content_before'  => '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_CHALLENGE . '">',
      'content_after'   => '</div>'
    ];

    $wpep_course_options_fields += [
      WPEP_COURSE_TYPE_POST_META => [
        'name'    => __('Course Type', "wpep"),
        'desc'    => __('By default a course is an execution plan.', "wpep"),
        'id'      => WPEP_COURSE_TYPE_POST_META,
        'type'    => 'select',
        'options' => wpep_course_layout_types(),
        'attributes' => [
          'data-wpep-conditional-display' => 'data-wpep-course-type-information-block'
        ]
      ],
      'course_theme' => [
        'name'    => __('Course Theme', "wpep"),
        'desc'    => __('This option in not available for shortcode embeds of this course.', "wpep"),
        'id'      => 'course_theme',
        'type'    => 'select',
        'options' => wpep_get_theme_list_as_entity_layout_option()
      ]
    ];

    if( wpep_get_setting_flag( 'enable-course-discussions-option' ) ) {
      $wpep_course_options_fields[ 'wpep_course_enable_discussions' ] = [
        'name'    => __( 'Discussions', "wpep" ),
        'desc'    => sprintf( __( 'Enable the %s Discussions feature within courses.', "wpep" ), WPEP_PLUGIN_NAME ),
        'id'      => 'wpep_course_enable_discussions',
        'type'    => 'select',
        'options' => wpep_flag_setting_field_options()
      ];

      $wpep_course_options_fields[ 'wpep_course_discussion_segmentation' ] = [
        'name'    => __( 'Discussions Segmentation', "wpep" ),
        'id'      => 'wpep_course_discussion_segmentation',
        'type'    => 'select',
        'options' => [
          'entity' => __( "On Each Page", "wpep" ),
          'course' => __( "Course Level", "wpep" )
        ],
        'content_before'  => '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_STANDARD . ' ' . WPEP_COURSE_TYPE_FOCUS  . ' ' . WPEP_COURSE_TYPE_VIDEO . ' ' . WPEP_COURSE_TYPE_CHALLENGE . '">',
        'content_after'   => '</div>'
      ];
    }

    $wpep_course_options_fields += [
      'wpep_standard_course_overview_page'  => [
        'name'    => __( 'Overview Page', "wpep" ),
        'desc'    => '<p>' . __( "Individually chose if this course should have an Index Page / Overview Page", "wpep" ) . '</p>',
        'id'      => 'wpep_standard_course_overview_page',
        'type'    => 'select',
        'options' => [
            'default' => sprintf( __( "Course Type Default ( %s )" ), ( wpep_get_setting_flag( 'course-standard-allow-index-page' ) ? __( 'Enabled', "wpep" ) : __( "Disabled", "wpep" ) ) ),
          ] + wpep_flag_setting_field_options(),
        'content_before'  => '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_STANDARD . '">',
        'content_after'   => '</div>'
      ],
      'wpep_video_course_overview_page' => [
        'name'    => __( 'Overview Page', "wpep" ),
        'desc'    => '<p>' . __( "Individually chose if this course should have an Index Page / Overview Page", "wpep" ) . '</p>',
        'id'      => 'wpep_video_course_overview_page',
        'type'    => 'select',
        'options' => [
            'default' => sprintf( __( "Course Type Default ( %s )" ), ( wpep_get_setting_flag( 'course-video-allow-index-page' ) ? __( 'Enabled', "wpep" ) : __( "Disabled", "wpep" ) ) ),
          ] + wpep_flag_setting_field_options() ,
        'content_before'  => '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_VIDEO . '">',
        'content_after'   => '</div>'
      ],
    ];

    if( wpep_assessments_enabled() ) {
      $wpep_course_options_fields[ 'wpep_course_assessments_required' ] = [
        'name'    => __( 'Assessments Are', "wpep" ),
        'desc'    => '<p>' . __( "The student is required to complete the section / module assessment in order to continue the course.", "wpep" ) . '</p>',
        'id'      => 'wpep_course_assessments_required',
        'type'    => 'select',
        'options' => [
          1         => __( 'Mandatory', "wpep" ),
          0         => __( 'Optional', "wpep" ),
        ],
      ];
    }

    $wpep_course_options_fields[ WPEP_PREREQUISITE_MODE_POST_META ] = [
      'name'    => __( 'Prerequisites Mode', "wpep" ),
      'desc'    => '<p>' . __( "How the prerequisites logic is handled.", "wpep" ) . '</p>',
      'id'      => WPEP_PREREQUISITE_MODE_POST_META,
      'type'    => 'select',
      'options' => wpep_get_prerequisite_mode_list()
    ];

    $selected_prerequisites = get_post_meta( get_the_ID(), 'wpep_prerequisite', false );

    if( !empty( $selected_prerequisites ) ) {
      $selected_prerequisites = get_posts( [
        'post_type' => WPEP_POST_TYPE_COURSE,
        'nopaging'  => true,
        'include'   => $selected_prerequisites
      ] );

      if( !empty( $selected_prerequisites ) )
        $selected_prerequisites = wpep_utility_map_to_array_assoc( $selected_prerequisites, 'ID', 'post_title' );
    }

    $wpep_course_options_fields[ 'wpep_prerequisite' ] = [
      'name'          => __( 'Prerequisites', "wpep" ),
      'desc'          => '<p>' . __( "The student must complete these courses in order to have access to this one.", "wpep" ) . '</p>',
      'id'            => 'wpep_prerequisite',
      'type'          => 'multiple_select',
      'options'       => $selected_prerequisites,
      'attributes'    => [
        'data-ec-library'      => 'select2',
        "data-ec-library-mode" => 'post-search-map|post_type:' . WPEP_POST_TYPE_COURSE
      ]
    ];

    $wpep_course_options_fields[ 'wpep_sidebar_position' ] = [
      'name'    => __('Sidebar', "wpep"),
      'desc'    =>
        '<p data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . '">' . __( "An Execution Plan will not have a sidebar present if you haven't setup one yourself", "wpep" ) . '</p>' .
        '<p data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_CHALLENGE . '">' . __( "An Challenge Course Type will always have a sidebar active, by default on the left side.", "wpep" ) . '</p>' .
        '<p data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_STANDARD . '">' . __( "There will always have a sidebar, we highly recommend disabling it if you know what you're doing.", "wpep" ) . '</p>' .
        '<p data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_VIDEO . '">' . __( "There will always be a sidebar while viewing the overview, but disabled during the video library itself.", "wpep" ) . '</p>' .
        '<p data-wpep-course-type-information-block="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_CHALLENGE ] ) . '" class="wpep-not-important">' . sprintf( __( "Normally sidebars are present on the %s" ), 'right' ) . '</p>',
      'id'      => 'wpep_sidebar_position',
      'type'    => 'select',
      'options' => [
        'default' => __( "Default", "wpep" ),
        'right'   => __( 'Right', "wpep" ),
        'bottom'  => __( 'Bottom', "wpep" ),
        'left'    => __( 'Left', "wpep" ),
        'disabled'=> __( 'Disabled', "wpep" )
      ],
      'content_before'  => '<div data-wpep-course-type-information-block="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_LEARN_MAP, WPEP_COURSE_TYPE_FOCUS ] ) . '">',
      'content_after'   => '</div>'
    ];

    $wpep_course_options_fields[ 'wpep_sequential_completion' ] = [
      'name'    => __( 'Lesson Completion', "wpep" ),
      'desc'    => '<p>' . __( "When the Course is Sequential, the Lessons have to be completed in the right order, and lessons cannot be skipped.", "wpep" ) . '</p>',
      'id'      => 'wpep_sequential_completion',
      'type'    => 'select',
      'options'         => [
        '0' => __( "Non Sequential", "wpep" ),
        '1' => __( "Sequential", "wpep" )
      ],
      'content_before'  => '<div data-wpep-course-type-information-block="' . wpep_course_layout_types_string( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP, WPEP_COURSE_TYPE_STANDARD, WPEP_COURSE_TYPE_VIDEO, WPEP_COURSE_TYPE_FOCUS ] ) . '">',
      'content_after'   => '</div>'
    ];

    $wpep_course_options_fields[ 'wpep_lesson_content_on_demand' ] = [
      'name'    => __( 'Load Lesson Content on', "wpep" ),
      'desc'    => '<p>' . __( "In normal circumstances you want to load the Lesson Content on Page Load, if the page load time is too slow, this option will dramatically increase performance, but may conflict with 3-rd party shortcode content.", "wpep" ) . '</p>',
      'id'      => 'wpep_lesson_content_on_demand',
      'type'    => 'select',
      'options'         => [
        '0' => __( "Page Load", "wpep" ),
        '1' => __( "Lesson Open", "wpep" )
      ],
      'content_before'  => '<div data-wpep-course-type-information-block="' . wpep_course_layout_types_string( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_LEARN_MAP ] ) . '">',
      'content_after'   => '</div>'
    ];

    $wpep_course_options_fields[ 'wpep_read_only' ] = [
      'name'    => __( 'Read Only', "wpep" ),
      'desc'    => '<p>' . __( "This course cannot be completed, all features that track progress are hidden.", "wpep" ) . '</p>',
      'id'      => 'wpep_read_only',
      'type'    => 'select',
      'options' => wpep_flag_setting_field_options(),
    ];
    $wpep_course_options_fields[ 'wpep_show_progress_bar' ] = [
      'name'    => __('Show Progress Bar', "wpep"),
      'desc'    => __('Chose to hide or display the course progress bar specifically for this course.', "wpep"),
      'id'      => 'wpep_show_progress_bar',
      'type'    => 'radio_inline',
      'options' => [
        'show'   => __('Show', "wpep"),
        'hide'   => __('Hide', "wpep")
      ],
      'content_before'  => '<div data-wpep-course-type-information-block="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_ONE_OFF, WPEP_COURSE_TYPE_FOCUS, WPEP_COURSE_TYPE_CHALLENGE ] ) . '">',
      'content_after'   => '</div>'
    ];

    if( wpep_get_setting_flag( 'enable-progress-repeat-support' ) ) {
      $wpep_course_options_fields[ 'wpep_progress_repeat_enabled' ] = [
        'name'    => __( 'Progress Repeat', 'wpep' ),
        'id'      => 'wpep_progress_repeat_enabled',
        'type'    => 'select',
        'options' => wpep_flag_setting_field_options(),
        'description' =>  __( "Allow the student to re-take this course", "wpep" )
      ];
    }

    $wpep_course_options_fields[ 'wpep_course_to_ebook_enabled' ] = [
      'name'        => __( 'PDF Download', 'wpep' ),
      'id'          => 'wpep_course_to_ebook_enabled',
      'type'        => 'select',
      'options'     => wpep_flag_setting_field_options(),
      'description' =>  __( 'Allow students to download this course in PDF format, video embeds may not be displayed and the video layout is not yet supported.', 'wpep' ),
      'content_before'  => '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . ' ' . WPEP_COURSE_TYPE_STANDARD . '">',
      'content_after'   => '</div>'
    ];

    $wpep_course_options_fields[ 'wpep_thumb_image' ] = [
      'name' => __('Thumbnail Image', "wpep"),
      'desc' => __('Set a picture that will show on Course Index Page. ( recommended size : 300px width x 190px height )', "wpep"),
      'id'   => 'wpep_thumb_image',
      'type' => 'image',
    ];

    if( wpep_get_setting_flag( 'index-page-enable-hover-image' ) ) {
      $wpep_course_options_fields[ 'wpep_thumb_image_hover' ] = [
        'name' => __('Thumbnail Image - Hover', "wpep"),
        'desc' => __('Set a picture that will show on Course Index Page on hover.', "wpep"),
        'id'   => 'wpep_thumb_image_hover',
        'type' => 'image',
      ];
    }

    $wpep_course_options_fields[ 'wpep_icon_image' ] =[
      'name'            => __('Icon Image', "wpep"),
      'desc'            => __('Set a picture that will show beside the Course Title. ( recommended size : 100px width x 100px height )', "wpep"),
      'id'              =>  'wpep_icon_image',
      'type'            => 'image',
      'content_before'  => '<div data-wpep-course-type-information-block="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . '">',
      'content_after'   => '</div>'
    ];

    if( wpep_get_setting_flag( 'enable-course-tracking-code-option' ) ) {
      $wpep_course_options_fields['wpep_tracking_code']  = [
        'name'    => __('Tracking Code', "wpep"),
        'id'      => 'wpep_tracking_code_completed',
        'type'    => 'textarea',
        'height'  => '150px;',
        'desc'    => __(
          "This is a custom javascript code or HTML code executed when the student has completed the course.",
          "wpep"
        )
      ];
    }

    return $wpep_course_options_fields;
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    global $post;

    $field_value = get_post_meta( $post->ID, $field_key, $single );

    return $field_value == '' ? $field_default : $field_value;
  }

}