<?php

namespace WPEP\ContentType\Skeleton;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Discussion as WPEP_Entity_Discussion;

abstract class Skeleton {

  public $data_post_ignored_meta = [ '_edit_lock', '_edit_last', '_oembed_*' ];

  public function setup() {}

  public function init() {}

  abstract public function get_post_type();
  abstract public function get_singular_name();
  abstract public function get_plural_name();

  public function get_label_all_items() {
    return __('All ' . $this->get_plural_name(), "wpep");
  }

  abstract public function get_cap_name();

  abstract public function register_post_type();

  /**
   * @return false|\WPEP\ContentType\Skeleton\Integration\LearningJourney
   */
  public function get_learning_journey_instance() {
    return false;
  }

  /**
   * @return false|\WPEP\ContentType\Skeleton\Integration\Builder
   */
  public function get_builder_instance() {
    return false;
  }

  public function is_enabled() {
    return true;
  }

  /**
   * @param [
   *  'time_from'        => '',
   *  'time_to'          => '',
   *  'user_ids'         => [] <- Needs to not be empty in order to actually limit by this.,
   *  'post_ids'         => [] <- Needs to not be empty in order to actually limit by this.
   * ] $args
   * @return array
   */
  public function data_user_activity_report( $args ) {
    return [];
  }

  /**
   * @param $args
   * @return array
   */
  public function data_user_post_activity_report( $args ) {
    return [];
  }

  /**
   * This function will return TRUE to have the Action Tracking default kick-in.
   * Return an array to change this.
   *
   * Note : Returning an non-array response will be treated as TRUE
   * @param [
   *  'current_page'     => '',
   *  'pagination_limit' => '',
   *  'time_from'        => '',
   *  'time_to'          => '',
   *  'user_ids'         => [] <- Needs to not be empty in order to actually limit by this.,
   *  'post_ids'         => [] <- Needs to not be empty in order to actually limit by this.
   * ] $args
   * @return true|[
   *    'entries_header' => []
   *    'entries'        => []
   *    'pagination'     => []
   * ]
   */
  public function data_user_post_activity_list( $args ) {
    return true;
  }

  public function data_asset_map() {
    return [];
  }

  public function data_post_connection() {
    return [];
  }

  public function data_export_array() {
    $response = [];

    $post_list = get_posts( [ 'post_type' => $this->get_post_type(), 'numberposts' => -1, 'post_status' => wpep_post_monitored_post_status_list() ] );

    foreach( $post_list as $post )
      $response[ $post->ID ] = $this->data_export_single_post_array( $post );

    return $response;
  }

  public function data_import_array( $post_list, $id_mapping = [] ) {
    $response = [];

    foreach( $post_list as $post ) {
      if( !isset( $post['post_information'] ) )
        continue;

      if( isset( $post['post_id'] ) )
        $response[ $post['post_id'] ] = $this->data_import_single_post_array( $post, $id_mapping );
      else
        $this->data_import_single_post_array( $post, $id_mapping );
    }

    return $response;
  }

  public function data_import_array_post_connection( $post_list, $id_mapping = [] ) {
    $response = [];

    foreach( $post_list as $post ) {
      if( !isset( $post['post_connection'] ) )
        continue;

      $this->data_import_single_post_connection( $post, $id_mapping, $id_mapping[ $post['post_id'] ] );
    }

    return $response;
  }

  public function data_export_single_post_array( $post, $include_user_data = true ) {
    $post_meta_entries = get_post_meta( $post->ID );

    foreach( $this->data_post_ignored_meta as $meta_key_alias ) {
      if( wpep_string_ends_with( $meta_key_alias, '*' ) ) {
        $meta_key_alias = rtrim( $meta_key_alias, '*' );

        foreach( $post_meta_entries as $post_meta_entry_key => $post_meta_entry_value ) {
          if( strpos( $post_meta_entry_key, $meta_key_alias ) === 0 )
            unset( $post_meta_entries[ $post_meta_entry_key ] );
        }

        continue;
      }

      unset( $post_meta_entries[ $meta_key_alias ] );
    }

    $response = [
      'post_id'          => $post->ID,
      'post_information' => [
        'post_content'      => $post->post_content,
        'post_title'        => $post->post_title,
        'post_excerpt'      => $post->post_excerpt,
        'post_status'       => $post->post_status,
        'comment_status'    => $post->comment_status,
        'ping_status'       => $post->ping_status,
        'post_date'         => $post->post_date,
        'post_date_gmt'     => $post->post_date_gmt,
        'post_modified'     => $post->post_modified,
        'post_modified_gmt' => $post->post_modified_gmt,
        'post_type'         => $post->post_type,
        'post_mime_type'    => $post->post_mime_type
      ],
      'meta'             => $post_meta_entries,
      'taxonomies'       => [],
      'post_connection'  => $this->_data_export_single_post_attach_post_connections( $post, $include_user_data )
    ];

    foreach( [ WPEP_TAXONOMY_CATEGORY ] as $taxonomy )
      $response['taxonomies'][$taxonomy] = wp_get_post_terms( $post->ID, $taxonomy, [ 'fields' => 'ids' ] );

    if( $include_user_data ) {
      $post_discussions = WPEP_Entity_Discussion::instance()->get_discussions( [ 'post_id' => $post->ID ] );

      if( !empty( $post_discussions ) )
        $response['discussions'] = $this->_data_export_single_post_attach_discussions( $post_discussions );
    }

    if( method_exists( $this, 'data_export_single_post_array_after' ) )
      $this->data_export_single_post_array_after( $response, $post, $include_user_data );

    return $response;
  }

  private function _data_export_single_post_attach_post_connections( $post, $include_user_data = false ) {
    $post_connections = $this->data_post_connection();

    if( empty( $post_connections ) )
      return [];

    $user_list = wpep_data_system()->get_users();

    $response = [];

    foreach( $post_connections as $post_connection_key => $post_connection ) {
      if( isset( $post_connection[ 'is_user_data' ] ) && $post_connection[ 'is_user_data' ] && !$include_user_data )
        continue;

      if( $post_connection['type'] === 'meta' ) {
        if( $post_connection['location'] === 'external' ) {
          $response[ $post_connection_key ] = wpep_wpdb()->get_col(
            'SELECT post_id FROM `' . wpep_wpdb()->postmeta . '` WHERE `meta_key` = "' . $post_connection['key'] . '" AND meta_value = ' . $post->ID
          );
        } else {
          $response[ $post_connection_key ] = get_post_meta( $post->ID, $post_connection['key'], false );
        }

        continue;
      }

      if( $post_connection['type'] === 'table' ) {

        if( !isset( $post_connection[ 'post_id_column' ] ) ) {
          if( !isset( $post_connection[ 'parent_connection_key' ] ) )
            continue;

          if( !isset( $response[ $post_connection[ 'parent_connection_key' ] ] )
              || !isset( $response[ $post_connection[ 'parent_connection_key' ] ]['ids'] ) )
            continue;

          $entries = wpep_wpdb()->get_results(
            'SELECT entry_table.* 
                     FROM `' . $post_connection[ 'table'] . '` entry_table 
                    WHERE entry_table.`' . $post_connection[ 'child_id_column' ] . '` IN ("' . implode( '","', $response[ $post_connection[ 'parent_connection_key' ] ]['ids'] ) . '")',
            ARRAY_A
          );
        } else {
          $entries = wpep_wpdb()->get_results( 'SELECT entry_table.* FROM `' . $post_connection[ 'table'] . '` entry_table WHERE entry_table.`' . $post_connection[ 'post_id_column' ] . '` = ' . $post->ID, ARRAY_A );
        }

        if( empty( $entries ) )
          continue;

        if( $include_user_data ) {
          $user_id_column = 'user_id';

          if( isset( $post_connection[ 'columns' ] ) && in_array( 'user_id', $post_connection[ 'columns' ] ) ) {
            foreach( $post_connection[ 'columns' ] as $post_connection_column_key => $post_connection_column_value )
              if( $post_connection_column_value === 'user_id' )
                $user_id_column = $post_connection_column_key;
          }

          foreach( $entries as $entry_key => $entry_data ) {
            if( isset( $entry_data[ $user_id_column ] ) && isset( $user_list[ $entry_data[ $user_id_column ] ] ) )
              $entries[ $entry_key ][ 'user_email_address' ] = $user_list[ $entry_data[ $user_id_column ] ]['email_address'];

          }
        }

        $response[ $post_connection_key ] = [
          'ids'     => wpep_utility_map_to_array_assoc( $entries, 'id', 'id' ),
          'entries' => $entries
        ];

        continue;
      }

    }

    return $response;
  }

  /**
   * @param $post_discussions
   * @return array
   */
  private function _data_export_single_post_attach_discussions( $post_discussions ) {
    $response = [];

    $user_list = wpep_data_system()->get_users();

    foreach( $post_discussions as $post_discussion ) {
      $discussion_controller_by_namespace = WPEP_Controller::instance()->discussion->get( $post_discussion->namespace );

      if( $discussion_controller_by_namespace == false )
        continue;

      $identifier_args = $discussion_controller_by_namespace->get_discussion_identifier_args_from_string( $post_discussion->identifier );

      if( $identifier_args === false )
        continue;

      $response[ $post_discussion->identifier ] = [
        'identifier_args' => $identifier_args,
        'namespace'       => $post_discussion->namespace,
        'created_at'      => $post_discussion->created_at,
        'last_activity'   => $post_discussion->last_activity,
        'comments'        => [],
        'user_data'       => []
      ];

      $discussion_comments = WPEP_Entity_Discussion::instance()->get_discussion_comments( [
        'discussion_id' => $post_discussion->id,
      ], [
        'do_not_order_by_pinned' => true,
        'order_by'               => 'parent_id',
        'order'                  => 'ASC'
      ] );

      if( empty( $discussion_comments ) )
        continue;

      $discussion_user_data_list = WPEP_Entity_Discussion::instance()->get_discussion_user_data( false, $post_discussion->id, false, false );

      foreach( $discussion_comments as $discussion_comment ) {
        $discussion_comment_information = [
          'id'                 => $discussion_comment->id,
          'parent_id'          => $discussion_comment->parent_id,
          'user_id'            => $discussion_comment->user_id,
          'user_email_address' => $user_list[ $discussion_comment->user_id ]['email_address'],
          'content'            => $discussion_comment->content,
          'created_at'         => $discussion_comment->created_at,
          'last_activity'      => $discussion_comment->last_activity,
          'is_pinned'          => $discussion_comment->is_pinned,
          'type'               => $discussion_comment->type
        ];

        $discussion_comment_user_data = WPEP_Entity_Discussion::instance()->get_discussion_comment_user_data( false, $discussion_comment->id, false, false );

        if( !empty( $discussion_comment_user_data ) ) {
          $discussion_comment_information[ 'user_data' ] = [];

          foreach( $discussion_comment_user_data as $discussion_comment_user_datum )
            $discussion_comment_information[ 'user_data' ][] = [
              'user_id'            => $discussion_comment_user_datum->user_id,
              'user_email_address' => $user_list[ $discussion_comment->user_id ]['email_address'],
              'key'                => $discussion_comment_user_datum->key,
              'value'              => $discussion_comment_user_datum->value,
              'last_activity'      => $discussion_comment_user_datum->last_activity,
            ];
        }

        $response[ $post_discussion->identifier ][ 'comments' ][ $discussion_comment->id ] = $discussion_comment_information;
      }

      foreach( $discussion_user_data_list as $discussion_user_data ) {
        $response[ $post_discussion->identifier ][ 'user_data' ][] = [
          'user_id'            => $discussion_user_data->user_id,
          'user_email_address' => $user_list[ $discussion_user_data->user_id ]['email_address'],
          'key'                => $discussion_user_data->key,
          'value'              => $discussion_user_data->value
        ];
      }

    }

    return $response;
  }

  /**
   * The Discussions will not be imported automatically.
   * Every content type that integrates discussions should integrate data_import_single_post_discussion
   *
   * @param $post
   * @param array $id_mapping
   * @return int|\WP_Error
   */
  public function data_import_single_post_array( $post, $id_mapping = [] ) {
    $post['post_information']['post_type'] = $this->get_post_type();

    $new_post_id = wp_insert_post( $post['post_information'], true );

    if( $new_post_id == 0 )
      return 0;

    if( isset( $post['meta'] ) && !empty( $post['meta'] ) ) {
      foreach( $post['meta'] as $meta_key => $meta_values ) {
        if( !is_array( $meta_values ) )
          $meta_values = [ $meta_values ];

        foreach( $meta_values as $meta_value )
          add_post_meta( $new_post_id, $meta_key, $meta_value );
      }
    }

    if( isset( $post['taxonomies'] ) && !empty( $post['taxonomies'] ) ) {
      foreach( $post['taxonomies'] as $taxonomy => $taxonomy_information ) {
        if( !isset( $id_mapping[ $taxonomy ] ) )
          continue;

        $term_ids = [];

        foreach( $taxonomy_information as $taxonomy_item ) {
          $taxonomy_term_id = ( is_array( $taxonomy_item ) ? $taxonomy_item['id'] : $taxonomy_item );

          if( !isset( $id_mapping[ $taxonomy ][$taxonomy_term_id] ) )
            continue;

          $term_ids[] = $id_mapping[ $taxonomy ][$taxonomy_term_id];
        }

        if( !empty( $term_ids ) )
          wp_set_post_terms( $new_post_id, $term_ids, $taxonomy );
      }
    }

    return $new_post_id;
  }

  public function data_import_single_post_connection( $post, $post_id_map, $new_post_id = false ) {
    $post_connection_map = $this->data_post_connection();

    if( empty( $post_connection_map ) || $new_post_id == false || !isset( $post['post_connection'] ) )
      return false;

    $post_connection_session_data = [];

    foreach( $post['post_connection'] as $post_connection_key => $post_connection_data ) {
      if( !isset( $post_connection_map[ $post_connection_key ] ) )
        continue;

      $post_connection = $post_connection_map[ $post_connection_key ];

      if( $post_connection['type'] === 'meta' ) {
        foreach( $post_connection_data as $post_connection_column_value ) {
          if( isset( $post_id_map[ $post_connection_column_value ] ) && $post_connection['value_type'] == 'post_id' )
            $post_connection_column_value = $post_id_map[ $post_connection_column_value ];

          if( $post_connection['location'] === 'external' )
            add_post_meta( $post_connection_column_value, $post_connection['key'], $new_post_id );
          else
            add_post_meta( $new_post_id, $post_connection['key'], $post_connection_column_value );
        }

        continue;
      }

      if( $post_connection['type'] === 'table' ) {
        if( !isset( $post_connection_data[ 'entries' ] ) || empty( $post_connection_data[ 'entries' ] ) )
          continue;

        if( isset( $post_connection_session_data[ $post_connection[ 'table_alias' ] ] )
            && !empty( $post_connection_session_data[ $post_connection[ 'table_alias' ] ] ) )
          continue;


        $post_connection_session_data = $this->_data_import_single_post_connection_type_table( $post_connection, $post_connection_data[ 'entries' ], $post_connection_map, $post, $new_post_id, $post_connection_session_data );
      }
    }

    return true;
  }

  private function _data_import_single_post_connection_type_table( $post_connection, $post_connection_entries, $post_connection_map, $post, $new_post_id, $post_connection_session_data = [] ) {
    $user_email_map = [];

    foreach( $post_connection_entries as $post_connection_entry_key => $post_connection_entry_value ) {
      if( isset( $post_connection[ 'post_id_column' ] ) ) {
        $post_connection_entry_value[ $post_connection[ 'post_id_column' ] ] = $new_post_id;
      } elseif( isset( $post_connection[ 'parent_connection_key' ] )
        && isset( $post['post_connection'][ $post_connection[ 'parent_connection_key' ] ] )
        && isset( $post_connection_map[ $post_connection[ 'parent_connection_key' ] ] ) ) {

        if( isset( $post_connection[ 'child_id_column' ] ) && $post_connection_entry_value[ $post_connection[ 'child_id_column' ] ] != 0 ) {

          $parent_table_alias = $post_connection_map[ $post_connection[ 'parent_connection_key' ] ][ 'table_alias' ];

          if( !isset( $post_connection_session_data[ $parent_table_alias . '_id' ] ) ) {
            if( !isset( $post[ 'post_connection' ][ $post_connection[ 'parent_connection_key' ] ] ) )
              continue;

            if( !isset( $post[ 'post_connection' ][ $post_connection[ 'parent_connection_key' ] ][ 'entries' ] )
                || empty( $post[ 'post_connection' ][ $post_connection[ 'parent_connection_key' ] ][ 'entries' ] ) )
              continue;

            $post_connection_session_data = $this->_data_import_single_post_connection_type_table(
              $post_connection_map[ $post_connection[ 'parent_connection_key' ] ],
              $post[ 'post_connection' ][ $post_connection[ 'parent_connection_key' ] ][ 'entries' ],
              $post_connection_map,
              $post,
              $new_post_id,
              $post_connection_session_data
            );

            if( !isset( $post_connection_session_data[ $parent_table_alias ] ) )
              continue;
          }

          if( isset( $post_connection_session_data[ $parent_table_alias ] ) )
            $post_connection_entry_value[ $post_connection[ 'child_id_column' ] ] = $post_connection_session_data[ $parent_table_alias ][ $post_connection_entry_value[ $post_connection[ 'child_id_column' ] ] ];
        }
      }

      if( isset( $post_connection[ 'is_user_data' ] ) && $post_connection[ 'is_user_data' ] ) {
        if( isset( $post_connection_entry_value[ 'user_email_address' ] ) ) {
          if( empty( $user_email_map ) )
            $user_email_map = wpep_data_system()->get_user_ids_mapped_by_email();

          if( !isset( $user_email_map[ $post_connection_entry_value[ 'user_email_address' ] ] ) )
            continue;

          if( isset( $post_connection_entry_value[ 'user_id' ] ) ) {
            $post_connection_entry_value[ 'user_id' ] = $user_email_map[ $post_connection_entry_value[ 'user_email_address' ] ];

            unset( $post_connection_entry_value[ 'user_email_address' ] );
          }
        }
      }

      if( isset( $post_connection[ 'columns' ] ) && !empty( $post_connection[ 'columns' ] ) ) {
        foreach( $post_connection[ 'columns' ] as $column_key => $column_data ) {
          if( !isset( $post_connection_entry_value[ $column_key ] ) )
            continue;

          if( is_array( $column_data ) ) {
            if( !isset( $column_data[ 'where' ] ) || !isset( $column_data[ 'value' ] ) )
              continue;

            foreach( $column_data[ 'where' ] as $where_key => $where_value ) {
              if( !isset( $post_connection_entry_value[ $where_key ] ) )
                continue 2;

              if( $post_connection_entry_value[ $where_key ] != $where_value )
                continue 2;
            }

            $column_data = $column_data[ 'value' ];
          }

          if( $column_data === 'user_id' ) {
            if( isset( $post_connection_entry_value[ 'user_email_address' ] ) ) {
              if( empty( $user_email_map ) )
                $user_email_map = wpep_data_system()->get_user_ids_mapped_by_email();

              if( !isset( $user_email_map[ $post_connection_entry_value[ 'user_email_address' ] ] ) )
                continue;

              $post_connection_entry_value[ 'user_id' ] = $user_email_map[ $post_connection_entry_value[ 'user_email_address' ] ];

              unset( $post_connection_entry_value[ 'user_email_address' ] );
            } elseif( $post_connection_entry_value[ $column_key ] != 0 && isset( $post_connection_entry_value[ 'user_id' ] ) ) {
              $post_connection_entry_value[ $column_key ] = $post_connection_entry_value[ 'user_id' ];
            }
          }

          if( isset( $post_connection_session_data[ $column_data ] ) ) {
            if( !isset( $post_connection_session_data[ $column_data ][ $post_connection_entry_value[ $column_key ] ] ) )
              continue;

            $post_connection_entry_value[ $column_key ] = $post_connection_session_data[ $column_data ][ $post_connection_entry_value[ $column_key ] ];
          }
        }
      }

      $current_id = false;

      if( $post_connection_entry_value[ 'id' ] ) {
        $current_id = $post_connection_entry_value[ 'id' ];
        unset( $post_connection_entry_value[ 'id' ] );
      }

      if( wpep_wpdb()->insert( $post_connection[ 'table' ], $post_connection_entry_value ) === false )
        continue;

      if( $current_id == false )
        continue;

      if( !isset( $post_connection_session_data[ $post_connection[ 'table_alias' ]  . '_id' ] ) )
        $post_connection_session_data[ $post_connection[ 'table_alias' ] . '_id' ] = [];

      $post_connection_session_data[ $post_connection[ 'table_alias' ] . '_id' ][ $current_id ] = wpep_wpdb()->insert_id;
    }

    return $post_connection_session_data;
  }

  /**
   * Ensure that you've changed the identifier_args to match the identifier after import.
   * @param $post_id
   * @param $discussion_data
   * @return int|false
   */
  public function data_import_single_post_discussion( $post_id, $discussion_data ) {
    $student_list = wpep_data_system()->get_user_ids_mapped_by_email();
    $discussion_namespace_instance = WPEP_Controller::instance()->discussion->get( $discussion_data['namespace' ] );

    if( $discussion_namespace_instance === false )
      return false;

    $discussion_id = WPEP_Entity_Discussion::instance()->discussion_id_by_identifier(
      $discussion_namespace_instance->get_discussion_identifier( $discussion_data['identifier_args'] ),
      $post_id,
      $discussion_data['namespace']
    );

    wpep_wpdb()->update( wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION, [
      'created_at'    => $discussion_data['created_at'],
      'last_activity' => $discussion_data['last_activity']
    ], [
      'id' => $discussion_id
    ] );

    if( isset( $discussion_data['comments'] ) && !empty( $discussion_data['comments'] ) ) {
      $discussion_comment_old_new_id_map = [];

      foreach( $discussion_data['comments'] as $discussion_comment_id => $discussion_comment ) {
        if( !isset( $student_list[ $discussion_comment['user_email_address'] ] ) )
          continue;

        if( !isset( $discussion_comment['id'] ) )
          $discussion_comment['id'] = $discussion_comment_id;

        $discussion_comment['parent_id'] = absint( $discussion_comment['parent_id'] );

        if( $discussion_comment['parent_id'] !== 0
            && !isset( $discussion_comment_old_new_id_map[ $discussion_comment['parent_id'] ] ) )
          continue;

        $added_discussion_comment_id = WPEP_Entity_Discussion::instance()->add_discussion_comment( [
          'parent_id'     => ( $discussion_comment['parent_id'] === 0 ? 0 : $discussion_comment_old_new_id_map[ $discussion_comment['parent_id'] ] ),
          'discussion_id' => $discussion_id,
          'user_id'       => $student_list[ $discussion_comment['user_email_address'] ],
          'content'       => $discussion_comment['content'],
          'created_at'    => $discussion_comment['created_at'],
          'last_activity' => $discussion_comment['last_activity'],
          'is_pinned'     => $discussion_comment['is_pinned'],
          'type'          => $discussion_comment['type']
        ], false );

        $discussion_comment_old_new_id_map[ $discussion_comment['id'] ] = $added_discussion_comment_id;

        if( !isset( $discussion_comment[ 'user_data' ] ) )
          continue;

        foreach( $discussion_comment[ 'user_data' ] as $discussion_comment_user_data ) {
          if( !isset( $student_list[ $discussion_user_data['user_email_address'] ] ) )
            continue;

          WPEP_Entity_Discussion::instance()->add_discussion_comment_user_data(
            $discussion_comment_user_data['key'],
            $discussion_comment_user_data['value'],
            $added_discussion_comment_id,
            $student_list[ $discussion_comment_user_data['user_email_address'] ],
            $discussion_comment_user_data[ 'last_activity' ]
          );
        }
      }
    }

    if( isset( $discussion_data['user_data'] ) && !empty( $discussion_data['user_data'] ) ) {
      foreach( $discussion_data['user_data'] as $discussion_user_data ) {
        if( !isset( $student_list[ $discussion_user_data['user_email_address'] ] ) )
          continue;

        WPEP_Entity_Discussion::instance()->add_discussion_user_data(
          $discussion_user_data['key'],
          $discussion_user_data['value'],
          $discussion_id,
          $student_list[ $discussion_user_data['user_email_address'] ]
        );
      }
    }

    return $discussion_id;
  }

}