<?php

namespace WPEP\Coordinator;

class ContentLibrary {

  /**
   * @var \WPEP_Content_Library_Integration[]
   */
  protected $_integrations = [];

  public function setup() {
    add_action( 'set_current_user', [ $this, 'clear_access_storage' ] );
  }

  /**
   * @param \WPEP_Content_Library_Integration $content_library_integration
   */
  public function register( $content_library_integration ) {
    $this->_integrations[ $content_library_integration->prefix ] = $content_library_integration;
  }

  public function is_integrated() {
    return !empty( $this->_integrations );
  }

  /**
   * @return \WPEP_Content_Library_Integration[]
   */
  public function get_integrations(){
    return $this->_integrations;
  }

  /**
   * @param $post_id
   * @return bool
   */
  public function has_access_to_post( $post_id ) {
    $has_access = true;

    foreach( $this->_integrations as $integration ) {
      if( $integration->has_access( $post_id ) )
        continue;

      $has_access = false;
      break;
    }

    return $has_access;
  }

  /**
   * @param $post_id
   * @param $user_id;
   * @return bool
   */
  public function user_has_access_to_post( $post_id, $user_id ) {
    if( $user_id == get_current_user_id() )
      return $this->has_access_to_post( $post_id );

    $has_access = true;

    foreach( $this->_integrations as $integration ) {
      if( $integration->user_has_access( $post_id, $user_id ) )
        continue;

      $has_access = false;
      break;
    }

    return $has_access;
  }

  /**
   * @param $post_id
   * @param $user_id
   * @return int|bool
   */
  public function get_post_access_source_user_id( $post_id, $user_id ) {
    foreach( $this->_integrations as $integration ) {
      $response = $integration->get_post_access_source_user_id( $post_id, $user_id );

      if( null !== $response )
        return $response;
    }

    return null;
  }

  /**
   * @param $post_id
   * @param $owner_user_id
   * @return array
   */
  public function get_sub_account_user_ids_by_post_id( $post_id, $owner_user_id ) {
    foreach( $this->_integrations as $integration ) {
      $response = $integration->get_sub_account_user_ids_by_post_id( $post_id, $owner_user_id );

      if( is_array( $response ) )
        return $response;
    }

    return [];
  }

  public function get_sub_account_receiver_user_ids_from_provider_user_id( $owner_user_id ) {
    foreach( $this->_integrations as $integration ) {
      $response = $integration->get_sub_account_receiver_user_ids_from_provider_user_id( $owner_user_id );

      if( is_array( $response ) )
        return $response;
    }

    return [];
  }

  /**
   * @param $owner_user_id
   * @param $receiver_user_id
   * @return array|bool|null
   */
  public function has_sub_account_access_from_user_id( $owner_user_id, $receiver_user_id ) {
    foreach( $this->_integrations as $integration ) {
      $response = $integration->has_sub_account_access_from_user_id( $owner_user_id, $receiver_user_id );

      if( null !== $response )
        return $response;
    }

    return false;
  }

  /**
   * @param $post_id
   * @param $owner_user_id
   * @param $receiver_user_id
   * @return array|bool|null
   */
  public function has_sub_account_access_to_post_id_from_user_id( $post_id, $owner_user_id, $receiver_user_id ) {
    foreach( $this->_integrations as $integration ) {
      $response = $integration->has_sub_account_access_to_post_id_from_user_id( $post_id, $owner_user_id, $receiver_user_id );

      if( null !== $response )
        return $response;
    }

    return false;
  }

  /**
   * @param $post_id
   * @return bool
   */
  public function get_default_sales_page( $post_id ) {
    foreach( $this->_integrations as $integration ) {
      if( $integration->_has_access( $post_id ) )
        continue;

      $link = $integration->get_sell_link( $post_id);

      if( $link !== '' )
        return $link;
    }

    return get_permalink( $post_id );
  }

  /**
   * @param $post_id
   * @param null $user_id
   * @return array
   */
  public function access_to_post_map( $post_id, $user_id = null ) {
    $response = [
      'has_access'     => true,
      'purchase_text'  => '',
      'purchase_link'  => ''
    ];

    foreach( $this->_integrations as $integration ) {
      if( $integration->user_has_access( $post_id, $user_id ) )
        continue;

      $response['has_access']     = false;
      $response['purchase_text']  = $integration->get_sell_text( $post_id );
      $response['purchase_link']  = $integration->get_sell_link( $post_id );

      return $response;
    }

    return $response;
  }

  public function clear_access_storage() {
    foreach( $this->_integrations as $integration )
      $integration->clear_access_storage();
  }

}