<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class Focus extends Skeleton\Navigation {

  public $_template_args = [
    'downloads_list' => [
      'downloads_template' => WPEP_COURSE_DOWNLOADS_TEMPLATE_ICON
    ]
  ];

  private $_focus_navigation_data = [];

  /**
   * Standard constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->course_has_overview_page       = true;
    $this->course_has_completion_redirect = true;
  }

  public function has_sidebar_content() {
    return $this->current_item !== null;
  }

  public function has_default_sidebar_content() {
    return false;
  }

  public function get_alias() {
    return "focus";
  }

  public function init() {
    parent::init();

    if( $this->has_sidebar_content() ) {
      add_action( 'wpep_single_content_before_primary_container', [ $this, 'display_header_content' ], 20 );
      add_filter( 'wpep_single_content_type_attr_classes', [ $this, '_content_type_attr_classes' ], 100 );
      add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    }
  }

  public function close() {
    parent::close();

    if( $this->has_sidebar_content() ) {
      remove_action( 'wpep_single_content_before_primary_container', [ $this, 'display_header_content' ], 20 );
      remove_filter( 'wpep_single_content_type_attr_classes', [ $this, '_content_type_attr_classes' ], 100 );
      remove_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    }
  }

  public function content() {
    if( $this->current_item == $this->lesson_slug ) {
      $course_lesson = $this->skeleton_navigation_current_lesson();

      if( $course_lesson === null )
        return '';

      return $this->skeleton_navigation_current_item_content_prepare( WPEP_Controller::instance()->template->get_template( 'course-single/focus-lesson-content.php', [
        'course_id'             => $this->course_id,
        'course_read_only'      => $this->read_only,
        'course_lesson'         => $course_lesson
      ] ), [
        'discussion_title'      => wpep_get_setting_translated( 'course-focus-discussion-title' )
      ] );
    }

    if( $this->current_item == $this->assessment_slug
        && $this->course_id == wpep_get_assessment_course_id( $this->current_item_id ) ) {
      $assessment_section_id = wpep_get_assessment_course_section_id( $this->current_item_id );

      if( $assessment_section_id == 0
          || ( isset( $this->course_sections_map[ $assessment_section_id ] )
              && isset( $this->course_sections_map[ $assessment_section_id ][ 'assessment_id' ] )
              && $this->course_sections_map[ $assessment_section_id ][ 'assessment_id' ] == $this->current_item_id )
      ) {
        return WPEP_Controller::instance()->template->get_template( 'course-single/focus-assessment-content.php', [
          'course_id'             => $this->course_id,
          'assessment_id'         => $this->current_item_id,
          'previous_lesson_id'    => $this->_get_current_previous_lesson_id(),
          'next_lesson_id'        => $this->_get_current_next_lesson_id()
        ] );
      }
    }

    $args = [
      'course_id'         => $this->course_id,
      'course_sections'   => $this->course_sections_map,
      'course_title'      => get_the_title( $this->course_id ),
      'course_content'    => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
    ];

    return WPEP_Controller::instance()->template->get_template( 'course-single/focus-content-overview.php', $args );
  }

  public function display_header_content( $course_id ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    echo WPEP_Controller::instance()->template->get_template( 'course-single/focus-content-item-header.php', [
      'previous_lesson_id'        => $this->_get_current_previous_lesson_id(),
      'next_lesson_id'            => $this->_get_current_next_lesson_id(),
      'current_item'              => $this->current_item,
      'current_item_is_completed' => $this->_get_current_item_is_completed(),
      'course_id'                 => $this->course_id,
      'current_lesson_id'         => ( $this->current_item === $this->lesson_slug ? $this->current_item_id : 0 ),
      'current_assessment_id'     => ( $this->current_item === $this->assessment_slug ? $this->current_item_id : 0 ),
      'course_sections'           => $this->course_sections_map,
      'is_read_only'              => $this->read_only
    ] );
  }

  public function display_sidebar_content( $course_id ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    echo WPEP_Controller::instance()->template->get_template( 'course-single/focus-lesson-sidebar.php', [
      'course_id'         => $this->course_id,
      'current_lesson_id' => ( $this->current_item === $this->lesson_slug ? $this->current_item_id : 0 ),
      'current_assessment_id' => ( $this->current_item === $this->assessment_slug ? $this->current_item_id : 0 ),
      'course_sections'   => $this->course_sections_map
    ] );
  }

  public function _content_type_attr_classes( $classes = [] ) {
    $classes['sidebar-position'] = 'sidebar-position-left';

    return $classes;
  }

  private function _setup_navigation_data() {
    $previous_lesson_id = null;
    $next_lesson_id = null;
    $current_item_is_completed = null;

    if( $this->current_item == $this->assessment_slug && $this->course_id == wpep_get_assessment_course_id( $this->current_item_id ) ) {
      $assessment_section_id = wpep_get_assessment_course_section_id($this->current_item_id);

      if ($assessment_section_id == 0) {
        $section_map = array_keys($this->course_sections_map);
        end($section_map);
        $previous_lesson_id = $this->_get_last_lesson_id_by_section_id($section_map[key($section_map)]);
      } elseif (!wpep_course_assessments_required($this->course_id) || wpep_user_assessment_status($this->current_item_id, get_current_user_id()) === WPEP_USER_ASSESSMENT_STATUS_COMPLETED) {
        $previous_lesson_id = $this->_get_last_lesson_id_by_section_id($assessment_section_id);
        $next_lesson_id = null;

        $last_section_id = 0;
        foreach ($this->course_sections_map as $section_id => $section_information) {
          if ($last_section_id === $assessment_section_id) {
            $next_lesson_id = $this->_get_first_lesson_id_by_section_id($section_id);
            break;
          }

          $last_section_id = $section_id;
        }
      }
    } elseif( $this->current_item === $this->lesson_slug ) {
      $course_lesson = $this->skeleton_navigation_current_lesson();

      if( $course_lesson !== null ) {
        if( isset( $course_lesson->previous_lesson_id ) )
          $previous_lesson_id = $course_lesson->previous_lesson_id;

        if( isset( $course_lesson->next_lesson_id ) )
          $next_lesson_id = $course_lesson->next_lesson_id;

        if( isset( $course_lesson->is_completed ) )
          $current_item_is_completed = intval( $course_lesson->is_completed );
      }
    }

    $this->_focus_navigation_data = [
      'previous_lesson_id' => $previous_lesson_id,
      'next_lesson_id'     => $next_lesson_id,
      'current_item_is_completed' => $current_item_is_completed
    ];
  }

  private function _get_current_previous_lesson_id() {
    if( empty( $this->_focus_navigation_data ) )
      $this->_setup_navigation_data();

    return ( isset( $this->_focus_navigation_data[ 'previous_lesson_id' ] ) ? $this->_focus_navigation_data[ 'previous_lesson_id' ] : null );
  }

  private function _get_current_next_lesson_id() {
    if( empty( $this->_focus_navigation_data ) )
      $this->_setup_navigation_data();

    return ( isset( $this->_focus_navigation_data[ 'next_lesson_id' ] ) ? $this->_focus_navigation_data[ 'next_lesson_id' ] : null );
  }

  private function _get_current_item_is_completed() {
    if( empty( $this->_focus_navigation_data ) )
      $this->_setup_navigation_data();

    return ( isset( $this->_focus_navigation_data[ 'current_item_is_completed' ] ) ? $this->_focus_navigation_data[ 'current_item_is_completed' ] : null );
  }

  private function _get_first_lesson_id_by_section_id( $section_id ) {
    if( !isset( $this->course_sections_map[ $section_id ] ) )
      return null;

    $section = $this->course_sections_map[ $section_id ];

    if( isset( $section[ 'lessons' ] ) ) {
      $lesson_map  = array_keys( $section[ 'lessons' ] );

      return $lesson_map[ key( $lesson_map ) ];
    }

    return null;
  }

  private function _get_last_lesson_id_by_section_id( $section_id ) {
    if( !isset( $this->course_sections_map[ $section_id ] ) )
      return null;

    $section = $this->course_sections_map[ $section_id ];

    if( isset( $section[ 'lessons' ] ) ) {
      $lesson_map  = array_keys( $section[ 'lessons' ] );
      end( $lesson_map );

      return $lesson_map[ key( $lesson_map ) ];
    }

    return null;
  }

  public function has_creator_toolbox() {
    return false;
  }

}