<?php

namespace WPEP\PlatformHealth;

use WPEP\Settings as WPEP_Settings;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

class Maintenance {

  public function get_report() {
    $old_options            = WPEP_Settings::instance()->get_legacy_options();
    $old_custom_css_files   = count( wpep_get_custom_style_old_files() );
    $empty_metadata         = intval( wpep_wpdb()->get_var( 'SELECT COUNT(*) FROM `' . wpep_wpdb()->postmeta . '` WHERE ( `meta_key` LIKE "wpep_%" AND `meta_value` = "" ) OR ( `meta_key` LIKE "_wpep_%" AND `meta_value` = "" )' ) );
    $system_log_old_entries = WPEP_Entity_SystemLog::get_old_entries_count();
    $junk_post_meta         = intval( wpep_wpdb()->get_var( 'SELECT COUNT(meta_id) FROM ' . wpep_wpdb()->postmeta . ' WHERE post_id != 0 AND post_id NOT IN ( SELECT ID FROM ' . wpep_wpdb()->posts . ')' ) );
    $junk_term_meta         = intval( wpep_wpdb()->get_var( 'SELECT COUNT(meta_id) FROM ' . wpep_wpdb()->termmeta . ' WHERE term_id != 0 AND term_id NOT IN ( SELECT term_id FROM ' . wpep_wpdb()->terms . ')' ) );
    $junk_user_meta         = intval( wpep_wpdb()->get_var( 'SELECT COUNT(umeta_id) FROM ' . wpep_wpdb()->usermeta . ' WHERE user_id != 0 AND user_id NOT IN ( SELECT ID FROM ' . wpep_wpdb()->users . ')' ) );

    $response = [];

    if( empty( $old_options ) ) {
      $response['settings_junk'] = [
        'status'  => 'success',
        'title'   => __( "Settings Junk - No Entries", "wpep" )
      ];
    } else {
      $response['settings_junk'] = [
        'status'        => 'danger',
        'title'         => sprintf( __( "Settings Junk - %s Entries", "wpep" ), '<strong>' . count( $old_options ) . '</strong>' ),
        'operation'     => function() {
          $old_options = WPEP_Settings::instance()->get_legacy_options();

          if( !empty( $old_options ) ) {
            WPEP_Settings::instance()->delete_setting( $old_options );

            return [
              'notification'      => sprintf( __( "Removed %s old %s Settings.", "wpep" ), '<strong>' . count( $old_options ) . '</strong>', WPEP_PLUGIN_NAME ),
              'notification_type' => 'success',
            ];
          }

          return true;
        }
      ];
    }

    if( $empty_metadata === 0 ) {
      $response['metadata_junk'] = [
        'status'  => 'success',
        'title'   => sprintf( __( "Empty %s Posts Metadata - No Entries", "wpep" ), WPEP_PLUGIN_NAME )
      ];
    } else {
      $response['metadata_junk'] = [
        'status'        => 'warning',
        'title'         => sprintf( __( "Empty %s Posts Metadata - %s Entries", "wpep" ), WPEP_PLUGIN_NAME, '<strong>' . $empty_metadata . '</strong>' ),
        'operation'     => function() {
          $count = wpep_wpdb()->query( 'DELETE FROM `' . wpep_wpdb()->postmeta . '` WHERE ( `meta_key` LIKE "wpep_%" AND `meta_value` = "" ) OR ( `meta_key` LIKE "_wpep_%" AND `meta_value` = "" )' );

          if( $count == 0 )
            return [
              'notification'      => __( "Empty metadata entries have been already removed.", "wpep" ),
              'notification_type' => 'info',
            ];

          return [
            'notification'      => sprintf( __("Removed %s empty metadata entries.", "wpep"), '<strong>' . $count . '</strong>' ),
            'notification_type' => 'success',
          ];
        }
      ];
    }

    if( $old_custom_css_files == 0 ) {
      $response['css_file_junk'] = [
        'status'  => 'success',
        'title'   => __( "Custom CSS - No Old Files", "wpep" )
      ];
    } else {
      $response['css_file_junk'] = [
        'status'        => 'warning',
        'title'         => sprintf( __( "Custom CSS - %s Old Files", "wpep" ), '<strong>' . $old_custom_css_files . '</strong>' ),
        'operation'     => function() {
          $old_custom_css_files = wpep_get_custom_style_old_files();

          if( count( $old_custom_css_files ) != 0 ) {
            $custom_css_file_directory = wpep_get_upload_dir();

            foreach( $old_custom_css_files as $custom_css_file ) {
              if( in_array( $custom_css_file, [ '.', '..' ] ) )
                continue;

              if( file_exists( $custom_css_file_directory . '/custom-stylesheet/' . $custom_css_file ) )
                unlink( $custom_css_file_directory . '/custom-stylesheet/' . $custom_css_file );

              if( file_exists( $custom_css_file_directory . '/' . $custom_css_file ) )
                unlink( $custom_css_file_directory . '/' . $custom_css_file );
            }

            return [
              'notification'      => sprintf(__("Removed %s old %s Custom CSS Files.", "wpep"), '<strong>' . count( $old_custom_css_files ) . '</strong>', WPEP_PLUGIN_NAME ),
              'notification_type' => 'success',
            ];
          }

          return true;
        }
      ];
    }

    if( empty( $system_log_old_entries ) ) {
      $response['system_log_old_entries'] = [
        'status'  => 'success',
        'title'   => __( "Old System Logs - No Entries", "wpep" )
      ];
    } else {
      $response['system_log_old_entries'] = [
        'status'        => 'warning',
        'title'         => sprintf( __( "Old System Logs - %s Entries", "wpep" ), '<strong>' . $system_log_old_entries . '</strong>' ),
        'operation'     => function() {
          WPEP_Entity_SystemLog::delete_old_entries();

          return [
            'notification'      => __( "Old System Log Entries have been cleared.", "wpep" ),
            'notification_type' => 'success',
          ];
        },
        'operation_text'=> __( "Cleanup", "wpep" )
      ];
    }

    if( $junk_post_meta == 0 ) {
      $response['junk_post_meta'] = [
        'status'  => 'success',
        'title'   => __( "Orphaned Post Meta - No Entries", "wpep" )
      ];
    } else {
      $response['junk_post_meta'] = [
        'status'        => 'warning',
        'title'         => sprintf( __( "Orphaned Post Meta - %s Entries", "wpep" ), '<strong>' . $junk_post_meta . '</strong>' ),
        'operation'     => function() {
          $count = wpep_wpdb()->query( 'DELETE FROM `' . wpep_wpdb()->postmeta . '` WHERE post_id != 0 AND post_id NOT IN ( SELECT ID FROM ' . wpep_wpdb()->posts . ')' );

          if( $count == 0 )
            return [
              'notification'      => __( "Orphaned Post Meta have been already removed.", "wpep" ),
              'notification_type' => 'info',
            ];

          return [
            'notification'      => sprintf( __("Removed %s Orphaned Post Meta.", "wpep"), '<strong>' . $count . '</strong>' ),
            'notification_type' => 'success',
          ];
        },
        'operation_text'=> __( "Cleanup", "wpep" )
      ];
    }

    if( $junk_term_meta == 0 ) {
      $response['junk_term_meta'] = [
        'status'  => 'success',
        'title'   => __( "Orphaned Term Meta - No Entries", "wpep" )
      ];
    } else {
      $response['junk_term_meta'] = [
        'status'        => 'warning',
        'title'         => sprintf( __( "Orphaned Post Meta - %s Entries", "wpep" ), '<strong>' . $junk_post_meta . '</strong>' ),
        'operation'     => function() {
          $count = wpep_wpdb()->query( 'DELETE FROM `' . wpep_wpdb()->termmeta . '` WHERE term_id != 0 AND term_id NOT IN ( SELECT term_id FROM ' . wpep_wpdb()->terms . ')' );

          if( $count == 0 )
            return [
              'notification'      => __( "Orphaned Term Meta have been already removed.", "wpep" ),
              'notification_type' => 'info',
            ];

          return [
            'notification'      => sprintf( __("Removed %s Orphaned Term Meta.", "wpep"), '<strong>' . $count . '</strong>' ),
            'notification_type' => 'success',
          ];
        },
        'operation_text'=> __( "Cleanup", "wpep" )
      ];
    }

    if( $junk_user_meta == 0 ) {
      $response['junk_user_meta'] = [
        'status'  => 'success',
        'title'   => __( "Orphaned User Meta - No Entries", "wpep" )
      ];
    } else {
      $response['junk_user_meta'] = [
        'status'        => 'warning',
        'title'         => sprintf( __( "Orphaned User Meta - %s Entries", "wpep" ), '<strong>' . $junk_post_meta . '</strong>' ),
        'operation'     => function() {
          $count = wpep_wpdb()->query( 'DELETE FROM `' . wpep_wpdb()->usermeta . '` WHERE user_id != 0 AND user_id NOT IN ( SELECT ID FROM ' . wpep_wpdb()->users . ')' );

          if( $count == 0 )
            return [
              'notification'      => __( "Orphaned User Meta have been already removed.", "wpep" ),
              'notification_type' => 'info',
            ];

          return [
            'notification'      => sprintf( __("Removed %s Orphaned User Meta.", "wpep"), '<strong>' . $count . '</strong>' ),
            'notification_type' => 'success',
          ];
        },
        'operation_text'=> __( "Cleanup", "wpep" )
      ];
    }

    $response['oembed_cache'] = [
      'status'        => 'info',
      'title'         => sprintf( __( "oEmbed Cache - %s Entries", "wpep" ), '<strong>' . $this->_get_oembed_cache_field_count() . '</strong>' ),
      'operation'     => function() {
        $ids = wpep_wpdb()->get_col( 'SELECT id FROM `' . wpep_wpdb()->posts . '` WHERE `post_type` = "oembed_cache"' );

        if( !empty( $ids ) ) {
          foreach( $ids as $id )
            wp_delete_post( $id );
        }

        wpep_wpdb()->query( 'DELETE FROM `' . wpep_wpdb()->postmeta . '` WHERE `meta_key` LIKE "_oembed%"' );

        wp_cache_flush();

        return [
          'notification'      => __( "oEmbed cache has been cleared.", "wpep" ),
          'notification_type' => 'success',
        ];
      },
      'operation_title' => __( "Reset", "wpep" )
    ];

    return apply_filters( 'wpep_platform_health_maintenance_report', $response );
  }

  public function get_report_data() {
    $response = $this->get_report();

    foreach( $response as $response_key => $response_data ) {
      if( !isset( $response_data[ 'operation'] ) )
        continue;

      if( is_callable( $response[ $response_key ][ 'operation' ] ) )
        $response[ $response_key ][ 'operation' ] = 1;
    }

    return $response;
  }

  private function _get_oembed_cache_field_count() {
    $count = intval( wpep_wpdb()->get_var( 'SELECT COUNT(*) FROM `' . wpep_wpdb()->posts . '` WHERE `post_type` = "oembed_cache"' ) );
    $count += intval( wpep_wpdb()->get_var( 'SELECT COUNT(*) FROM `' . wpep_wpdb()->postmeta . '` WHERE `meta_key` LIKE "_oembed%"' ) );

    return $count;
  }

}