<?php

namespace WPEP\PlatformHealth;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEPCronController as WPEP_Cron;

class Service {

  private $_notification_count_transient_name = 'wpep_platform_health_error_detection';

  public function get_notification_count() {
    $error_count = ( isset( $_GET['page'] ) && $_GET['page'] == 'wpep-platform-health' ? false : get_transient( $this->_notification_count_transient_name ) );

    if( $error_count !== false )
      return intval( $error_count );

    $error_count = 0;

    foreach( $this->get_report() as $service_status_information ) {
      if( !isset( $service_status_information['status'] ) )
        continue;

      if( $service_status_information['status'] !== 'danger' )
        continue;

      $error_count++;
    }

    set_transient( $this->_notification_count_transient_name, $error_count, HOUR_IN_SECONDS );

    return $error_count;
  }

  public function reset_notification_count_transient() {
    delete_transient( $this->_notification_count_transient_name );
  }

  /**
   * @return array
   */
  public function get_report() {
    $is_http2 = wpep_link_is_http2( WPEP_BASE_URL_PATH . 'interface/app.min.js' );

    $response = [
      'database_integrity' => [
        'title'             => __( "Database Integrity", "wpep" ),
        'status'            => 'success'
      ],
      'user_roles_integrity'  => [
        'title'             => __( "User Roles", "wpep" ),
        'status'            => ( !in_array( 'error', WPEP_Controller::instance()->capabilities->get_role_list() ) ? 'success' : 'danger' ),
        'operation_title'   => __( "Refresh", "wpep" ),
        'operation'         => function() {
          if( WPEP_Controller::instance()->capabilities->register_caps_and_roles() )
            return [
              'notification'      => sprintf( __( "%s User Roles & User Capabilities have been refreshed.", "wpep" ), WPEP_PLUGIN_NAME ),
              'notification_type' => 'success',
            ];

          return [
            'notification'      => sprintf( __( "There was a problem refreshing the %s User Roles & User Capabilities.", "wpep" ), WPEP_PLUGIN_NAME ),
            'notification_type' => 'danger',
          ];
        }
      ],
      'rewrite_rules' => [
        'title'             => __( "Rewrite Rules ( Post Permalinks )", "wpep" ),
        'status'            => ( wpep_content_type()->are_registered_content_types_reachable()  ? 'success' : 'danger' ),
        'operation_title'   => __( "Reset", "wpep" ),
        'operation'         => function() {
          flush_rewrite_rules();

          return [
            'notification'      => __( "WordPress Rewrite Rules have been refreshed.", "wpep" ),
            'notification_type' => 'success',
          ];
        }
      ],
      'http2'     => [
        'title'  => sprintf( __( "HTTP/2 Support : %s", "wpep" ), ( $is_http2 === null ? __( "Cannot determine.", "wpep" ) : ( $is_http2 ? __( "Yes", "wpep" ) : __( "No", "wpep" ) ) ) ),
        'status' => ( $is_http2 === null ? 'warning' : ( $is_http2 ? 'success' : 'info' ) ),
      ],
      'cron_jobs' => [
        'title'   => __( "Cron Jobs - Not required", "wpep" ),
        'status'  => 'info'
      ],
      'custom_css_delivery'  => [
        'title'             => __( "Custom CSS Delivered by File", "wpep" ),
        'status'            => 'success',
        'operation_title'   => __( "Deliver on Page", "wpep" ),
        'operation'         => function() {
          WPEP_Settings::instance()->update_settings( [
            'wpep-custom-css-by-file' => 0
          ] );

          return [
            'notification'      => __( "Custom CSS & Color Variations will now be delivered on page.", "wpep" ),
            'notification_type' => 'info',
          ];
        }
      ],
      'course_use_content_filter' => [
        'title'             => __( "Deliver Course Content by Filtering - Disabled", "wpep" ),
        'status'            => 'info',
        'operation_title'   => __( "Enable", "wpep" ),
        'operation'         => function() {
          WPEP_Settings::instance()->update_settings( [
            'course-use-the-content-filter' => 1
          ] );

          return [
            'notification'      => __( "The Course Content will now be delivered using the_content filter.", "wpep" ),
            'notification_type' => 'success',
          ];
        },
      ]
    ];

    if( WPEP_Controller::instance()->adminController->migration->is_database_integrity_check_required() ) {
      $response[ 'database_integrity' ][ 'status' ] = 'danger';
      $response[ 'database_integrity' ][ 'operation_title' ] = __( "Resolve", "wpep" );
      $response[ 'database_integrity' ][ 'operation' ] = function() {
        WPEP_Controller::instance()->adminController->migration->handle_database_integrity_check();

        return [
          'notification'      => __( "Database Integrity Issues have been resolved, if the issue persists, ensure the DB user has the right to modify tables.", "wpep" ),
          'notification_type' => 'info',
        ];
      };
    }

    if( WPEP_Cron::has_jobs() ) {
      $cron_jobs_status = WPEP_Cron::get_jobs_status();

      if( $cron_jobs_status === false ) {
        $response[ 'cron_jobs' ] = [
            'title'             => __( "Cron Jobs - Reset Required", "wpep" ),
            'operation_title'   => __( "Reset", "wpep" ),
            'operation'         => function() {
              WPEP_Cron::reset_cron_events();

              return [
                'notification'      => sprintf( __( "%s Cron Jobs have been reset.", "wpep" ), WPEP_PLUGIN_NAME ),
                'notification_type' => 'success',
              ];
            },
            'status'            => 'danger'
          ];
      } elseif( $cron_jobs_status === null ) {
        $response[ 'cron_jobs' ] = [
          'title'   => __( "Cron Jobs - Status Unknown", "wpep" ),
          'status'  => 'warning'
        ];
      } elseif( !WPEP_Cron::is_job_list_executing() ) {
        $doing_cron_transient = get_transient( 'doing_cron' );

        if( false !== $doing_cron_transient && $doing_cron_transient + 120 < time() ) {
          $response[ 'cron_jobs' ] = [
            'title'             => sprintf( __( "Cron Jobs - Executing since %s", "wpep" ), wpep_format_date_with_time( $doing_cron_transient ) ),
            'operation_title'   => __( "Learn More", "wpep" ),
            'operation'         => WPEP_DOC_CRON_JOBS_NOT_WORKING,
            'status'            => 'warning'
          ];
        } else {
          $response[ 'cron_jobs' ] = [
            'title'             => __( "Cron Jobs - Not Executing", "wpep" ),
            'operation_title'   => __( "Learn More", "wpep" ),
            'operation'         => WPEP_DOC_CRON_JOBS_NOT_WORKING,
            'status'            => 'danger'
          ];
        }
      } elseif( ! ( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON ) ) {
        $response[ 'cron_jobs' ] = [
          'title'           => __( "Cron Jobs - WP Triggered", "wpep" ),
          'operation_title' => __( "Learn More", "wpep" ),
          'operation'       => WPEP_DOC_CRON_JOBS_FROM_SERVER,
          'status'          => 'info'
        ];
      } else {
        $response[ 'cron_jobs' ] = [
          'title'  => __( "Cron Jobs - Server Triggered", "wpep" ),
          'status' => 'success'
        ];
      }
    }

    if( !wpep_get_setting_flag( 'wpep-custom-css-by-file' ) ) {
      $response[ 'custom_css_delivery' ] = [
        'title'             => __( "Custom CSS Delivered on Page", "wpep" ),
        'status'            => 'info',
        'operation_title'   => __( "Deliver on File", "wpep" ),
        'operation'         => function() {
          WPEP_Settings::instance()->update_settings( [
            'wpep-custom-css-by-file' => 1
          ] );

          return [
            'notification'      => __( "Custom CSS & Color Variations will now be delivered from a file.", "wpep" ),
            'notification_type' => 'success',
          ];
        }
      ];
    } else if( wpep_get_custom_style_file() == false ) {
      $response[ 'custom_css_delivery' ][ 'title' ]  = __( "Custom CSS cannot be delivered from file", "wpep" );
      $response[ 'custom_css_delivery' ][ 'status' ] = 'danger';
    }

    if( wpep_get_setting_flag( 'course-use-the-content-filter' ) ) {
      $response[ 'course_use_content_filter' ]['title']   = __( "Deliver Course Content by Filtering", "wpep" );
      $response[ 'course_use_content_filter' ]['status']  = 'success';
      $response[ 'course_use_content_filter' ]['operation_title'] = __( "Disable", "wpep" );
      $response[ 'course_use_content_filter' ]['operation'] = function() {
        WPEP_Settings::instance()->update_settings( [
          'course-use-the-content-filter' => 0
        ] );

        return [
          'notification'      => __( "The Course Content will now be delivered without using the_content filter.", "wpep" ),
          'notification_type' => 'info',
        ];
      };
    }

    $response[ 'ttf_fonts' ] = [
      'title'             => __( "TTF Font Library", "wpep" ),
      'status'            => ( wpep_image_builder_get_font_directory_path() === false ? 'danger' : 'success' )
    ];

    $response[ 'product_licensing' ] = [
      'title'             => sprintf( __( "%s Licensed", "wpep" ), WPEP_PLUGIN_NAME ),
      'status'            => ( wpep_controller()->licenseLibrary->has_valid_licenses() ? 'success' : 'danger' )
    ];

    return $response;
  }

  public function get_report_data() {
    $response = $this->get_report();

    foreach( $response as $response_key => $response_data ) {
      if( !isset( $response_data[ 'operation'] ) )
        continue;

      if( is_callable( $response[ $response_key ][ 'operation' ] ) )
        $response[ $response_key ][ 'operation' ] = 1;
    }

    return $response;
  }

}