<?php

namespace WPEP;

/**
 * Class Request
 * @package WPEP
 * @author Robert Rusu
 */
class Request {

  protected static $instance;

  public static function instance() {
    if (!isset(self::$instance))
      self::$instance = new self();

    return self::$instance;
  }

  /**
   * Used to store any key -> value based type information while handling the current request.
   * @var array
   */
  protected $_current_request_setting = [];

  private $_permalink_structure = '';
  private $_current_post_id     = null;
  private $_current_post_type   = null;
  private $_url_to_post_id      = [];

  public function __construct() {
    $this->_permalink_structure = get_option('permalink_structure');
  }

  /**
   * @param string $type : admin|ajax|cron|frontend
   * @return bool|null
   */
  public function is_request_type( $type ) {
    if( $type == 'admin' )
      return is_admin() && !defined( 'DOING_AJAX' );

    if( $type == 'ajax' )
      return defined( 'DOING_AJAX' );

    if( $type == 'cron' )
      return defined( 'DOING_CRON' );

    if( $type == 'frontend' )
      return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );

    return null;
  }

  /**
   * @param int|string $post_id
   */
  public function set_current_post_id( $post_id ) {
    $this->_current_post_id = $post_id;
  }

  /**
   * @return int
   */
  public function get_current_post_id() {
    if( $this->_current_post_id === NULL ) {
      if( $this->is_request_type( 'admin' ) ) {
        $this->_current_post_id = 0;

        return $this->_current_post_id;
      }

      $current_url = $this->get_current_url();

      $this->_current_post_id = $this->get_post_id_by_url( $current_url );
    }
    return $this->_current_post_id;
  }

  /**
   * @return int
   */
  public function get_current_post_type() {
    if( $this->_current_post_type === NULL ) {
      $post_id = $this->get_current_post_id();

      if( $post_id == 0 ) {
        $this->_current_post_type = false;

        return $this->_current_post_type;
      }

      $this->_current_post_type = get_post_type( $post_id );
    }
    return $this->_current_post_type;
  }

  /**
   * @param $current_url
   * @return int
   */
  public function get_post_id_by_url( $current_url ) {
    if( isset( $this->_url_to_post_id[ $current_url ] ) )
      return $this->_url_to_post_id[ $current_url ];

    $post_id = 0;

    if( $this->_permalink_structure === ''
      && strpos( $current_url, '?' ) !== false ) {
      $current_url_parse = parse_url( $current_url );
      parse_str( $current_url_parse['query'], $current_url_query_parse );

      foreach( $current_url_query_parse as $url_param_key => $url_param_value ) {
        if( !wpep_content_type()->is_enabled( $url_param_key ) )
          continue;

        $page_by_path = get_page_by_path( $url_param_value, OBJECT, $url_param_key );

        if( $page_by_path !== null && isset( $page_by_path->ID ) )
          $post_id = $page_by_path->ID;
      }
    } else {
      $post_id = url_to_postid( $current_url );

      if( $post_id == 0 && strpos( $current_url, '?' ) !== false ) {
        $current_url = strtok( $current_url, '?' );

        $post_id = url_to_postid( $current_url );
      }
    }

    $this->_url_to_post_id[ $current_url ] = $post_id;

    return $this->_url_to_post_id[ $current_url ];
  }

  /**
   * Get the current request URL, if HTTP_HOST or REQUEST_URI is missing, will return false.
   * @return bool|string
   */
  public function get_current_url() {
    if( !isset( $_SERVER['HTTP_HOST'] ) || !isset( $_SERVER['REQUEST_URI'] ) )
      return false;

    return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
  }

  /**
   * @param $key
   * @param $value
   */
  public function set_request_setting( $key, $value ) {
    $this->_current_request_setting[ $key ] = $value;
  }

  /**
   * @param $key
   * @return mixed|array|string|int|null
   */
  public function get_request_setting( $key ) {
    return ( isset( $this->_current_request_setting[ $key ] ) ? $this->_current_request_setting[ $key ] : null );
  }

}
