<?php

use WPEP\Entity\Discussion as WPEP_Discussion;

abstract class WPEP_Discussion_Integration {

  public function __construct() {
    wpep_controller()->discussion->register_integration( $this );
  }

  public function get_discussion_name( $discussion ) {
    return isset( $discussion->post_id ) && $discussion->post_id != 0 ? get_the_title( $discussion->post_id ) : $discussion->identifier;
  }

  public function get_discussion_link( $discussion, $user_id = false ) {
    if( isset( $discussion->post_id ) )
      return get_the_permalink( $discussion->post_id );

    return '#';
  }

  public function get_discussion_comment_highlight_link( $discussion, $comment_id, $user_id = false ) {
    $discussion_link = $this->get_discussion_link( $discussion, $user_id );
    $discussion_link .= ( strpos( $discussion_link, '#' ) === false ? '#' : '&' ) . 'comment-' . $comment_id;

    return $discussion_link;
  }

  public function get_discussion_icon_data( $discussion ) {
    if( !isset( $discussion->post_id ) )
      return [
        'type'  => 'image',
        'image' => wpep_get_placeholder_image()
      ];

    if( $discussion->post_id == 0 )
      return [
        'type'  => 'image',
        'image' => wpep_get_placeholder_image()
      ];


    $thumbnail_id  = get_post_thumbnail_id( $discussion->post_id );

    if( empty( $thumbnail_id ) )
      return [
        'type'  => 'image',
        'image' => wpep_get_placeholder_image()
      ];

    $thumbnail = get_post( $thumbnail_id, 'full' );

    if( isset( $thumbnail->guid ) )
      return [
        'type'  => 'image',
        'image' => $thumbnail->guid
      ];

    return [
      'type'  => 'image',
      'image' => wpep_get_placeholder_image()
    ];
  }

  public function pre_administration_discussions_query() {
    return [];
  }

  public function is_private_namespace() {
    return false;
  }

  public function get_namespace_title() {
    return wpep_alias_to_label( $this->get_namespace() );
  }

  abstract public function get_namespace();
  abstract public function current_user_can_view( $discussion_id );
  abstract public function current_user_can_post( $discussion_id );

  public function current_user_can_delete_comment( $discussion_comment ) {
    if( get_current_user_id() === 0 )
      return false;

    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return true;

    $discussion_comment = is_numeric( $discussion_comment ) ? WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => absint( $discussion_comment ) ] ) : $discussion_comment;

    if( $discussion_comment->user_id == get_current_user_id() )
      return true;

    return false;
  }

  public function current_user_can_pin_comment( $discussion_comment ) {
    if( get_current_user_id() === 0 )
      return false;

    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return true;

    return false;
  }

  public function get_email_notifications_status() {
    return false;
  }

  /**
   * @param $identifier_args
   * @param $post_id
   * @return int
   */
  public function discussion_id_by_identifier_args( $identifier_args, $post_id ) {
    $discussion_id = WPEP_Discussion::instance()->discussion_id_by_identifier( $this->get_discussion_identifier( $identifier_args ), $post_id, $this->get_namespace() );

    return $discussion_id;
  }

  public function get_discussion_identifier( $args ) {
    return 'post_comments_' . ( isset( $args['post_id'] ) ? $args['post_id'] : 0 );
  }

  /**
   * @param string $identifier
   * @return false|array
   */
  public function get_discussion_identifier_args_from_string( $identifier ) {
    if( strpos( $identifier, 'post_comments_' ) !== 0 )
      return false;

    return [
      'post_id' => intval( str_replace( 'post_comments_', '', $identifier ) )
    ];
  }

  public function has_comment_type( $type, $discussion_id = null ) {
    if( $type === '' )
      return true;

    $comment_types = $this->get_comment_types( $discussion_id );

    return isset( $comment_types[ $type ] );
  }

  public function get_comment_types( $discussion_id = null ) {
    return [];
  }

  public function get_comment_form_editor_args() {
    return [
      'media_upload'  => false,
      'has_mentions'  => wpep_get_setting_flag( 'comments-mentions-status' )
    ];
  }

  /**
   * @param bool $discussion_id
   * @param string $comments_structure
   * @return string
   */
  public function get_comments_template( $discussion_id = false, $comments_structure = 'threaded' ) {
    ob_start();

    $this->load_comments_template( $discussion_id, $comments_structure );

    $ret = ob_get_contents();

    ob_end_clean();

    return $ret;
  }

  /**
   * @param int $discussion_id
   * @param string $comments_structure
   */
  public function load_comments_template( $discussion_id, $comments_structure = 'threaded' ) {
    do_action( "wpep_discussion_comments_load_template_before", $this, $comments_structure );

    WPEP\Controller::instance()->template->load_template( 'discussion/template.php', [
      'discussion_namespace'     => $this->get_namespace(),
      'discussion_id'            => $discussion_id
    ] );

    do_action( "wpep_discussion_comments_load_template_after", $this, $comments_structure );
  }

  public function load_direct_url_template_content( $discussion ) {
    $this->load_comments_template( $discussion->id );
  }

  public function get_discussion_users( $discussion_id, $args = [] ) {
    $data = WPEP\Entity\Discussion::instance()->get_discussion_user_ids( $discussion_id, $args );

    return [
      'total'   => $data['total'],
      'entries' => ( empty( $data['ids'] ) ? [] : get_users( [ 'paged' => -1, 'include' => $data['ids'] ] ) )
    ];
  }

  public function user_get_discussions_args( $args ) {
    return $args;
  }

}